'''
Created on Mar 17, 2015

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import logging
import json
import os
import types
import app.Consts as APP_CONSTS

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


## CommonCrawlerDataStorage Class, implements functional of common crawler data storage
#
class CommonCrawlerDataStorage(object):

  JSON_EXTENSION = ".json"


  # #Class's constructor
  #
  def __init__(self):
    self.storeDict = {}


  # #Method saveElement saves incoming elements in the storage file that sets in storageDir and domain
  #
  # @param storageDir - root storage dir
  # @param domain - concretes files subputh
  # @param siteId - site's id, that concretes file name
  # @param element - data to save
  def saveElement(self, storageDir, domain, siteId, element):
    jsonStr = json.dumps(element, indent=4)
    if jsonStr is not None and jsonStr != "":
      if os.path.isdir(storageDir):
        localDir = storageDir
        if localDir[-1] != "/":
          localDir += "/"
        localDir += domain
        if not os.path.isdir(localDir):
          try:
            os.makedirs(localDir)
          except Exception:
            pass
        localFileName = localDir
        localFileName += "/"
        localFileName += str(siteId)
        localFileName += self.JSON_EXTENSION
        try:
          fd = open(localFileName, "w")
          fd.write(jsonStr)
          fd.close()
        except IOError:
          logger.debug(">>> CommonCrawlerDataStorage.saveElement can't open file to write, file=" + localFileName)
      else:
        logger.debug(">>> CommonCrawlerDataStorage.saveElement can't find root dir, dir=" + storageDir)


  # #Method loadHeaders reads headers data from storage file
  #
  # @param storageDir - root storage dir
  # @param domain - concretes files subputh
  # @param siteId - site's id, that concretes file name
  # @param externalElement - incoming headers that will mix with reading data
  # @param readFromFS - bool value - get element from internal cache or not
  # @return site storage element
  def loadElement(self, storageDir, host, siteId, externalElement=None, readFromFS=False):
    nodeElem = None
    if not readFromFS and host in self.storeDict and siteId in self.storeDict[host]:
      nodeElem = self.storeDict[host][siteId]
    if nodeElem is None:
      if os.path.isdir(storageDir):
        localDir = storageDir
        if localDir[-1] != "/":
          localDir += "/"
        localDir += host
        if os.path.isdir(localDir):
          localFileName = localDir
          localFileName += "/"
          localFileName += str(siteId)
          localFileName += self.JSON_EXTENSION
          try:
            fd = open(localFileName, "r")
            fileBuf = fd.read()
            if fileBuf is not None and fileBuf != "":
              nodeElem = json.loads(fileBuf)
            fd.close()
          except IOError:
            logger.debug(">>> CommonCrawlerDataStorage.loadElement can't open file to read, file=" + localFileName)
          except Exception as exp:
            logger.debug(">>> CommonCrawlerDataStorage.loadElement some exception, = " + str(exp))
        else:
          logger.debug(">>> CommonCrawlerDataStorage.loadElement can't find storage dir, dir=" + localDir)
      else:
        logger.debug(">>> CommonCrawlerDataStorage.loadElement can't find root dir, dir=" + storageDir)
    #save nodeElem in class storage hash
    if nodeElem is not None:
      if host not in self.storeDict:
        self.storeDict[host] = {}

      self.storeDict[host][siteId] = nodeElem
    if externalElement is not None:
      if nodeElem is None:
        nodeElem = {}
      for headerKey in externalElement:
        if headerKey not in nodeElem:
          nodeElem[headerKey] = {}
        for valueKey in externalElement[headerKey]:
          if valueKey not in nodeElem[headerKey]:
            nodeElem[headerKey][valueKey] = 0
    return nodeElem


  # #Method returns header name from fileStorageHeaders with least freq and which is present in the siteStorageHeaders
  #
  # @param fileStorageElements - incoming headers with file storage structure
  # @param siteStorageHeaders -
  # @return optimal header name
  def fetchLowFreqHeader(self, fileStorageElements, siteStorageElements=None):
    ret = None
    minValue = None
    if fileStorageElements is not None:
      for headerKey in fileStorageElements:
        for valueKey in fileStorageElements[headerKey]:
          if (minValue is None or minValue > fileStorageElements[headerKey][valueKey]) and \
          (siteStorageElements is None or \
          (headerKey in siteStorageElements and type(siteStorageElements[headerKey]) == types.ListType and \
          valueKey in siteStorageElements[headerKey])):
            minValue = fileStorageElements[headerKey][valueKey]
            ret = (headerKey, valueKey)

    return ret


  # #Method converts incoming jsonBuf to the siteStorageElements dict and return them
  #
  # @param jsonBuf - incoming json string with siteStorageElements structure
  # @return siteStorageElements element
  def extractSiteStorageElement(self, jsonBuf):
    ret = None
    try:
      ret = json.loads(jsonBuf)
    except Exception as exp:
      logger.debug(">>> CommonCrawlerDataStorage.extractSiteStorageElement can't load data from incoming jsonBuf " +
                   "(may be not json format...) exception=" + str(exp))
    return ret

