'''
Created on Sep 11, 2015

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''


import hashlib
import logging
import re
import types
import app.Consts as APP_CONSTS

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class ContentHashCalculator(object):

  ALGO_INCOME_BUF = 1
  ALGO_SIMPLE_SPLITTING = 2
  ALGO_SNOWBALL_SPLITTING = 3
  ALGO_SOUNDEX_SPLITTING = 4

  RE_SPLITTER = '\s'


  @staticmethod
  def langDetect(incomeBuf):
    ret = None
    try:
      from langdetect import detect
      import pycountry
      langSmallName = detect(incomeBuf).split('-')[0]
      ret = pycountry.languages.get(iso639_1_code=langSmallName).name.lower()
    except Exception as ecxp:
      logger.debug(">>> Some snowball exception = " + str(ecxp))
    return ret


  @staticmethod
  def commonSplitMethod(incomeBuf, minWLen):
    ret = re.split(ContentHashCalculator.RE_SPLITTER, incomeBuf, flags=re.LOCALE)
    if len(ret) > 0:
      ret = list(set(ret))
      ret.sort()
      ret = [x for x in ret if len(x) >= minWLen]
    return ret


  @staticmethod
  def hashCalculateSimple(incomeBuf, minWLen):
    ret = 0
    splittedList = ContentHashCalculator.commonSplitMethod(incomeBuf, minWLen)
    if len(splittedList) > 0:
      ret = hashlib.md5(''.join(splittedList)).hexdigest()
    return ret


  @staticmethod
  def hashCalculateSnowball(incomeBuf, minWLen, additionData, stemmerLang):# pylint: disable=W0613
    ret = 0
    try:
      import snowballstemmer
      if stemmerLang is None:
        stemmerLang = ContentHashCalculator.langDetect(incomeBuf)
        if stemmerLang is None:
          stemmerLang = 'english'

      #if additionData is not None and type(additionData) in types.StringTypes:
      splittedList = ContentHashCalculator.commonSplitMethod(incomeBuf, minWLen)
      if len(splittedList) > 0:
        stemmer = snowballstemmer.stemmer(stemmerLang)
        for i in xrange(0, len(splittedList)):
          try:
            splittedList[i] = stemmer.stemWord(splittedList)
          except Exception as ecxp:
            splittedList[i] = ""
        ret = hashlib.md5(''.join(splittedList)).hexdigest()
    except Exception as ecxp:
      logger.debug(">>> Some snowball exception = " + str(ecxp))
    return ret


  @staticmethod
  def hashCalculateSoundex(incomeBuf, minWLen):
    ret = 0
    try:
      import soundex
      splittedList = ContentHashCalculator.commonSplitMethod(incomeBuf, minWLen)
      if len(splittedList) > 0:
        s = soundex.getInstance()
        for i in xrange(0, len(splittedList)):
          try:
            splittedList[i] = s.soundex(splittedList[i])
          except Exception as ecxp:
            splittedList[i] = ""
        ret = hashlib.md5(''.join(splittedList)).hexdigest()
    except Exception as ecxp:
      logger.debug(">>> Some soundex exception = " + str(ecxp))
    return ret


  @staticmethod
  def hashCalculate(incomeBuf, algo, minWLen=3, additionData=None, stemmerLang=None):
    ret = 0
    incomeBuf = incomeBuf.lower()
    if algo == ContentHashCalculator.ALGO_INCOME_BUF:
      ret = hashlib.md5(incomeBuf).hexdigest()
    elif algo == ContentHashCalculator.ALGO_SIMPLE_SPLITTING:
      ret = ContentHashCalculator.hashCalculateSimple(incomeBuf, minWLen)
    elif algo == ContentHashCalculator.ALGO_SNOWBALL_SPLITTING:
      ret = ContentHashCalculator.hashCalculateSnowball(incomeBuf, minWLen, additionData, stemmerLang)
    elif algo == ContentHashCalculator.ALGO_SOUNDEX_SPLITTING:
      ret = ContentHashCalculator.hashCalculateSoundex(incomeBuf, minWLen)
    return ret

  