'''
Created on Mar 28, 2014

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import types
import logging
import app.Consts as APP_CONSTS
from SelectorWrapper import SelectorWrapper
import copy

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

class ExtendInnerText(object):


  def __init__(self, tagReplacers=None, delimiter=' ', innerDelimiter=' '):
    self.stripHtml = ''
    self.errorString = ''
    self.delimiter = delimiter
    self.innerDelimiter = innerDelimiter
    self.tagReplacers = tagReplacers if tagReplacers is not None else {}


  def nodeCallbackOpenHandler(self, nodeElem, level):  # pylint: disable=W0613
    openTagName = str(nodeElem.xpath("name()")[0].extract())
    if openTagName in self.tagReplacers:
      self.stripHtml += self.tagReplacers[openTagName]


  def nodeCallbackCloseHandler(self, nodeElem, level):  # pylint: disable=W0613
    closeTagName = "/" + str(nodeElem.xpath("name()")[0].extract())
    closeTag = "<" + closeTagName + ">"
    if (len(str(nodeElem.extract())) >= len(closeTag)) and \
    str(nodeElem.extract()).rfind(closeTag) == (len(str(nodeElem.extract())) - len(closeTag)):
      if closeTagName in self.tagReplacers:
        self.stripHtml += self.tagReplacers[closeTagName]


  def textCallbackHandler(self, nodeElem, level):  # pylint: disable=W0613
    if str(nodeElem.extract()).strip() != "":
      self.stripHtml += str(nodeElem.extract()) + self.innerDelimiter


  def innerText(self, contentBuf, xPath, tagRemoves=None):
    self.stripHtml = ''
    self.errorString = ''
    if xPath is not None:
      if tagRemoves is None:
        tagRemoves = ['script', '']
      try:
        if type(xPath) in types.StringTypes:
          sel = SelectorWrapper(text=contentBuf)
          selectorElem = sel.xpath(xPath)
        else:
          selectorElem = xPath
        localBuf = ''
        for elem in selectorElem:
          self.stripHtml = ''
          elemList = []
          elemList.append(elem)
          ExtendInnerText.traversalNodes(elemList, 0, self.nodeCallbackOpenHandler, self.nodeCallbackCloseHandler,
                                         self.textCallbackHandler, tagRemoves)
          localBuf += self.stripHtml.strip(self.innerDelimiter)
          localBuf += self.delimiter
        self.stripHtml = localBuf.strip(self.delimiter)
      except Exception as excp:
        self.errorString = str(excp)


  @staticmethod
  def traversalNodes(elemList, level=0, nodeCallbackOpen=None, nodeCallbackClose=None, textCallback=None,
                   excludeTags=None):
    if excludeTags is None:
      excludeTags = ['script', '']
    for elem in elemList:
      if len(elem.xpath("name()")) > 0:
        if nodeCallbackOpen is not None:
          nodeCallbackOpen(elem, level)
        if str(elem.xpath("name()")[0].extract()) not in excludeTags:
          ExtendInnerText.traversalNodes(elem.xpath("node()"), level + 1, nodeCallbackOpen, nodeCallbackClose,
                                         textCallback, excludeTags)
        if nodeCallbackClose is not None:
          nodeCallbackClose(elem, level)
      else:
        if textCallback is not None:
          textCallback(elem, level)


  def innerTextToList(self, contentBuf, xPath, tagRemoves=None):
    stripHtmlList = []
    self.stripHtml = ''
    self.errorString = ''
    if xPath is not None:
      if tagRemoves is None:
        tagRemoves = ['script', '']
      try:
        if type(xPath) in types.StringTypes:
          sel = SelectorWrapper(text=contentBuf)
          selectorElem = sel.xpath(xPath)
        else:
          selectorElem = xPath

        for elem in selectorElem:
          self.stripHtml = ''
          elemList = []
          elemList.append(elem)
          ExtendInnerText.traversalNodes(elemList, 0, self.nodeCallbackOpenHandler,
                                         self.nodeCallbackCloseHandler,
                                         self.textCallbackHandler, tagRemoves)

          stripHtmlList.append(self.stripHtml.strip(self.innerDelimiter) + self.delimiter)

        self.stripHtml = stripHtmlList
      except Exception as excp:
        self.errorString = str(excp)
