'''
Created on Mar 17, 2015

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import logging
import json
import os
import types
import app.Consts as APP_CONSTS


logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class HTTPHeadersStorage(object):

  JSON_EXTENSION = ".json"

  def __init__(self):
    self.storeDict = {}


  # #Method saveHeaders saves incoming headers in the storage file that sets in storageDir and domain
  #
  # @param storageDir - root storage dir
  # @param domain - concretes files subputh
  # @param siteId - site's id, that concretes file name
  # @param headersDict - data to save
  def saveHeaders(self, storageDir, domain, siteId, headersDict):
    jsonStr = json.dumps(headersDict, indent=4)
    if jsonStr is not None and jsonStr != "":
      if os.path.isdir(storageDir):
        localDir = storageDir
        if localDir[-1] != "/":
          localDir += "/"
        localDir += domain
        if not os.path.isdir(localDir):
          try:
            os.makedirs(localDir)
          except Exception:
            pass
        localFileName = localDir
        localFileName += "/"
        localFileName += str(siteId)
        localFileName += self.JSON_EXTENSION
        try:
          fd = open(localFileName, "w")
          fd.write(jsonStr)
          fd.close()
        except IOError:
          logger.debug(">>> HTTPHeadersStorage.saveHeaders can't open file to write, file=" + localFileName)
      else:
        logger.debug(">>> HTTPHeadersStorage.saveHeaders can't find root dir, dir=" + storageDir)


  # #Method loadHeaders reads headers data from storage file
  #
  # @param storageDir - root storage dir
  # @param domain - concretes files subputh
  # @param siteId - site's id, that concretes file name
  # @param externalHeaders - incoming headers that will mix with reading data
  # @param readFromFS - bool value - get element from internal cache or not
  # @return site storage element
  def loadHeaders(self, storageDir, host, siteId, externalHeaders=None, readFromFS=False):
    nodeElem = None
    if not readFromFS and host in self.storeDict and siteId in self.storeDict[host]:
      nodeElem = self.storeDict[host][siteId]
    if nodeElem is None:
      if os.path.isdir(storageDir):
        localDir = storageDir
        if localDir[-1] != "/":
          localDir += "/"
        localDir += host
        if os.path.isdir(localDir):
          localFileName = localDir
          localFileName += "/"
          localFileName += str(siteId)
          localFileName += self.JSON_EXTENSION
          try:
            fd = open(localFileName, "r")
            fileBuf = fd.read()
            if fileBuf is not None and fileBuf != "":
              nodeElem = json.loads(fileBuf)
            fd.close()
          except IOError:
            logger.debug(">>> HTTPHeadersStorage.saveHeaders can't open file to read, file=" + localFileName)
          except Exception as exp:
            logger.debug(">>> HTTPHeadersStorage.saveHeaders some exception, = " + str(exp))
        else:
          logger.debug(">>> HTTPHeadersStorage.saveHeaders can't find storage dir, dir=" + localDir)
      else:
        logger.debug(">>> HTTPHeadersStorage.saveHeaders can't find root dir, dir=" + storageDir)
    #save nodeElem in class storage hash
    if nodeElem is not None:
      if host not in self.storeDict:
        self.storeDict[host] = {}

      self.storeDict[host][siteId] = nodeElem
    if externalHeaders is not None:
      if nodeElem is None:
        nodeElem = {}
      for headerKey in externalHeaders:
        if headerKey not in nodeElem:
          nodeElem[headerKey] = {}
        for valueKey in externalHeaders[headerKey]:
          if valueKey not in nodeElem[headerKey]:
            nodeElem[headerKey][valueKey] = 0
    return nodeElem


  # #Method returns header name from fileStorageHeaders with least freq and which is present in the siteStorageHeaders
  #
  # @param fileStorageHeaders - incoming headers with file storage structure
  # @param siteStorageHeaders -
  # @return optimal header name
  def fetchLowFreqHeader(self, fileStorageHeaders, siteStorageHeaders):
    ret = None
    minValue = None
    if fileStorageHeaders is not None and siteStorageHeaders is not None:
      for headerKey in fileStorageHeaders:
        for valueKey in fileStorageHeaders[headerKey]:
          if (minValue is None or minValue > fileStorageHeaders[headerKey][valueKey]) and \
          (headerKey in siteStorageHeaders and type(siteStorageHeaders[headerKey]) == types.ListType and \
          valueKey in siteStorageHeaders[headerKey]):
            minValue = fileStorageHeaders[headerKey][valueKey]
            ret = (headerKey, valueKey)

    return ret


  # #Method converts incoming jsonBuf to the siteStorageHeaders dict and return them
  #
  # @param jsonBuf - incoming json string with siteStorageHeaders structure
  # @return siteStorageHeaders element
  def extractSiteStorageHeader(self, jsonBuf):
    ret = None
    try:
      ret = json.loads(jsonBuf)
    except Exception as exp:
      logger.debug(">>> HTTPHeadersStorage.extractSiteStorageHeader can't load data from incoming jsonBuf " +
                   "(may be not json format...) exception=" + str(exp))
    return ret

