'''
Created on Mar 28, 2014

@package: app
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''


import json
import re
import sys
import os
import traceback
import urlparse
import Exceptions
import types
import time
import urllib
#from scrapy.utils.url import canonicalize_url
from datetime import datetime
from decimal import Decimal
import logging
import app.Consts as APP_CONSTS
import Url
import threading
from HTMLParser import HTMLParser
from ExtendInnerText import ExtendInnerText


try:
  import cPickle as pickle
except ImportError:
  import pickle


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
lock = threading.Lock()


META_RE_0 = "<meta (.*?)>"
META_RE_1 = "http-equiv=\"refresh\""
META_RE_2 = "url=(.*?)\""
# #PropertiesValidator contains only 1 method - isValueIn, that find value in class(classType param)
# attributes that began with "prefix"
#
class PropertiesValidator(object):


  def __init__(self):
    pass


  @staticmethod
  def isValueIn(classType, prefix, value):
    retVal = False
    for localValue in classType.__dict__:
      if str(localValue).find(prefix) == 0:
        if value == getattr(classType, localValue, None):
          retVal = True
          break
    return retVal


# #getPath global function, finds and return value by path in json string or dict document
# dictionary - incoming dictionary (optional)
# jsonString - incoming json string (optional)
# path - incoming path to find
# method return valid value or raises exceptions -
# [ValueError] - bad jsodnString format
# [TypeError, KeyError, IndexError] - excpetions raised if path not found
# Warning!!! path don't checks by syntaxis
def getPath(dictionary, jsonString, path):
  if jsonString != None:
    dictionary = json.loads(jsonString)
  for i, p in re.findall(r'(\d+)|(\w+)', path):
    dictionary = dictionary[p or int(i)]
  return dictionary



# #Json serialization
#
class JsonSerializable(object):

  # #constructor
  # initialize task's fields
  #
  def __init__(self):
    pass

  @staticmethod
  def json_serial(obj):
    if isinstance(obj, datetime):
      return obj.isoformat()
    else:
      if isinstance(obj, Decimal):
        return str(obj)
      else:
        #if isinstance(obj, type.DictProxy):
        if isinstance(obj, types.DictProxyType):
          return dict(obj)
        else:
          return obj.__dict__


  def toJSON(self):
    # return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    # Support custom serialization of datetime
    return json.dumps(self.__dict__, default=JsonSerializable.json_serial, sort_keys=True, indent=4)



class SQLExpression(object):
  def __init__(self, stringExpression=""):
    super(SQLExpression, self).__init__()
    if stringExpression is None:
      self.str = ""
    else:
      self.str = str(stringExpression)

  def __str__(self):
    return self.str



# #The PathMaker class
#
# This class purpose to use as static method container to split FS directory pattern on two parts
# @param string is a directory pattern
# @param subdirLen is a number of characters in sub-directory item
#
class PathMaker(object):

  SUBDIR_LEVEL1_LEN = 2
  SUBDIR_CHAR = "/"

  def __init__(self, string, subdirLen=SUBDIR_LEVEL1_LEN):
    super(PathMaker, self).__init__()

    self.string = string
    self.subdirLen = subdirLen

    if "CONTENT_STORE_PATH" in os.environ and os.environ["CONTENT_STORE_PATH"] != "":
      logger.debug("os.environ[CONTENT_STORE_PATH]: set to %s", os.environ["CONTENT_STORE_PATH"])
      self.string += "/"
      self.string += os.environ["CONTENT_STORE_PATH"]
    else:
      logger.debug("os.environ[CONTENT_STORE_PATH]: not set.")

  def getDir(self):
    if len(self.string) > self.subdirLen:
      return self.string[:self.subdirLen] + self.SUBDIR_CHAR + self.string[self.subdirLen:]
    else:
      return self.string


# #The ConfigParamsList class
#
# This class purpose to use as object few options read from config
# @param initial_data - input dictionaries
# @param kwargs - keyword arguments
#
class ConfigParamsList(object):
  def __init__(self, *initial_data, **kwargs):
    for dictionary in initial_data:
      for key in dictionary:
        setattr(self, key, dictionary[key])
    for key in kwargs:
      setattr(self, key, kwargs[key])


# #The function to safe getting config parameters and  return default value if cannot get value
#
# @param parser - instance of ConfigParser class
# @param section  - section name of parameter
# @param option - option name of parameter
# @param defValue - default value
# @return - if success extracted value, otherwise return default value
#
def getConfigParameter(parser, section, option, defValue):
  ret = defValue

  if parser and parser.has_option(section, option):
    try:
      ret = parser.get(section, option, defValue)
    except Exception:
      ret = defValue

  return ret


# #The function to get traceback information string prepared for logging
#
# This function collects traceback information and creates sreing representation ready to log it
# @param linesNumberMax max number of traceback lines to include in to the collection, None - signs all
# @ret return string
#
def getTracebackInfo(linesNumberMax=None):
  ret = ""
  n = 0

  type_, value_, traceback_ = sys.exc_info()
  stack = traceback.format_tb(traceback_)
  del type_
  del value_
  for item in stack:
    ret = ret + "\n" + (str(item))
    n = n + 1
    if linesNumberMax != None and n == linesNumberMax:
      break

  return ret



# #The function to get accumulate the traceback information in global variable __tracebackList
tracebackList = []
tracebackTimeQueue = []
tracebackIdent = False
tracebackIdentFiller = "-"
tracebackMessageCall = "call"
tracebackMessageExit = "exit"
tracebackmessageDelimiter = ":"
tracebackTimeMark = True
tracebackTimeMarkFormat = "%Y-%m-%d %H:%M:%S.%f"
tracebackTimeMarkDelimiter = " "
tracebackIncludeInternalCalls = False
tracebackIncludeLineNumber = True
tracebackIncludeLineNumberDelimiter = ":"
tracebackIncludeFileNumber = True
tracebackIncludeFileNumberDelimiter = ":"
tracebackFunctionNameDelimiter = ":"
tracebackExcludeModulePath = ["/usr/lib/", "/usr/local/lib/"]
tracebackExcludeFunctionName = ["varDump"]
tracebackExcludeFunctionNameStarts = ["<"]
tracebackIncludeExitCalls = True
tracebackRecursionlimit = 0
tracebackRecursionlimitErrorMsg = "RECURSION STACK LIMIT REACHED "
tracebackIncludeLocals = False
tracebackIncludeArg = False
tracebackIncludeLocalsPrefix = "\nLOCALS:\n"
tracebackIncludeArgPrefix = "\nARG:\n"
tracebackLogger = None
tracebackElapsedTimeDelimiter = ""
tracebackElapsedTimeFormat = "{:.6f}"
tracebackUnknownExceptionMsg = "Unknown exception!"

#
# This function collects traceback information and creates sreing representation ready to log it
# @param linesNumberMax max number of traceback lines to include in to the collection, None - signs all
# @ret return string
#
def tracefunc(frame, event, arg, indent=None):  # pylint: disable=W0613
  if indent is None:
    indent = [0]

  if event == "call" or event == "return":
    lock.acquire()

    try:
      if event == "call":
        indent[0] += 2
        if tracebackIdent:
          idents = tracebackIdentFiller * indent[0]
        else:
          idents = ""
        message = tracebackMessageCall
        tracebackTimeQueue.append(time.time())
        te = ""
      elif event == "return":
        if tracebackIdent:
          idents = tracebackIdentFiller * indent[0]
        else:
          idents = ""
        indent[0] -= 2
        message = tracebackMessageExit
        te = "{:.6f}".format(time.time() - tracebackTimeQueue.pop())

      if tracebackTimeMark:
        #t = time.strftime(tracebackTimeMarkFormat)
        t = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
      else:
        t = ""

      if tracebackIncludeLineNumber:
        ln = str(frame.f_lineno)
      else:
        ln = ""

      if tracebackIncludeFileNumber:
        fn = str(frame.f_code.co_filename)
      else:
        fn = ""

      excludedP = False
      for item in tracebackExcludeModulePath:
        if item in frame.f_code.co_filename:
          excludedP = True
          break

      excludedF = False
      for item in tracebackExcludeFunctionName:
        if frame.f_code.co_name == item:
          excludedF = True
          break

      excludedF2 = False
      for item in tracebackExcludeFunctionNameStarts:
        if frame.f_code.co_name.startswith(item):
          excludedF2 = True
          break

      if tracebackIncludeLocals or tracebackIncludeArg:
        oldRL = sys.getrecursionlimit()
        if oldRL < tracebackRecursionlimit:
          sys.setrecursionlimit(tracebackRecursionlimit)
        else:
          oldRL = None

        if tracebackIncludeLocals:
          localsDump = ""
          try:
            #localsDump = varDump(frame.f_locals)
            localsDump = str(frame.f_locals)
            localsDump = tracebackIncludeLocalsPrefix + localsDump
          except:  # pylint:disable=W0702
            localsDump = tracebackRecursionlimitErrorMsg + str(tracebackRecursionlimit)
            #pass
        else:
          localsDump = ""

        if tracebackIncludeArg:
          argDump = ""
          try:
            #argDump = varDump(arg)
            argDump = str(arg)
            argDump = tracebackIncludeArgPrefix + argDump
          except:  # pylint:disable=W0702
            argDump = tracebackRecursionlimitErrorMsg + str(tracebackRecursionlimit)
            #pass
        else:
          argDump = ""

        if oldRL is not None:
          sys.setrecursionlimit(oldRL)
      else:
        localsDump = ""
        argDump = ""

      if (not (tracebackIncludeInternalCalls == False and frame.f_code.co_name.startswith("__"))) and\
        (not (tracebackIncludeExitCalls == False and event == "return")) and\
        (not excludedP) and (not excludedF) and (not excludedF2):
        tmsg = idents + message + tracebackmessageDelimiter + \
                             fn + tracebackIncludeFileNumberDelimiter + \
                             ln + tracebackIncludeLineNumberDelimiter + \
                             frame.f_code.co_name + "()" + tracebackFunctionNameDelimiter + \
                             tracebackElapsedTimeDelimiter + te + localsDump + argDump
        if tracebackLogger is None:
          tracebackList.append(t + tracebackTimeMarkDelimiter + tmsg)
        else:
          tracebackLogger.debug("%s", tmsg)
          if len(tracebackTimeQueue) == 0:
            tracebackLogger.debug("%s", APP_CONSTS.LOGGER_DELIMITER_LINE)

    except Exception as e:
      if tracebackLogger is None:
        tracebackList.append("Exception: " + str(e))
      else:
        tracebackLogger.error("%s", str(e))
    except:  # pylint: disable=W0702
      if tracebackLogger is None:
        tracebackList.append(tracebackUnknownExceptionMsg)
      else:
        tracebackLogger.error("%s", tracebackUnknownExceptionMsg)

    lock.release()

  return tracefunc



# #The function to get a printable representation of an object for debugging
#
#
# @param obj The object to print
# @ret return string dump
#
def varDump(obj, stringify=True):
  newobj = obj

  try:
    if isinstance(obj, list):
      newobj = []
      for item in obj:
        newobj.append(varDump(item, False))
    elif isinstance(obj, tuple):
      temp = []
      for item in obj:
        temp.append(varDump(item, False))
      newobj = tuple(temp)
    elif isinstance(obj, set):
      temp = []
      for item in obj:
        temp.append(str(varDump(item, False)))
      newobj = set(temp)
    elif isinstance(obj, dict):
      newobj = {}
      for key, value in obj.items():
        newobj[str(varDump(key, False))] = varDump(value, False)
    # elif isinstance(obj, types.FunctionType):
    #  newobj = repr(obj)
    elif '__dict__' in dir(obj):
      newobj = obj.__dict__.copy()
      if ' object at ' in str(obj) and not '__type__' in newobj:
        newobj['__type__'] = str(obj).replace(" object at ", " #").replace("__main__.", "")
      for attr in newobj:
        newobj[attr] = varDump(newobj[attr], False)
  except Exception:
    pass

  if stringify:
    return str(newobj)
  else:
    return newobj


 # pylint: disable=W0702
def memUsage(point=""):
  import resource
  #usage = resource.getrusage(resource.RUSAGE_SELF)
  return '''%s: mem=%s mb
      ''' % (point, resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1000)



# #class UrlParser makes URL operation
#
class UrlParser(object):


  def __init__(self):
    pass


  # #The function check protocol and dimain name in the incoming url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def isValidURL(url):
    ret = False
    parseUrl = urlparse.urlparse(url)
    if parseUrl.scheme != None and parseUrl.netloc != None:
      ret = True
    return ret


  # #The function generates base url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def generateDomainUrl(url):
    ret = ""
    parseUrl = urlparse.urlparse(url)
    if UrlParser.isValidURL(url):
      ret = parseUrl.scheme + "://" + parseUrl.netloc
    else:
      raise Exceptions.UrlParseException("Empty protocol or domain name")
    return ret


  # #The function extracts domain name
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def getDomain(url):
    auth = urlparse.urlsplit(url.strip())[1]
    ret = (re.search('([^@]*@)?([^:]*):?(.*)', auth).groups())[1]
    return ret



# #class UrlNormalizator makes URL normalization
#
class UrlNormalizator(object):

  OWN_ALGO = 0
  T100_ALGO = 1

  BAD_URL_PREFIX = "normalization-error://?"


  def __init__(self):
    pass


  # #The function normalize the incoming url according  RFC 3986
  #
  # url - param, incoming url
  # @ret normalized url
  @staticmethod
  def normalize(url, supportProtocols=None):
    norm_url = url.strip()
    if supportProtocols is not None and type(supportProtocols) is types.ListType:
      colonPos = norm_url.find(':')
      slashPos = norm_url.find('/')
      if colonPos != -1 and (slashPos == -1 or slashPos > colonPos):
        if len(norm_url.split(':')) > 1:
          protocol = norm_url.split(':')[0]
          if protocol not in supportProtocols:
            try:
              norm_url = UrlNormalizator.BAD_URL_PREFIX + urllib.quote(norm_url)
            except Exception as err:
              logger.error(">>> urllib.quote error = " + str(err))
              norm_url = UrlNormalizator.BAD_URL_PREFIX + norm_url

    if norm_url == url:
      try:
        # logger.debug("input url: <%s>", url)
        # url.strip() removes trailing special characters like '\n'
        # without it urls inserted with '\n' symbol at the end
        # as result will be crawled only sites ended by '/' like 'html://www.foo.com/\n'
        # however full urls won't crawled like 'html:/www.foo.com/file.html\n'
        # norm_url = str(urlnorm.norm(url.strip()))
        # logger.debug("AAAAA: <%s>", norm_url)
        norm_url = str(urinormpath(url.strip()))
        #norm_url = str(canonicalize_url(url.strip()))
        # logger.debug("norm_url: <%s>", norm_url)
        # except urlnorm.InvalidUrl:
        #  logger.error("Normalization InvalidUrl")
        #  norm_url = ""
      except Exception as e:
        logger.error("Normalization error: " + str(e) + "\nURL: [" + url + "]\n" + str(getTracebackInfo()))

    return norm_url


  # #The check url by bad url prefix
  #
  # url - param, incoming url
  # @ret bool value is url valid or not
  @staticmethod
  def isNormalUrl(url):
    return False if url.find(UrlNormalizator.BAD_URL_PREFIX) == 0 else True


  ##The function encode entities "&" to "&amp;" if needed
  #
  #@param url to encode
  #@param entities dict, keys are not encode and values are encoded forms
  #@return encoded url
  @staticmethod
  def entitiesEncode(url, entities=None):
    ret = url
    if entities is None:
      entities = {"&": "&amp;"}

    for k in entities:
      le = len(entities[k])
      p = -1
      while True:
        l = len(ret)
        p = ret.find(k, p + 1)
        if p == -1:
          break
        else:
          if (p + le - 1 > l) or ((p + le - 1 <= l) and (ret[p:p + le] != entities[k])):
            ret = ret[:p] + entities[k] + ret[p + 1:]
          else:
            continue

    return ret



# ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
# # storePickleOnDisk
#
def storePickleOnDisk(input_pickled_object, env_path, file_name):
  if env_path in os.environ and os.environ[env_path] != "":
    logger.debug("os.environ[%s]: set to %s", env_path, os.environ[env_path])
    open(os.environ[env_path] + file_name, "wb").write(input_pickled_object)
  else:
    logger.debug("os.environ[%s]: not set.", env_path)



# This function taken from uritools module as it was removed from module
def urinormpath(path, algorithm=UrlNormalizator.T100_ALGO, stripWWW=False, useValidator=False):  # pylint: disable=W0613
  #Remove '.' and '..' path segments from a URI path.
  # RFC 3986 5.2.4. Remove Dot Segments
  ret = None
  ret1 = None

  try:
    if path is None or path == "":
      ret1 = path
    else:
      out = []
      for s in path.split('/'):
        if s == '.':
          continue
        elif s != '..':
          out.append(s)
        elif out:
          out.pop()
      # Fix leading/trailing slashes
      if path.startswith('/') and (not out or out[0]):
        out.insert(0, '')
      if path.endswith('/.') or path.endswith('/..'):
        out.append('')
      ret = '/'.join(out)

      if stripWWW:
        splitPath = path.split("?")
        if len(splitPath) > 0:
          splitPath[0] = splitPath[0].replace("://www.", "://")
          localPath = splitPath[0]
          for elem in splitPath[1:]:
            localPath += "?"
            localPath += elem
      else:
        localPath = path
      resultUrlDict = Url.Url(localPath)
      if useValidator and not Url.Url.GetStats([resultUrlDict])[0]["valid"]:
        raise Exception(path + " NOT VALIDATE!")
      ret1 = Url.Url.GetStats([resultUrlDict])[0]["canonicalized"]

      if ret is not None and ret1 is not None and ret != ret1:
        logger.debug("--->>>> URLS DIFFERTNT <<<<---")
        logger.debug(ret)
        logger.debug(ret1)
  except Exception as e:
    logger.error("Normalization error: " + str(e) + "\npath: [" + path + "]\n" + str(getTracebackInfo()))


  return ret1



# #Logger file name generator
#
#
class LoggerFileName(object):


  def __init__(self, loggerInst=None):
    self.loggerInst = loggerInst


  def getFreeProcInstanceNumber(self, sock_prefix="module", min_number=0, max_number=32):
    ret = ""

    for i in range(min_number, max_number):
      try:
        import socket
        global lock_socket  # pylint: disable=W0601
        lock_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        # Create an abstract socket
        lock_socket.bind('\0' + "dc_process_lock_" + sock_prefix + str(i))
        ret = str(i)
        break
      except socket.error:
        #Socket already exists
        continue

    return ret


  def findReplace(self, newFile=None):
    log_file = None

    if self.loggerInst is None:
      lg = logging.getLogger('')
    else:
      lg = self.loggerInst
    for h in lg.__dict__['handlers']:
      if h.__class__.__name__ == 'FileHandler':
        log_file = h.baseFilename
        if newFile is not None:
          h.baseFilename = newFile
          if h.stream:
            h.stream.close()
            h.stream = None
          h.stream = h._open()  # pylint: disable=W0212
          h.doRollover()
        break
      elif h.__class__.__name__ == 'TimedRotatingFileHandler':
        log_file = h.baseFilename
        if newFile is not None:
          h.baseFilename = newFile
          if h.stream:
            h.stream.close()
            h.stream = None
          h.stream = h._open()  # pylint: disable=W0212
          h.doRollover()
        break

    return log_file



# #accumulateSubstrings accumulates substr list in one string and returns it, also adds prefixies between
#  substrings in resulting string. substrList and prefixes must be List[str] type with equal length
# @param substrList - substrings list
# @param prefixes - prefixies list
# @returns - accumulate string
def accumulateSubstrings(substrList, prefixes):
  ret = ""
  if substrList is None or type(substrList) is not types.ListType:
    raise Exception(">>> error substrList is None or not List type")
  if prefixes is None or type(prefixes) is not types.ListType:
    raise Exception(">>> error prefixes is None or not List type")
  if len(substrList) != len(prefixes):
    raise Exception(">>> error substrList and prefixes lists have different lengths")
  i = 0
  for substr in substrList:
    if type(substr) in types.StringTypes:
      if type(prefixes[i]) in types.StringTypes:
        ret += str(prefixes[i])
      ret += str(substr)
    i += 1
  return ret



class DataReplacementConstants(object):

  CUR_YEAR_FULL = "@CUR_YEAR_FULL"
  CUR_YEAR_SHORT = "@CUR_YEAR_SHORT"
  CUR_MONTH = "@CUR_MONTH"
  CUR_DAY = "@CUR_DAY"


# #generateReplacementDict method generates and returns replacement dict with current datatime
# @returns - replacement dict with current datatime
def generateReplacementDict():
  ret = {}
  ret[DataReplacementConstants.CUR_YEAR_FULL] = datetime.now().strftime("%Y")
  ret[DataReplacementConstants.CUR_YEAR_SHORT] = datetime.now().strftime("%y")
  ret[DataReplacementConstants.CUR_MONTH] = datetime.now().strftime("%m")
  ret[DataReplacementConstants.CUR_DAY] = datetime.now().strftime("%d")
  return ret


# #parseHost parse the root host name from url
# for example: the result of http://s1.y1.example.com/path/to is example.com
# @param url the full url
# @return host of the url, eg: example.com
def parseHost(url):
  host = None
  if urlparse.urlparse(url).hostname:
    host = '.'.join(urlparse.urlparse(url).hostname.split('.')[-2:])
  return host


# # convert date str to HTTP header format
# 2014-07-29 20:31:50 (GMT+8) to Tue, 29 Jul 2014 12:31:50 GMT
# @param date_str date str, 2014-07-29 20:31:50
# @return HTTP header formated date str : Tue, 29 Jul 2014 12:31:50 GMT
def convertToHttpDateFmt(date_str):
  stamp = time.mktime(date_str.timetuple())
  # stamp = time.mktime(time.strptime(date_str, '%Y-%m-%d %H:%M:%S'))
  return time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime(stamp))


# # method returns siteId, substitutes to "0" value if incoming siteId is None
#@param siteId - ID of site
#@param logger - logger instanse for log usage
def autoFillSiteId(siteId, logger):
  ret = siteId
  if siteId is None:
    ret = "0"
    if logger is not None:
      logger.debug("set siteId = '0' from 'autoFillSiteId'")

  return ret


# # method strips incoming html from html comments
# @param htmlBuf incoming content in string format
# @param soup incoming content as bs object
# @return clean html buff
def stripHTMLComments(htmlBuf=None, soup=None):
  from bs4 import Comment

  ret = htmlBuf
  if soup is not None:
    for elem in soup.findAll(text=lambda text: isinstance(text, Comment)):
      elem.extract()
  elif htmlBuf is not None:
    ret = re.sub(r"\<![ \r\n\t]*(--([^\-]|[\r\n]|-[^\-])*--[ \r\n\t]*)\>", "", htmlBuf)
  return ret


# Strips from all HTML tags with set of different methods
# @param htmlTxt input content
# @param method 0 - by BeautifulSoup, 1 - with RE 1, 2 - RE 2, 3 - HTML parser, 4 - clear Python w/o lib, 5 - xml lib
# @param joinGlue - the glue string to joing parts
# @param regExp - the custom re for the method 1 or 2
# @return cleared content
def stripHTMLTags(htmlTxt, method=0, joinGlue=' ', regExp=None):
  ret = ''

  if htmlTxt is not None and htmlTxt.strip() != '':
    if method == 0:
      from bs4 import BeautifulSoup
      ret = joinGlue.join(BeautifulSoup(htmlTxt).findAll(text=True))
    elif method == 1 or method == 2:
      if regExp is not None:
        r = regExp
      else:
        if method == 1:
          r = r'<[^<]+?>'
        else:
          r = r'(<!--.*?-->|<[^>]*>)'
      ret = re.sub(r, joinGlue, htmlTxt)
    elif method == 3:
      ret = MLStripper()
      ret.feed(htmlTxt)
      ret = ret.get_data()
    elif method == 4:
      tag = False
      quote = False
      for c in htmlTxt:
        if c == '<' and not quote:
          tag = True
        elif c == '>' and not quote:
          tag = False
        elif (c == '"' or c == "'") and tag:
          quote = not quote
        elif not tag:
          ret = ret + joinGlue + c
    elif method == 5:
      import xml
      ret = joinGlue.join(xml.etree.ElementTree.fromstring(htmlTxt).itertext())

  if method == 1 or method == 2:
    import cgi
    ret = cgi.escape(ret)
    ret = re.sub('[<>]', '', ret)

  return ret.strip()



class MLStripper(object, HTMLParser):
  def __init__(self, joinGlue=' '):
    super(MLStripper, self).__init__()
    self.reset()
    self.fed = []
    self.joinGlue = joinGlue


  def handle_data(self, d):
    self.fed.append(d)


  def get_data(self):
    return self.joinGlue.join(self.fed)


# # function extracts text from html content, strips all tags
# @param htmlText incoming raw html
# @param stripComment is strip comment or not
# @param stripScript is strip script or not
# @return inner html text
def innerHTMLText(htmlBuf, stripComment=True, stripScript=True):
  from bs4 import BeautifulSoup

  soup = BeautifulSoup(htmlBuf, 'lxml')

  if stripScript:
    for elem in soup.findAll(name='script'):
      elem.extract()
  if stripComment:
    stripHTMLComments(htmlBuf=None, soup=soup)

  return ''.join(soup.findAll(text=True))


# # function concatinates all HTMLTags from extractor also strips elements
# @param selectorList incoming Selector
# @return inner text from incoming selector
def innerText(selectorList, delimiter=' ', innerDelimiter=' ', tagReplacers=None):
  extendInnerText = ExtendInnerText(tagReplacers, delimiter, innerDelimiter)
  extendInnerText.innerText(None, selectorList, None)
  ret = extendInnerText.stripHtml
  return ret


# # function concatinates all HTMLTags from extractor also strips elements
# @param selectorList incoming Selector
# @return list of inner text from incoming selector
def innerTextToList(selectorList, delimiter=' ', innerDelimiter=' ', tagReplacers=None):
  extendInnerText = ExtendInnerText(tagReplacers, delimiter, innerDelimiter)
  extendInnerText.innerTextToList(None, selectorList, None)
  ret = extendInnerText.stripHtml
  return ret


# # function looks fiers not empty extracted XPath from subXPathes, using subXPathPattern for real xpath creating
# @param xpath - incoming root xpath
# @param sel - incoming selector
# @param subXPathPattern - subXPath creation pattern
# @param subXPathes - list of subXPathes
# @return retXPath and retXPathValue values
def getFirstNotEmptySubXPath(xpath, sel, subXPathPattern, subXPathes):
  retXPath = None
  retXPathValue = None
  for subXPath in subXPathes:
    retXPath = xpath + (subXPathPattern % subXPath)
    try:
      retXPathValue = sel.xpath(retXPath).extract()
    except Exception as excp:
      logger.info(">>> Common xPath extractor exception, = " + retXPath + " excp=" + str(excp))
      retXPathValue = None
      continue
    if len(retXPathValue) > 0 and ''.join(retXPathValue).strip() != '':
      break
  return retXPath, retXPathValue


# # function call splitPairs for each element in incomeDict and fills return dict
# @param incomeDict incoming dict
# @param splitters incoming splitters
# @return result dict
def getPairsDicts(incomeDict, splitters=','):
  ret = {}
  if type(incomeDict) == types.DictType:
    for key in incomeDict:
      if type(incomeDict[key]) in types.StringTypes:
        ret[key] = splitPairs(incomeDict[key], splitters)
  return ret


# # function extracts splits incoming string by splitters into dict of name=value pairs
# @param buf incoming text buf
# @param splitters incoming splitters
# @return result dict
def splitPairs(buf, splitters=','):
  ret = {}
  splitStr = buf.split(splitters)
  for elem in splitStr:
    localStr = elem.split('=')
    if type(localStr) == types.ListType and len(localStr) >= 2:
      ret[localStr[0]] = localStr[1]
  return ret


# # function looks is str2 an a tail of str1
# @param str1 main string
# @param str2 searching tail substring
# @return False or True
def isTailSubstr(str1, str2):
  ret = False
  if str1.find(str2) > 0 and ((len(str1) - str1.find(str2)) == len(str2)):
    ret = True
  return ret


# # function make string raplacement while
# @param buf incoming text buf
# @param replaceFrom substring for replacement from
# @param replaceTo substring for replacement to
# @return replacement string
def replaceLoopValue(buf, replaceFrom, replaceTo):
  localValue = buf
  replaceValue = localValue.replace(replaceFrom, replaceTo)
  while len(replaceValue) != len(localValue):
    localValue = replaceValue
    replaceValue = localValue.replace(replaceFrom, replaceTo)
  return localValue


# # function extract html redirect link from meta
# @param utf8Buff incoming buff of html page
# @return html redirect link
def extractHTMLRedirectFromMeta(utf8Buff):
  ret = None
  matchingVal = re.compile(META_RE_0, re.I)
  localREList = matchingVal.findall(utf8Buff)
  if len(localREList) > 0 and localREList[0] != "":
    bodyStr = localREList[0]
    if bodyStr.find(META_RE_1) >= 0:
      matchingVal = re.compile(META_RE_2, re.I)
      localREList = matchingVal.findall(bodyStr)
      if len(localREList) > 0:
        ret = localREList[0]
  return ret


# # function parse incoming email adress
# @param href - incoming email href
# @param onlyName - extract email names instead full email names
# @param defaultSeparator - default separator between email elements
# @return parsed email
def emailParse(href, onlyName=False, defaultSeparator=' '):  # pylint: disable=W0613
  ret = href
  splitHref = href.split('?')
  if splitHref is not None and len(splitHref) > 0:
    adresses = splitHref[0]
    adresses = adresses.split(',')
    if onlyName:
      names = []
      for adress in adresses:
        adress = adress.split('@')
        if adress is not None and len(adress) > 0:
          names.append(adress[0])
      adresses = names
    ret = ''
    for adress in adresses:
      ret += adress
      ret += ' '
    ret = ret.strip()
  return ret



# #Multi process logger
#
class MPLogger(object):


  # #constructor
  #
  def __init__(self):
    super(MPLogger, self).__init__()


##rotation log files of logger
  #
  #@param loggerName - name of logger
  #@return - None
  def getLogger(self, loggerName=None):
    if loggerName is None:
      ln = APP_CONSTS.LOGGER_NAME
    else:
      ln = loggerName

    try:
      # Get regular logger
      lg = logging.getLogger(ln)
      # Replace logger name for processes instances
      lfn = LoggerFileName(lg)
      fname = lfn.findReplace()
      if fname is not None and fname != '':
        pin = lfn.getFreeProcInstanceNumber(os.path.basename(fname))
        if pin != '' and pin != '0':
          pin = '.' + pin + '.log'
          fname = lfn.findReplace(fname + pin)
          lg = logging.getLogger(ln)

      return lg

    except Exception, err:
      raise Exception('Logger initialization error:' + str(err) + "\n" + getTracebackInfo())


def strToUnicode(inputStr):
  ret = inputStr

  if isinstance(inputStr, str):
    ret = inputStr.decode('utf-8')

  return ret


# Split string removes duplicated peaces and joing back
# @param inStr - input string
# @param delimiter - splitter delimiter
# @param joingGlue - optional glue string to joing with, if None or omitted - the delimiter used
# @param trimMode - peaces trim mode: 0 - not trimmed, 1 - trimmed left, 2 - trimmed right, 3 - trimmed both
# @return string with duplicated peaces removed
def removeDuplicated(inStr, delimiter="\n", joingGlue=None, trimMode=1, skipEmpty=False):
  ret = inStr.split(delimiter)

  if joingGlue is None:
    glue = delimiter
  else:
    glue = joingGlue

  prev = None
  new = []
  for item in ret:
    if trimMode > 0:
      if trimMode == 1:
        item = item.lstrip()
      elif trimMode == 2:
        item = item.rstrip()
      else:
        item = item.strip()
    if skipEmpty and item == '':
      continue
    if item != prev:
      new.append(item)
    prev = item
  ret = new

  return glue.join(ret).strip()


# Checks is the input content possible contains an CSS markup, possible is an in-line STYLE tag innerHTML
#
# @param content - to analyse
# @return zero if presence of the CSS markup is not detected or number of the detected fragments
def getContentCSSMarkupEntrancesNumber(content):
  return len(re.findall(r'\{.+?\}', content))


# Class ExceptionLog for logging of the exception common way
class ExceptionLog:

  ##Constans used in class
  LEVEL_NAME_ERROR = 'error'
  LEVEL_NAME_INFO = 'info'
  LEVEL_NAME_DEBUG = 'debug'

  LEVEL_VALUE_ERROR = logging.ERROR
  LEVEL_VALUE_INFO = logging.INFO
  LEVEL_VALUE_DEBUG = logging.DEBUG

  ##Constructor
  #
  #@param logger - logger instance
  #@param error - Exception inherited object instance
  #@param message - error message string
  #@param objects - objects tuple to dump
  def __init__(self, logger, error, message, objects):
    self.logger = logger
    self.error = error
    self.message = message
    self.objects = objects


  ## Static handler for logging of the exception
  #
  #@param logger - logger instance
  #@param error - Exception inherited object instance
  #@param message - error message string
  #@param objects - objects tuple to dump
  @staticmethod
  def handler(logger, error, message, objects=(), levels={}):
    #dictionary with default values
    levelsDict = { \
                  ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_ERROR, \
                  ExceptionLog.LEVEL_NAME_INFO:ExceptionLog.LEVEL_VALUE_INFO, \
                  ExceptionLog.LEVEL_NAME_DEBUG:ExceptionLog.LEVEL_VALUE_DEBUG \
                 }

    #filling levelDict necessary log level values
    for name, level in levels.items():
      if levelsDict.has_key(name):
        levelsDict[name] = level

    #Log the error message and Exception object with the ERROR level
    logger.log(levelsDict[ExceptionLog.LEVEL_NAME_ERROR], message + ' ' + str(error))

    #Log the traceback with INFO level.
    logger.log(levelsDict[ExceptionLog.LEVEL_NAME_INFO], getTracebackInfo())

    #Log the objects dumps with DEBUG level.
    if isinstance(objects, tuple):
      for obj in objects:
        logger.log(levelsDict[ExceptionLog.LEVEL_NAME_DEBUG], varDump(obj))


  ## Dump log of the exception
  #
  #@param - None
  #@return - None
  def dump(self):
    ExceptionLog.handler(self.logger, self.error, self.message, self.objects)
