"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file CrawlerTask.py
@author scorp <developers.hce@gmail.com>
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""
# import os
import os.path
import sys
import time
import hashlib

# import pickle
try:
  import cPickle as pickle
except ImportError:
  import pickle

# import logging
import logging.config
import ConfigParser
import re
import requests
import types
import copy
from dc_crawler.Fetcher import BaseFetcher
#from dc_crawler import LangDetector
import json
import datetime
import lxml.etree
import MySQLdb as mdb
import tidylib
import CollectURLs
import CollectProperties
import base64
import urlparse
import DetectModified
from app.Filters import Filters
from dateutil.parser import *  # pylint: disable=W0401,W0614
from cement.core import foundation
from collections import namedtuple
from time import strftime

from app.Utils import varDump
from dc.EventObjects import Site
from dc.EventObjects import Batch
from CrawledResource import CrawledResource
from UrlSchema import UrlSchema
from app.Utils import PathMaker
from CrawlerException import CrawlerException
from DBTasksWrapper import DBTasksWrapper
from app.Utils import SQLExpression
from app.CommonCrawlerDataStorage import CommonCrawlerDataStorage
from RobotsParser import RobotsParser
from RefererHeaderResolver import RefererHeaderResolver
from app.Exceptions import SeleniumFetcherException
from ProxyResolver import ProxyResolver
import app.Utils as Utils  # pylint: disable=F0401
import dc.Constants as DC_CONSTS
import dc_crawler.Constants as CONSTS
import dc_processor.Constants as PCONSTS
import app.Consts as APP_CONSTS
import dc.EventObjects as dc_event
import URLProcess
import ResourceProcess
import app.Profiler
from app.Utils import ExceptionLog


DB_SITES = "dc_sites"
DB_URLS = "dc_urls"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting. "
MSG_ERROR_LOAD_OPTIONS = "Error loading options. Exciting. "
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Can't load logging config file. Exiting. "
MSG_ERROR_LOAD_SITE_DATA = "Can't load site data: "
MSG_ERROR_LOAD_URL_DATA = "Can't load url data: "
MSG_ERROR_PROCESS_BATCH_ITEM = "Can't process batch item "
MSG_ERROR_WRITE_CRAWLED_DATA = "Can't write crawled data "
MSG_ERROR_COLLECT_URLS = "Can't collect urls "
MSG_ERROR_ADD_URL_TO_BATCH_ITEM = "Can't add url to batch item "
MSG_ERROR_LOAD_SITE_PROPERTIES = "Can't load site properties "
MSG_ERROR_CRAWL_SITE = "Can't crawl site "
MSG_ERROR_CHECK_SITE = "Site don't passed check site "
MSG_ERROR_GET_DIR = "Can't get dir "
MSG_ERROR_READ_SITE_FROM_DB = "Can't read site data from db"
MSG_ERROR_EMPTY_RESPONSE_SIZE = "Empty response"
MSG_ERROR_CONTENT_TYPE = "Invalid content-type"

MSG_INFO_PROCESS_BATCH = "ProcessBatch "
MSG_INFO_STORE_COOKIES_FILE = "Store cookies file on disk."

MSG_DEBUG_NON_PROCESSING = "ProcessorName is NONE. Exclude batch item from further processing."

SITE_MD5_EMPTY = "d41d8cd98f00b204e9800998ecf8427e"

DEFAULT_MAX_SIZE = 1000000
EMPTY_RESPONSE_SIZE = "0"

APP_NAME = "crawler-task"

HTTP_COOKIE = "HTTP_COOKIE"
DEFAULT_HTTP_COOKIE = ""
HTTP_HEADERS = "HTTP_HEADERS"
DEFAULT_HTTP_HEADER = ""

DC_URLS_DB_NAME = "dc_urls"
DC_URLS_TABLE_PREFIX = "urls_"
DC_SITES_DB_NAME = "dc_sites"
DC_SITES_PROPERTIES_TABLE_NAME = "sites_properties"
DC_SITES_TABLE_NAME = "sites"
DC_URLS_TABLE_NAME = "urls"
COOKIES_FILE_POSTFIX = ".cookies.txt"

NON_PROCESSING = "NONE"

HTTP_REDIRECT = "<Response [301]>"
HTML_REDIRECT = ""
MAX_HTTP_REDIRECTS_UNLIMITED = 0
MAX_HTML_REDIRECTS_UNLIMITED = 0
META_XPATH = "//meta[contains(@content, 'url')]/@content"

Results = namedtuple("Results", "exit_code, output, err")

ROBOTS_PATTERN = re.compile(r'(https?://[^/]+).*', re.I)

TEXT_CONTENT_TYPE_PATTERN = re.compile('text', re.I)

ENV_CRAWLER_STORE_PATH = "ENV_CRAWLER_STORE_PATH"
# SiteProperties = namedtuple("http_header", "http_cookie")
# CrawlResponse = namedtuple("html_content", "html_header", "html_request")
# named tuple for filters
#Filter = namedtuple("Filter", "pattern, pattern_name, type, state")

DETECT_MIME_MAIN_CONTENT = "1"
RECOVER_IF_FAILED = "2"

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

# #The CrawlerTask class, is a interface for fetching content from the web
#
# This object is a run at once application
class CrawlerTask(foundation.CementApp):


  # Configuration settings options names
  DB_HOST_CFG_NAME = "db_host"
  DB_PORT_CFG_NAME = "db_port"
  DB_USER_CFG_NAME = "db_user"
  DB_PWD_CFG_NAME = "db_pwd"
  DB_SITES_CFG_NAME = "db_dc_sites"
  DB_URLS_CFG_NAME = "db_dc_urls"

  RAW_DATA_DIR = "raw_data_dir"
  SITE_TEMPLATES = "dc_site_template"
  KEY_VALUE_STORAGE_DIR = "key_value_storage_dir"
  DB_DATA_DIR = "db_data_dir"


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      #self.site_table_row = None
      self.url_xpath_list = []


  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.refererHeaderResolver = RefererHeaderResolver()
    self.dcUrlsDbConnect = None
    self.raw_data_dir = None
    self.batchItem = None
    self.dc_sites_db_connect = None
    self.siteTable = None
    self.site_connector = None
    self.urlConnector = None
    self.logger = None
    self.urlTable = None
    self.batch = None
    self.defaultHeaderFile = None
    self.defaultCookieFile = None
    self.kvDbDir = None
    self.defaultIcrCrawlTime = None
    self.dbWrapper = None
    self.store_items = []
    self.max_fetch_time = CONSTS.FETCHER_TIME_LIMIT_MAX
    self.collect_additional_prop = False
    self.exit_code = EXIT_SUCCESS
    self.keep_old_resources = False
    self.urlProcess = None
    self.resourceProcess = None
    self.siteHTTPHeaders = None
    self.tidyOptions = {'numeric-entities': 1, 'char-encoding': 'utf8'}
    self.headerFileDir = None
    self.robotsFileDir = None
    self.fileHeaders = None
    self.siteHeaders = None
    self.hTTPHeadersStorage = CommonCrawlerDataStorage()
    self.robotsParser = RobotsParser()
    self.feedItems = None
    self.collectURLsItems = []
    self.errorMask = APP_CONSTS.ERROR_OK
    self.httpApplyHeaders = None
    self.chainDict = {}
    self.chainIndex = 0
    self.schemaBatchItems = []
    self.useZeroSiteIdSiteNotExists = False
    self.dir = None
    self.curBatchIterations = 1
    self.resetVars()


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load mandatory options
    self.loadOptions()

    # load db backend
    self.loadDBBackend()

    # load key-value db
    self.loadKeyValueDB()

    # make processing batch data
    self.processBatch()

    # Finish logging
    self.logger.info(APP_CONSTS.LOGGER_DELIMITER_LINE)



  # #fetchSiteHeader reads from file and returns fileHeader element
  #
  # @return is collect successfully
  # bullshit, move it to the separate module (like URLCollector)
  def fetchFileHeader(self, url, siteId):
    ret = None
    if self.headerFileDir is not None:
      host = Utils.UrlParser.getDomain(url)
      if host is not None:
        ret = self.hTTPHeadersStorage.loadElement(self.headerFileDir, host, siteId, self.siteHeaders)
    return ret


  # #fetchSiteHeader reads from file and returns fileHeader element
  #
  # @return is collect successfully
  # bullshit, move it to the separate module (like URLCollector)
  def saveFileHeader(self, url, siteId, localFileHeaders):
    if self.headerFileDir is not None:
      auth = urlparse.urlsplit(url.strip())[1]
      host = re.search('([^@]*@)?([^:]*):?(.*)', auth).groups()[1]
      if host is not None:
        self.hTTPHeadersStorage.saveElement(self.headerFileDir, host, siteId, localFileHeaders)


  # #RLs collect URL from response body and fills new batch items for batching iterations
  #
  # @return is collect successfully
  def collectURLs(self):
    collectURLsResult = False
    if self.dom is not None:
      collectURLs = CollectURLs.CollectURLs()
      collectURLs.crawledResource = self.crawledResource
      collectURLs.url = self.url
      collectURLs.dom = self.dom
      collectURLs.realUrl = self.realUrl
      collectURLs.processorName = self.processorName
      collectURLs.batchItem = self.batchItem
      collectURLs.urlXpathList = self.url_xpath_list
      collectURLs.siteProperties = self.siteProperties
      collectURLs.site = self.site
      collectURLs.dbWrapper = self.dbWrapper
      collectURLs.autoRemoveProps = self.auto_remove_props
      collectURLs.autoDetectMime = self.autoDetectMime
      collectURLs.processContentTypes = self.processContentTypes
      collectURLs.postForms = self.postForms
      collectURLs.urlProcess = self.urlProcess
      if "COLLECT_ROBOTS_MODE" not in self.siteProperties or int(self.siteProperties["COLLECT_ROBOTS_MODE"]) > 0:
        collectURLs.robotsParser = self.robotsParser

      collectURLsResult, internalLinks, externalLinks, urlObjects, self.feedItems, chainUrls = \
                          collectURLs.process(self.crawledResource.http_code,
                                              not self.batch.dbMode & dc_event.Batch.DB_MODE_W,
                                              self.httpApplyHeaders)
      if collectURLsResult and self.collect_additional_prop and (len(internalLinks) > 0 or len(externalLinks)):
        collectPropertiesObj = CollectProperties.CollectProperties()
        collectPropertiesObj.siteId = Utils.autoFillSiteId(self.batchItem.siteId, self.logger)
        collectPropertiesObj.kvDbDir = self.kvDbDir
        collectPropertiesObj.res = self.res
        collectPropertiesObj.batchItem = self.batchItem
        collectPropertiesObj.realUrl = self.realUrl
        collectPropertiesObj.process(self.dom, internalLinks, externalLinks)

      # Fill new batch items in self.collectURLsItems
      if urlObjects is not None and len(urlObjects) > 0 and \
      (self.batchItem.depth > 0 or self.processorName == PCONSTS.PROCESSOR_RSS):
        self.fillItemsFromIterations(urlObjects)
      if chainUrls is not None and len(chainUrls) > 0:
        self.fillItemsFromIterationsWithChain(chainUrls, self.batchItem)


  # #writeData  write the response body and headers to file
  #
  # @param
  def writeData(self):
    # check wether need store to disk
    if self.needStoreMime and self.needStoreMime != '*':
      if self.crawledResource.content_type.lower() not in self.needStoreMime:
        self.logger.info("do not store resource files to disk")
        return
        # self.logger.info("do not store resource files to disk")
        # self.updateURLForFailed(CONSTS.ERROR_MIME_NOT_WRITE)
        # raise CrawlerException("MIME Content-Type not allow store content to disk")

    #self.logger.info("store resource files to disk")
    #self.logger.debug("urlMd5: %s" % str(self.batchItem.urlId))
    #self.makeDir()
    # html content
    #base_path = os.path.join(self.dir, self.batchItem.urlId + "_" + self.time_postfix)

    # save raw file
    #html_content_file_name = base_path + ".bin"
    #self.logger.debug("self.crawledResource: %s" % varDump(self.crawledResource))
    #with open(html_content_file_name, "wb") as f:

    urlPut_list = []

    if TEXT_CONTENT_TYPE_PATTERN.match(self.crawledResource.content_type):
      # save UTF-8 encoding text for text content types
      if self.crawledResource.dynamic_fetcher_type:
        raw_unicode_content = self.crawledResource.meta_content
      else:
        raw_unicode_content = self.crawledResource.html_content
      if raw_unicode_content:
        if self.crawledResource.charset is not None and self.crawledResource.charset.lower() != "utf-8":
          raw_unicode_content = raw_unicode_content.decode(self.crawledResource.charset)

        putDict = {"data": base64.b64encode(raw_unicode_content)}
        contentType = dc_event.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
        #f.write(raw_unicode_content)

        # save tidy recovered file
        localDom = None
        if self.htmlRecover is not None and self.htmlRecover == "2":
          localDom = self.resourceProcess.domParser(None, raw_unicode_content, 200,
                          self.crawledResource.charset if self.crawledResource is not None else None)
        if self.htmlRecover is not None and (self.htmlRecover == "1" or self.htmlRecover == "2" and localDom is None):
          #with open(base_path + ".tidy", "wb") as tidy_f:
          tidy_content = tidylib.tidy_document(raw_unicode_content, self.tidyOptions)[0]
          putDict = {"data": base64.b64encode(tidy_content)}
          contentType = dc_event.Content.CONTENT_TIDY_CONTENT
          urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
          #tidy_f.write(tidy_content)
    else:
      # save origin binary data for non-text content types
      if self.crawledResource.binary_content:
        putDict = {"data": base64.b64encode(self.crawledResource.binary_content)}
        contentType = dc_event.Content.CONTENT_RAW_CONTENT
        urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
        #f.write(self.crawledResource.binary_content)

    # save rendered file
    if self.crawledResource.dynamic_fetcher_type and self.crawledResource.html_content:
      putDict = {"data": base64.b64encode(self.crawledResource.html_content)}
      contentType = dc_event.Content.CONTENT_DYNAMIC_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
      #rendered_file_name = base_path + "." + self.crawledResource.dynamic_fetcher_type + ".dyn"
      #with open(rendered_file_name, "wb") as f:
        #f.write(self.crawledResource.html_content)

    # html header
    if self.store_http_headers and self.crawledResource.response_header:
      putDict = {"data": base64.b64encode(self.crawledResource.response_header)}
      contentType = dc_event.Content.CONTENT_HEADERS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
      #html_header_file_name = base_path + ".headers.txt"
      #with open(html_header_file_name, "wb") as f:
        #self.logger.warn("file is %s", html_header_file_name)
        #if self.crawledResource.response_header:
          #f.write(self.crawledResource.response_header)

    # html request
    if self.storeHttpRequest and self.crawledResource.html_request:
      putDict = {"data": base64.b64encode(self.crawledResource.html_request)}
      contentType = dc_event.Content.CONTENT_REQUESTS_CONTENT
      urlPut_list.append(dc_event.URLPut(self.batchItem.siteId, self.batchItem.urlId, contentType, putDict))
      #html_request_file_name = base_path + ".request.txt"
      #with open(html_request_file_name, "wb") as f:
        #f.write("GET %s HTTP/1.0\r\n\r\n" % (self.realUrl,))
        #if self.crawledResource.html_request:
          #f.write(self.crawledResource.html_request)

    # Write raw content on disk via db-task
    # Check if Real-Time crawling
    self.dbWrapper.putURLContent(urlPut_list)

    #change url's contentMask
    self.url.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
    self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK


  # #getDir prepare dir
  #
  # @param
  def getDir(self):
    if len(self.batchItem.siteId):
      self.dir = os.path.join(self.raw_data_dir, self.batchItem.siteId, PathMaker(self.batchItem.urlId).getDir())
    else:
      self.dir = os.path.join(self.raw_data_dir, "0", PathMaker(self.batchItem.urlId).getDir())


  # #makeDir prepare dir
  #
  # @param
  def makeDir(self):
    if not os.path.exists(self.dir):
      os.makedirs(self.dir)
    elif not os.path.isdir(self.dir):
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_WRITE_FILE_ERROR
      self.updateURLForFailed(APP_CONSTS.ERROR_WRITE_FILE_ERROR)
      raise Exception("path %s exists, but is not a directory" % (self.dir,))


  # #updateURLForFailed
  #
  # @param error_bit BitMask of error
  def updateURLForFailed(self, errorBit, httpCode=None, status=dc_event.URL.STATUS_CRAWLED, updateUdate=True):
    self.urlProcess.siteId = self.batchItem.siteId
    self.urlProcess.updateURLForFailed(errorBit, self.batchItem, httpCode, status, updateUdate)


  # #httpRequestWrapper method makes http request, using detectModified object if it present
  #
  # @param url - http requests url
  # @param headers - http headers, using in http request
  # @param auth - authentification
  # @param postData - data using in post request
  # @param urlObj - url's Obj
  # @return http resource (http response)
  def httpRequestWrapper(self, url, headers, auth, postData, urlObj, incomingContent):
    ret = None
    #(1) Fetcher type detection section
    localFetchType = copy.deepcopy(self.site.fetchType)
    if localFetchType == BaseFetcher.TYP_SPECIAL_DYNAMIC:
      if urlObj is not None and urlObj.parentMd5 is not None and \
      "DYNAMIC_FETCH_ONLY_FOR_ROOT_URL" in self.siteProperties and \
      int(self.siteProperties["DYNAMIC_FETCH_ONLY_FOR_ROOT_URL"]) > 0 and urlObj.parentMd5 == "":
        localFetchType = BaseFetcher.TYP_DYNAMIC
      else:
        localFetchType = BaseFetcher.TYP_NORMAL
    elif localFetchType == BaseFetcher.TYP_AUTO:
      localFetchType = BaseFetcher.TYP_NORMAL
    if self.curBatchIterations == 2 and self.processorName == PCONSTS.PROCESSOR_RSS and \
    localFetchType == BaseFetcher.TYP_DYNAMIC:
      localFetchType = BaseFetcher.TYP_NORMAL

    #(1) END
    self.logger.debug(">>> FetchType before applying = " + str(localFetchType))
    self.logger.debug(">>> Last mofify obj = " + str(self.detectModified))
    if self.detectModified is not None:
      self.logger.debug(">>> Last mofify field = " + str(self.detectModified.modifiedSettings))

    if self.detectModified is None or self.detectModified.modifiedSettings is None or \
    (urlObj is not None and urlObj.crawled == 0):
      if incomingContent is None:
        localFilters = Filters(self.site.filters, self.dbWrapper, self.batchItem.siteId)
        ret = BaseFetcher.get_fetcher(localFetchType).open(url,
          external_url=self.external_url, timeout=int(self.url.httpTimeout) / 1000.0,
          headers=headers, allow_redirects=self.allow_http_redirects,
          proxies=self.proxies, auth=auth, data=postData, logger=self.logger,
          allowed_content_types=self.processContentTypes, max_resource_size=self.site.maxResourceSize,
          max_redirects=self.max_http_redirects, filters=localFilters, depth=urlObj.depth)
      else:
        ret = BaseFetcher.get_fetcher(BaseFetcher.TYP_CONTENT).open(url, inputContent=incomingContent)
    else:
      self.detectModified.expiredData = urlObj.CDate
      self.detectModified.eTags = urlObj.eTag
      self.detectModified.prevContentLen = urlObj.size
      self.detectModified.prevContentCrc32 = urlObj.rawContentMd5
      self.detectModified.prevContentDate = urlObj.CDate
      httpParams = {}
      httpParams["url"] = url
      httpParams["externalUrl"] = self.external_url
      httpParams["httpTimeout"] = int(self.url.httpTimeout) / 1000.0
      httpParams["httpHeader"] = headers
      httpParams["allowHttpRedirects"] = self.allow_http_redirects
      httpParams["proxies"] = self.proxies
      httpParams["auth"] = auth
      httpParams["postData"] = postData
      httpParams["processContentTypes"] = self.processContentTypes
      httpParams["maxResourceSize"] = self.site.maxResourceSize
      httpParams["maxHttpRedirects"] = self.max_http_redirects
      ret = self.detectModified.makeHTTPRequest(localFetchType, httpParams)

      if self.detectModified.isResourceNotChanged:
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_NOT_MODIFY
        self.detectModified.raiseExceptionIfNotModified()

    return ret


  # #notModifiedStateProcessing process resource in case of "not modified" state
  #
  # @param siteId - resource's siteId
  # @param url - resource's url
  def notModifiedStateProcessing(self, siteId, url):
    if self.detectModified is not None and self.detectModified.isResourceNotChanged:
      localStatus = 4
      updateUDate = True
      if self.detectModified.getBehaviour() == self.detectModified.BEHAVIOR_PROCESSED_STATUS:
        urlContentObj = dc_event.URLContentRequest(siteId, url, dc_event.URLContentRequest.CONTENT_TYPE_PROCESSED)
        urlContentResponse = self.dbWrapper.urlContent([urlContentObj])
        if len(urlContentResponse.processedContents) > 0:
          localStatus = 7
      elif self.detectModified.getBehaviour() == self.detectModified.BEHAVIOR_SAVE_UDATE:
        updateUDate = False
      self.updateURLForFailed(APP_CONSTS.ERROR_NOT_MODIFY, None, localStatus, updateUDate)


  # #crawl site
  #
  # @return should continue to write data and collect URLs
  def crawl(self, incomingContent):
    self.crawledResource = None
    # delay
    time.sleep(self.url.requestDelay / 1000.0)
    #(1) Decode url - [UrlProcess]
    # TODO move to - [UrlProcess]
    self.urlProcess.url = self.url.url
    #(1) END
    url = self.urlProcess.getRealUrl()
    self.realUrl = url
    startTime = time.time()

    #(3.1) Resolve addition http headers here
    self.fileHeaders = self.fetchFileHeader(url, self.batchItem.siteId)
    self.httpApplyHeaders = self.hTTPHeadersStorage.fetchLowFreqHeader(self.fileHeaders, self.siteHeaders)
    if self.httpApplyHeaders is not None:
      self.fileHeaders[self.httpApplyHeaders[0]][self.httpApplyHeaders[1]] += 1
      self.httpApplyHeaders = {self.httpApplyHeaders[0] : self.httpApplyHeaders[1]}
      self.saveFileHeader(url, self.batchItem.siteId, self.fileHeaders)
    else:
      self.httpApplyHeaders = self.headersDict
    self.logger.debug(">>> Addition headers are: " + str(self.httpApplyHeaders))

    self.refererHeaderResolver.resolveRefererHeader(self.httpApplyHeaders, int(self.siteProperties["REFERER_SELF_URL"])
        if "REFERER_SELF_URL" in self.siteProperties else RefererHeaderResolver.MODE_SIMPLE, url,
        self.batchItem.siteId, self.url.parentMd5, self.dbWrapper)

    try:
      #self.logger.debug("self.siteProperties:\n%s", varDump(self.siteProperties))
      if "ROBOTS_MODE" not in self.siteProperties or int(self.siteProperties["ROBOTS_MODE"]) > 0:
        self.logger.debug("Robots.txt obey mode ON")
        if self.robotsParser.loadRobots(url, self.batchItem.siteId, self.httpApplyHeaders):
          isAllowed, retUserAgent = self.robotsParser.checkUrlByRobots(url, self.batchItem.siteId,
                                                                       self.httpApplyHeaders)
          if not isAllowed:
            self.logger.debug(">>> URL " + url + " is NOT Allowed by user-agent:" + str(retUserAgent))
            self.errorMask = self.errorMask | APP_CONSTS.ERROR_ROBOTS_NOT_ALLOW
            self.updateSite(APP_CONSTS.ERROR_ROBOTS_NOT_ALLOW)
            self.updateURLForFailed(self.errorMask)
            return False
      #self.logger.info("allowed in robots.txt")
      self.logger.info("start to fetch: %s", url)
      #(2) Makes auth tuple - []
      # TODO move to - []
      if self.authName and self.authPwd:
        auth = (self.authName, self.authPwd)
        self.logger.info("using http basic auth %s:%s", self.authName, self.authPwd)
      else:
        auth = None
      #(2) END
      #(3) Resolve HTTP Method (move convertToHttpDateFmt to Utils) - [UrlProcess]
      # TODO move to - [UrlProcess]
      self.urlProcess.urlObj = self.url
      postData = self.urlProcess.resolveHTTP(self.postForms, self.httpApplyHeaders)
      #(3) END

      # Workaround
      url = self.urlProcess.urlTemplateApply(url, self.batch.crawlerType, self.urlTempalteRegular,
                                             self.urlTempalteRealtime, self.urlTempalteRegularEncode,
                                             self.urlTempalteRealtimeEncode)
      #(3.2) HTTP Fetcher with html redirect resolving
      localUrl = url
      prevUrl = localUrl
      while True:
        res = self.httpRequestWrapper(localUrl, self.httpApplyHeaders, auth, postData, self.url, incomingContent)
        if (res is None) or (self.max_html_redirects is None) or (not self.allow_html_redirects):
          break
        elif self.htmlRedirects > self.max_html_redirects:
          self.logger.debug(">>> Max html redirect was reached")
          self.errorMask = self.errorMask | APP_CONSTS.ERROR_MAX_ALLOW_HTML_REDIRECTS
          self.updateURLForFailed(APP_CONSTS.ERROR_MAX_ALLOW_HTML_REDIRECTS)
          return False
        else:
          if res.rendered_unicode_content is not None:
            prevUrl = localUrl
            localUrl = Utils.extractHTMLRedirectFromMeta(res.rendered_unicode_content)
            if localUrl is None or localUrl == "":
              break
            else:
              if not localUrl.lower().startswith("http://") and not localUrl.lower().startswith("https://"):
                localUrl = urlparse.urljoin(prevUrl, localUrl)
              # normalization
              localUrl = dc_event.URL(0, localUrl).getURL()
              self.logger.debug(">>> HTML redurect to = " + localUrl)
          else:
            break
          self.htmlRedirects += 1

      #(4) Checks and update url in db-task them - [UrlProcess]
      # TODO move to - [UrlProcess]
      # (OLD if version) if res.headers is None:
      if res.error_mask != 0:
        self.updateURLForFailed(res.error_mask)
        self.errorMask = self.errorMask | res.error_mask
        return False

      if "content-length" in res.headers and res.headers["content-length"] == EMPTY_RESPONSE_SIZE:
        self.logger.debug("content-length: %s", str(res.headers["content-length"]))
        self.logger.debug(MSG_ERROR_EMPTY_RESPONSE_SIZE)
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_EMPTY_RESPONSE
        self.updateURLForFailed(APP_CONSTS.ERROR_EMPTY_RESPONSE)
        return False
      #(4) END

      # save res to self, will use it in next steps
      #(5) Create and fill resource struct - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      self.res = res
      self.logger.info("response code:%s", str(self.res.status_code))
      # use charset to improve encoding detect
      self.crawledTime = time.time()
      self.resourceProcess.urlObj = self.url
      resource = self.resourceProcess.generateResource(startTime, res, self.headers, self.crawledTime,
                                                       self.defaultIcrCrawlTime)
      # updated by Oleksii
      # save cookies from response
      resource.cookies = res.cookies
      resource.dynamic_fetcher_type = res.dynamic_fetcher_type
      self.crawledResource = resource
      #(5) END

      if self.crawledResource.http_code >= 400:
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_HTTP_ERROR
        self.updateSite(APP_CONSTS.ERROR_HTTP_ERROR)
        self.updateURLForFailed(APP_CONSTS.ERROR_HTTP_ERROR, self.crawledResource.http_code)
        return False

      # Added by Oleksii
      # support of HTTP_REDIRECTS_MAX
      #(7) Check response for redirect count [???]
      self.checkResponse()

      if not self.allow_http_redirects:
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS
        self.updateSite(APP_CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS)
        self.updateURLForFailed(APP_CONSTS.ERROR_MAX_ALLOW_HTTP_REDIRECTS, self.crawledResource.http_code)
        return False
      #(7) END

      #(8) Check Content size and response code - [ResourceProcess]
      # TODO move to - [ResourceProcess]
      self.resourceProcess.resource = resource
      self.resourceProcess.batchItem = self.batchItem
      if not self.resourceProcess.checkResourcesResponse(res, self.site.maxResourceSize, self.updateSite):
        self.errorMask = self.errorMask | resource.error_mask
        self.updateURLForFailed(resource.error_mask, self.crawledResource.http_code)
        return False
      #(8) END

    except requests.exceptions.Timeout, err:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_REQUEST_TIMEOUT
      self.updateURLForFailed(APP_CONSTS.ERROR_REQUEST_TIMEOUT)
      return False
    except requests.exceptions.InvalidURL:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_BAD_URL
      self.updateURLForFailed(APP_CONSTS.ERROR_BAD_URL)
      return False
    except requests.exceptions.TooManyRedirects:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_BAD_REDIRECTION
      self.updateURLForFailed(APP_CONSTS.ERROR_BAD_REDIRECTION)
      return False
    except requests.exceptions.ChunkedEncodingError:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_PAGE_CONVERT_ERROR
      self.updateURLForFailed(APP_CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except requests.exceptions.ConnectionError:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_CONNECTION_ERROR
      self.updateURLForFailed(APP_CONSTS.ERROR_CONNECTION_ERROR)
      return False
    except requests.exceptions.ContentDecodingError:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_PAGE_CONVERT_ERROR
      self.updateURLForFailed(APP_CONSTS.ERROR_PAGE_CONVERT_ERROR)
      return False
    except lxml.etree.XMLSyntaxError:  # pylint: disable=E1101
      self.logger.warning("XML HTML syntax error")
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_DTD_INVALID
      self.updateURLForFailed(APP_CONSTS.ERROR_DTD_INVALID)
      return False
    except SeleniumFetcherException, err:
      self.logger.error("Selenium fetcher error: " + str(err.message))
      self.errorMask = self.errorMask | err.code
      self.updateURLForFailed(err.code)
      return False
    except DetectModified.NotModifiedException:
      self.notModifiedStateProcessing(self.batchItem.siteId, self.realUrl)
      return False
    except Exception, err:
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_HTML_STRUCTURE
      self.updateURLForFailed(APP_CONSTS.ERROR_HTML_STRUCTURE)
      ExceptionLog.handler(self.logger, err, "Can't crawl resource.", (err))
      self.logger.error(str(type(err)) + "\n")
      return False

    return True


  # updateAVGSpeed update sites.AVGSpeed property
  def updateAVGSpeed(self):
    if self.res.status_code == 304:
      # not modified will return empty response body
      return
    newSpeed = (self.site.avgSpeed * self.site.avgSpeedCounter + self.crawledResource.bps) / \
               (self.site.avgSpeedCounter + 1)
    localSiteUpdate = dc_event.SiteUpdate(self.batchItem.siteId)
    for attr in localSiteUpdate.__dict__:
      if hasattr(localSiteUpdate, attr):
        setattr(localSiteUpdate, attr, None)
    localSiteUpdate.id = self.batchItem.siteId
    localSiteUpdate.avgSpeed = newSpeed
    localSiteUpdate.avgSpeedCounter = SQLExpression("`AVGSpeedCounter` + 1")
    self.dbWrapper.siteNewOrUpdate(localSiteUpdate)


  # saveCookies
  def saveCookies(self):
    self.logger.debug(MSG_INFO_STORE_COOKIES_FILE)
    timePostfix = ""
    if self.keep_old_resources:
      timePostfix = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
    self.logger.debug("Store cookies properties:\n%s", varDump(self.store_cookies))
    if self.store_cookies and self.store_cookies == "1":
      cookies_str = [key + ": " + value + "; " for (key, value) in self.crawledResource.cookies.items()]
      self.makeDir()
      self.logger.debug("Response cookies dict: %s", str(self.crawledResource.cookies))
      self.logger.debug("Response cookies string: %s", str(cookies_str))
      self.logger.debug("self.batchItem.urlId: %s", str(self.batchItem.urlId))
      self.logger.debug("timePostfix: %s", str(timePostfix))
      if timePostfix == "":
        base_path = os.path.join(self.dir, self.batchItem.urlId)
      else:
        base_path = os.path.join(self.dir, self.batchItem.urlId + "_" + str(timePostfix))
      cookies_file_name = base_path + COOKIES_FILE_POSTFIX
      with open(cookies_file_name, "wb") as f:
        f.write(''.join(sorted(cookies_str)))


  # #prepare request headers and cookie
  #
  # @param site - site object
  # TODO move to - [SiteProcess]
  def readSiteProperties(self):
    # TODO not sure is it right that fetch headers and cookie
    # by `Name` = 'headers'/'cookie'
    self.storeHttpRequest = True
    self.store_http_headers = True
    self.headersDict = {}
    self.postForms = {}
    self.headers, self.cookie = None, None
    self.proxies = None
    self.needStoreMime = '*'
    self.authName, self.authPwd = None, None
    self.external_url = None
    self.auto_remove_props = {}
    self.htmlRecover = None
    self.autoDetectMime = None
    self.processContentTypes = []
    cookie = None

    try:
      #stub
      self.logger.debug("self.site.properties: %s", varDump(self.site.properties))

      # Update site properties from batch item properties
      keys = [localProperty["name"] for localProperty in self.site.properties]
      self.logger.debug("keys: %s" % str(keys))
      for key in self.batchItem.properties.keys():
        if key in keys:
          for localProperty in self.site.properties:
            if localProperty["name"] == key:
              self.logger.debug("%s exist in site properties. Rewrite localProperty" % key)
              self.logger.debug("Old value: %s" % varDump(localProperty["value"]))
              localProperty["value"] = self.batchItem.properties[key]
              self.logger.debug("New value: %s" % varDump(localProperty["value"]))
        else:
          self.logger.debug("%s don't exist in site properties. Add localProperty" % key)
          self.site.properties.append({"name":key, "value":self.batchItem.properties[key],
                                       "URLMd5":self.batchItem.urlId})
      self.logger.debug("Updated site's properties: " + str(self.site.properties))

      self.siteProperties = {}
      for item in self.site.properties:
        self.siteProperties[item["name"]] = item["value"]

      if 'HTTP_HEADERS' in self.siteProperties:
        localHeaders = self.siteProperties['HTTP_HEADERS']
        self.siteHeaders = self.hTTPHeadersStorage.extractSiteStorageElement(localHeaders)
      if 'HTTP_COOKIE' in self.siteProperties:
        cookie = self.siteProperties['HTTP_COOKIE']
      if 'STORE_HTTP_REQUEST' in self.siteProperties:
        self.storeHttpRequest = int(self.siteProperties['STORE_HTTP_REQUEST']) != 0
      if 'STORE_HTTP_HEADERS' in self.siteProperties:
        self.store_http_headers = int(self.siteProperties['STORE_HTTP_HEADERS']) != 0
      if 'MIME_TYPE_STORE_ON_DISK' in self.siteProperties:
        allowMimes = self.siteProperties['MIME_TYPE_STORE_ON_DISK']
        if allowMimes == '' or allowMimes == '*':
          self.needStoreMime = '*'
        else:
          self.needStoreMime = set([mime.lower() for mime in allowMimes.split(',')])
      if 'HTTP_AUTH_NAME' in self.siteProperties:
        self.authName = self.siteProperties['HTTP_AUTH_NAME']
      if 'HTTP_AUTH_PWD' in self.siteProperties:
        self.authPwd = self.siteProperties['HTTP_AUTH_PWD']
      for key in self.siteProperties.keys():
        if key.startswith('HTTP_POST_FORM_'):
          self.postForms[key[15:]] = self.siteProperties[key]
      if 'EXTERNAL_FETCHER_URL' in self.siteProperties:
        self.external_url = self.siteProperties['EXTERNAL_FETCHER_URL']  # pylint: disable=W0201
      if 'HTML_RECOVER' in self.siteProperties:
        self.htmlRecover = self.siteProperties['HTML_RECOVER']  # pylint: disable=W0201
      if 'MIME_TYPE_AUTO_DETECT' in self.siteProperties:
        self.autoDetectMime = self.siteProperties['MIME_TYPE_AUTO_DETECT']  # pylint: disable=W0201
      if 'ALLOWED_CTYPES' in self.siteProperties:
        self.processContentTypes = self.siteProperties['ALLOWED_CTYPES'].lower().split(',')  # pylint: disable=W0201
      if 'PROCESSOR_NAME' in self.siteProperties:
        self.processorName = self.siteProperties['PROCESSOR_NAME']
      if DC_CONSTS.SITE_PROP_SAVE_COOKIES in self.siteProperties:
        self.store_cookies = self.siteProperties[DC_CONSTS.SITE_PROP_SAVE_COOKIES]

      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in self.siteProperties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] = \
        self.siteProperties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in self.siteProperties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] = \
        self.siteProperties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in self.siteProperties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] = \
        self.siteProperties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE]
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE_ACTIVE in self.siteProperties:
        self.auto_remove_props[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE_ACTIVE] = \
        self.siteProperties[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE_ACTIVE]

      if 'HTTP_REDIRECTS_MAX' in self.siteProperties:
        self.max_http_redirects = int(self.siteProperties['HTTP_REDIRECTS_MAX'])
      if 'HTML_REDIRECTS_MAX' in self.siteProperties:
        self.max_html_redirects = int(self.siteProperties['HTML_REDIRECTS_MAX'])
      if 'URL_XPATH_LIST' in self.siteProperties:
        self.url_xpath_list = json.loads(self.siteProperties['URL_XPATH_LIST'])['xpath_list']
      if 'URL_TEMPLATE_REGULAR' in self.siteProperties:
        self.urlTempalteRegular = self.siteProperties['URL_TEMPLATE_REGULAR']
      if 'URL_TEMPLATE_REALTIME' in self.siteProperties:
        self.urlTempalteRealtime = self.siteProperties['URL_TEMPLATE_REALTIME']
      if 'URL_TEMPLATE_REGULAR_ENCODE' in self.siteProperties:
        self.urlTempalteRegularEncode = self.siteProperties['URL_TEMPLATE_REGULAR_ENCODE']
      if 'URL_TEMPLATE_REALTIME_ENCODE' in self.siteProperties:
        self.urlTempalteRealtimeEncode = self.siteProperties['URL_TEMPLATE_REALTIME_ENCODE']
      if 'PROTOCOLS' in self.siteProperties:
        if self.urlProcess is not None:
          self.urlProcess.setProtocols(self.siteProperties['PROTOCOLS'])
      if 'DETECT_MODIFIED' in self.siteProperties:
        self.detectModified = DetectModified.DetectModified(self.siteProperties['DETECT_MODIFIED'])

      if self.defaultHeaderFile is not None:
        # read request headers from crawler-task_headers.txt file
        self.headers = CrawlerTask.readSmallFileContent(self.defaultHeaderFile)
      if cookie is None:
        # read request cookies from crawler-task_headers.txt file
        cookie = CrawlerTask.readSmallFileContent(self.defaultCookieFile)
      if cookie and cookie.lower().startswith('cookie:'):
        self.cookie = cookie[len('cookie:'):]
      else:
        self.cookie = cookie

      for header in self.headers.splitlines():
        if not header:
          continue
        try:
          key, value = header[:header.index(':')].strip(), header[header.index(':') + len(':'):].strip()
        except Exception:
          self.logger.warn("header:%s", header)
        self.headersDict[key] = value
      if self.cookie:
        self.headersDict['Cookie'] = self.cookie

      #self.proxies = {'http' : 'http://%s:%s' % (proxyHost, proxyPort)}
      self.logger.debug("proxies: %s", self.proxies)
    except Exception, err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_LOAD_SITE_PROPERTIES)


  # # resolveProxy resolve http proxies for current url
  #
  def resolveProxy(self):
    ret = None
    if self.url is not None:
      proxyResolver = ProxyResolver(self.siteProperties, self.dbWrapper, self.batchItem.siteId, self.url.url)
      result = proxyResolver.getProxy()
      if result is not None:
        ret = ("http", result[0], result[1], None, None)
    return ret


  # # readSmallFileContent read small file content
  #
  # @param path the file path to read
  @staticmethod
  def readSmallFileContent(path):
    with open(path, 'r') as f:
      return ''.join(f.readlines())


  # #update site params
  # update site params
  # @param mask ErrorMask
  # @param is_suspend should set state to suspend for this site
  def updateSite(self, mask, is_suspend=False):
    response = False
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batchItem.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batchItem.siteId
      #TODO: possible redundant for the URLFetch algorithm and need to be removed
      if mask:
        localSiteUpdate.errors = SQLExpression("`Errors` + 1")
        localSiteUpdate.errorMask = SQLExpression(("`ErrorMask` | %s" % mask))
        if is_suspend:
          localSiteUpdate.state = Site.STATE_SUSPENDED
      updated_count = self.dbWrapper.siteNewOrUpdate(localSiteUpdate)
      if updated_count > 0:
        response = True
    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_LOAD_SITE_DATA)
      raise

    return response


  # #check site params
  # check site params
  #
  # @return should continue execute this BatchItem
  def checkSite(self, urlObj):
    # TODO: self.site and self.url not must be none
    if not self.site or not urlObj:
      self.logger.error("Error: wrong condition,\nself.site:%s\nurlObj:%s", varDump(self.site), varDump(urlObj))
      return False
    self.logger.debug("urlObj:" + str(urlObj))

    if((self.batch.crawlerType != dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state != Site.STATE_ACTIVE)) \
    or ((self.batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER) and (self.site.state == Site.STATE_DISABLED)):
      self.logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!"
                        % (str(self.batch.crawlerType), str(self.site.state)))
      self.errorMask = self.errorMask | APP_CONSTS.ERROR_MASK_SITE_STATE
      return False

    # if not root url
    if urlObj.parentMd5 != "":
      if self.site.maxErrors > 0 and self.site.errors > self.site.maxErrors:
        self.logger.debug("Errors=" + str(self.site.errors) + " limit is reached maxErrors=" +
                          str(self.site.maxErrors))
        # TODO: possible improve suspend logic to use the property to define, suspend the site if limit reached or not
        # self.updateSite(CONSTS.ERROR_SITE_MAX_ERRORS, True)
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_SITE_MAX_ERRORS
        self.updateSite(APP_CONSTS.ERROR_SITE_MAX_ERRORS)
        return False

    return True


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  # TODO move to - [SiteProcess]
  def loadSite(self):
    try:
      ## FIXED alexv 2015-11-11
      #if not len(self.batchItem.siteId):
      #  self.batchItem.siteId = "0"
      self.readSiteFromDB()
      # Check if site not exist then read site for siteId=0
      if self.site.id == SITE_MD5_EMPTY and bool(self.useZeroSiteIdSiteNotExists):
        self.logger.debug("Site not found. Take site id 0")
        self.batchItem.siteId = "0"
        self.batchItem.urlObj.siteId = "0"
        self.readSiteFromDB()
        self.logger.debug("set siteId = '0' from 'loadSite'")

      self.logger.debug(">>> site before = " + varDump(self.site))
      if self.site is not None and self.batchItem.siteObj is not None:
        self.site.rewriteFields(self.batchItem.siteObj)
      self.logger.debug(">>> site after = " + varDump(self.site))
    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_LOAD_SITE_DATA)
      raise


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def readSiteFromDB(self):
    siteStatus = dc_event.SiteStatus(self.batchItem.siteId)
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.SITE_STATUS, siteStatus)
    responseDRCESyncTasksCover = self.dbWrapper.process(drceSyncTasksCoverObj)
    self.site = None
    self.siteTable = DC_SITES_TABLE_NAME
    try:
      self.site = responseDRCESyncTasksCover.eventObject
    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_READ_SITE_FROM_DB)
      raise
    #self.logger.debug("site object: " + varDump(self.site))


  # #load url
  # the site object to crawl
  # @param site - object to crawl
  def loadURL(self):
    # do siteId
    self.urlConnector = self.dcUrlsDbConnect
    if len(self.batchItem.siteId):
      self.urlTable = DC_URLS_TABLE_PREFIX + self.batchItem.siteId
    else:
      self.urlTable = DC_URLS_TABLE_PREFIX + "0"
    self.logger.debug("db backend Table for siteId %s is: %s" % (self.batchItem.siteId, self.urlTable))
    self.urlProcess.siteId = self.batchItem.siteId
    self.urlProcess.urlTable = self.urlTable
    self.url = self.urlProcess.readUrlFromDB(self.batchItem, self.batch.crawlerType, self.site.recrawlPeriod,
                                             self.auto_remove_props)
    if self.urlProcess.isUpdateCollection:
      self.updateCollectedURLs()


  # #resetVars
  def resetVars(self):
    self.needStoreMime = None  # pylint: disable=F0401,W0201
    self.site = None  # pylint: disable=F0401,W0201
    self.headersDict = None  # pylint: disable=F0401,W0201
    self.store_http_headers = None  # pylint: disable=F0401,W0201
    self.cookie = None  # pylint: disable=F0401,W0201
    self.proxies = None  # pylint: disable=F0401,W0201
    self.realUrl = None  # pylint: disable=F0401,W0201
    self.url = None
    self.crawledResource = None
    self.headers = None  # pylint: disable=F0401,W0201
    self.crawledTime = None  # pylint: disable=F0401,W0201
    self.storeHttpRequest = None  # pylint: disable=F0401,W0201
    self.dir = None  # pylint: disable=F0401,W0201
    self.kvConnector = None  # pylint: disable=F0401,W0201
    self.kvCursor = None  # pylint: disable=F0401,W0201
    self.processorName = None  # pylint: disable=F0401,W0201
    self.auto_remove_props = None  # pylint: disable=F0401,W0201
    self.store_cookies = None  # pylint: disable=F0401,W0201
    self.allow_http_redirects = True
    self.allow_html_redirects = True  # pylint: disable=F0401,W0201
    self.httpRedirects = 0  # pylint: disable=F0401,W0201
    self.htmlRedirects = 0  # pylint: disable=F0401,W0201
    self.max_http_redirects = CONSTS.MAX_HTTP_REDIRECTS_LIMIT  # pylint: disable=F0401,W0201
    self.max_html_redirects = CONSTS.MAX_HTML_REDIRECTS_LIMIT  # pylint: disable=F0401,W0201
    self.dom = None
    self.res = None  # pylint: disable=F0401,W0201
    self.postForms = None  # pylint: disable=F0401,W0201
    self.authName = None  # pylint: disable=F0401,W0201
    self.authPwd = None  # pylint: disable=F0401,W0201
    self.siteProperties = {}  # pylint: disable=F0401,W0201
    self.url_xpath_list = []  # pylint: disable=F0401,W0201
    self.errorMask = APP_CONSTS.ERROR_OK
    self.feed = None  # pylint: disable=F0401,W0201
    self.feedItems = None
    self.urlTempalteRegular = None  # pylint: disable=F0401,W0201
    self.urlTempalteRealtime = None  # pylint: disable=F0401,W0201
    self.urlTempalteRegularEncode = None  # pylint: disable=F0401,W0201
    self.urlTempalteRealtimeEncode = None  # pylint: disable=F0401,W0201
    self.detectModified = None  # pylint: disable=F0401,W0201
    self.collectURLsItems = []
    self.schemaBatchItems = []


  # #updateBatchItemAfterCarwling fills some field in the class fields objects after crawling
  def updateBatchItemAfterCarwling(self, status=dc_event.URL.STATUS_CRAWLED):
    self.urlProcess.urlObj = self.url
    self.urlProcess.siteId = self.batchItem.siteId
    self.logger.debug("set siteId = '" + str(self.urlProcess.siteId) + "' from 'updateBatchItemAfterCarwling'")

    self.urlProcess.batchItemUrlObj = self.batchItem.urlObj
    self.urlProcess.updateCrawledURL(self.crawledResource, self.batchItem, self.res.content_size, status)
    if status is not None:
      self.batchItem.urlObj.status = dc_event.URL.STATUS_CRAWLED
    self.batchItem.urlObj.crawlingTime = self.crawledResource.crawling_time
    self.batchItem.urlObj.errorMask |= self.crawledResource.error_mask
    if self.batchItem.urlObj.charset == "":
      self.batchItem.urlObj.charset = self.crawledResource.charset
    if self.batchItem.urlObj.httpCode == 0:
      self.batchItem.urlObj.httpCode = self.crawledResource.http_code
    if self.batchItem.urlObj.contentType == "":
      self.batchItem.urlObj.contentType = self.crawledResource.content_type
    self.batchItem.urlObj.crawled = 1
    if self.batchItem.urlObj.size == 0:
      self.batchItem.urlObj.size = self.res.content_size
    self.batchItem.urlObj.CDate = self.batchItem.urlObj.UDate = self.batchItem.urlObj.PDate = \
    datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


  # #process batch item
  # the batch item is the one from list of batch items within batch object
  # @param item - the batch item is the one from list of batch items within batch object
  def processBatchItem(self):
    self.resetVars()
    self.logger.info("Batch item: siteId: %s, urlId: %s" % (self.batchItem.siteId, self.batchItem.urlId))
    isResourceCrawled = False
    nextStep = True
    detectedMime = None

    try:
      # TODO move to - [SiteProcess]
      self.logger.debug("BatchItem: %s" % varDump(self.batchItem))
      self.logger.debug("self.batch.crawlerType: %s" % str(self.batch.crawlerType))
      self.loadSite()

      # prepare headers and cookie
      self.readSiteProperties()
      #self.batchItem = self.changeBatchItemByUrlSchema(self.batchItem, self.batch.id)
      self.setChainId(self.batchItem)
      if self.robotsParser is not None:
        self.robotsParser.initFiends(None,
        (bool(int(self.siteProperties['USE_ROBOTS_CACHE'])) if 'USE_ROBOTS_CACHE' in self.siteProperties else False),
        self.robotsFileDir)
      #Check if Real-Time crawling
      if self.batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        if self.batchItem.urlObj.urlPut is not None:
          self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
          # save tidy recovered file
          if "data" in self.batchItem.urlObj.urlPut.putDict:
            raw_unicode_content = self.batchItem.urlObj.urlPut.putDict["data"]
            localDom = None
            if self.htmlRecover is not None and self.htmlRecover == "2":
              localDom = self.resourceProcess.domParser(None, raw_unicode_content, 200,
                          self.crawledResource.charset if self.crawledResource is not None else None)
            if self.htmlRecover is not None and (self.htmlRecover == "1" or self.htmlRecover == "2" and\
              localDom is None):
              tidy_content = tidylib.tidy_document(raw_unicode_content, self.tidyOptions)[0]
              self.batchItem.urlObj.urlPut.putDict["data"] = base64.b64encode(tidy_content)
              self.logger.info(">>> Call tidy for ONLY_PROCESSING algorithm.")
          self.logger.info(">>> ONLY_PROCESSING algorithm. Skip batch item.")
        if self.batch.dbMode & dc_event.Batch.DB_MODE_W == 0:
          self.url = self.batchItem.urlObj  # pylint: disable=W0201
      else:
        self.loadURL()
      self.proxies = self.resolveProxy()  # pylint: disable=F0401,W0201
      # TODO move to - [SiteProcess]
      if not self.checkSite(self.url):
        self.logger.debug("Site check was failed. ")
        nextStep = False
        #self.batchItem = None
        #return self.batchItem

      if nextStep and not self.url:
        self.logger.debug("Can't load url. ")
        nextStep = False
        #self.batchItem = None
        #return self.batchItem

      #Check if Real-Time crawling resource already crawled
      if nextStep and self.batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER and self.url.crawled and \
        self.url.errorMask == APP_CONSTS.ERROR_OK and self.url.tagsCount > 0 and \
        self.errorMask == APP_CONSTS.ERROR_OK:
        self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty")
        self.logger.debug("Batch item properties: %s" % varDump(self.batchItem.properties))
        if PCONSTS.RECRAWL_KEY not in self.batchItem.properties or \
        self.batchItem.properties[PCONSTS.RECRAWL_KEY] == PCONSTS.RECRAWL_VALUE_NO:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty. " +
                            "Don't need to recrawl.")
          # set contentMask stored on disk
          self.batchItem.urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
          #return self.batchItem
          nextStep = False
        else:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty but " +
                            "properties recrawl is <%s>. Send to recrawling." % \
                            str(self.batchItem.properties[PCONSTS.RECRAWL_KEY]))
      if nextStep:
        # reset error mask for current crawled url
        self.urlProcess.siteId = self.batchItem.siteId
        self.urlProcess.resetErrorMask(self.batchItem)
        self.logger.debug("set siteId = '" + str(self.urlProcess.siteId) + "' from 'processBatchItem'")

        # update state to crawling
        # TODO move to - [UrlProcess]
        self.urlProcess.urlObj = self.url
        self.urlProcess.siteId = self.batchItem.siteId
        self.urlProcess.batchItemUrlObj = self.batchItem.urlObj
        self.urlProcess.updateURL(self.batchItem, self.batch.id)

        # crawl
        isResourceCrawled = self.crawl(self.batchItem.urlObj.urlPut.putDict["data"] if \
                                       (self.batchItem.urlObj.urlPut is not None and \
                                       self.batchItem.urlObj.urlPut.putDict is not None and "data" in \
                                       self.batchItem.urlObj.urlPut.putDict) else None)
        if self.crawledResource is not None and isResourceCrawled:
          # TODO move to - [SiteProcess]
          self.updateAVGSpeed()
          # Build DOM object
          #(6) DOM processing [???]
          self.dom = self.resourceProcess.domParser(self.htmlRecover,  # pylint: disable=W0201
                                          self.crawledResource.html_content, self.crawledResource.http_code,
                                          self.crawledResource.charset if self.crawledResource is not None else None)
          #(6) END
          self.getDir()
          try:
            if self.processorName != PCONSTS.PROCESSOR_FEED_PARSER:
              self.writeData()
            self.saveCookies()
            detectedMime = self.resourceProcess.mimeDetectByContent(self.crawledResource)
          except CrawlerException, cerr:
            self.logger.debug(cerr.message)
            raise
          except Exception as err:
            self.errorMask = self.errorMask | APP_CONSTS.ERROR_WRITE_FILE_ERROR
            self.updateURLForFailed(APP_CONSTS.ERROR_WRITE_FILE_ERROR)
            self.logger.error(MSG_ERROR_WRITE_CRAWLED_DATA + err.message, exc_info=True)
        else:
          nextStep = False
          self.logger.debug(">>> Wrong !!!! self.crawledResourc not set or isResourceCrawled is False")

      if nextStep:
        self.collectURLs()
        if self.batchItem.urlObj.chainId is not None and self.batchItem.urlObj.chainId in self.chainDict and \
        self.chainDict[self.batchItem.urlObj.chainId]["batchItem"].urlId == self.batchItem.urlId:
          self.updateBatchItemAfterCarwling(None)
          self.logger.debug(">>> ChainId, update URL without status")
        else:
          self.updateBatchItemAfterCarwling()
        # TODO move to - [UrlProcess]
        self.urlProcess.siteId = self.batchItem.siteId
        self.logger.debug("set siteId = '" + str(self.urlProcess.siteId) + "' from 'processBatchItem'")
        if self.res is not None:
          self.urlProcess.updateCollectTimeAndMime(detectedMime, self.batchItem, self.crawledTime, self.autoDetectMime,
                                                   self.res.headers, self.res.str_content)
        else:
          self.urlProcess.updateCollectTimeAndMime(detectedMime, self.batchItem, self.crawledTime, self.autoDetectMime)
      #(1) END
      #self.addResource()
    except CrawlerException as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_PROCESS_BATCH_ITEM, (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
    except Exception as err:
      self.exit_code = EXIT_FAILURE
      ExceptionLog.handler(self.logger, err, MSG_ERROR_PROCESS_BATCH_ITEM, (err))


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      self.logger.debug("len(input_pickle)=%i", len(input_pickled_object))
      input_batch = pickle.loads(input_pickled_object)
      app.Profiler.messagesList.append("Batch.id=" + str(input_batch.id))
      self.logger.info("Input batch id: %s, items: %s", str(input_batch.id), str(len(input_batch.items)))
      self.logger.debug("input_batch:\n" + varDump(input_batch))
      self.logger.debug("len before (batch_items)=%i", len(input_batch.items))
      self.regenerateBatchitems(input_batch)
      self.logger.debug("len after (batch_items)=%i", len(input_batch.items))
      self.batch = input_batch
      Utils.storePickleOnDisk(input_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.in." + str(self.batch.id))
      batch_items = []
      self.curBatchIterations = 1
      maxBatchIterations = input_batch.maxIterations
      #input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        #TODO: temporary set as 2 to test w/o support on client side
        #maxBatchIterations = 2
      self.logger.debug("maxBatchIterations=%s", str(maxBatchIterations))
      if self.batch.dbMode & dc_event.Batch.DB_MODE_W == 0:
        self.dbWrapper.affect_db = False
      while True:
        self.curBatchIterations += 1
        beforeItems = 0
        self.logger.debug("self.curBatchIterations=%s, beforeItems=%s", str(self.curBatchIterations), str(beforeItems))
        for batchItem in self.batch.items:
          self.logger.debug("batchItem.urlObj.status=%s", str(batchItem.urlObj.status))
          if batchItem.urlObj.status == dc_event.URL.STATUS_NEW or \
          batchItem.urlObj.status == dc_event.URL.STATUS_SELECTED_CRAWLING or \
          batchItem.urlObj.status == dc_event.URL.STATUS_SELECTED_CRAWLING_INCREMENTAL:
            self.errorMask = batchItem.urlObj.errorMask
            batchItem.urlObj.status = dc_event.URL.STATUS_CRAWLING
            self.batchItem = batchItem
            self.processBatchItem()
            self.fillChainUrlMD5List(self.batchItem)
            #Currently we update URL.ErrorMask for each url in batch
            self.updateUrlObjInBatchItem(batchItem.urlObj)
            if self.processorName != PCONSTS.PROCESSOR_RSS:
              if not self.batchItem:
                self.updateBatchItem(batchItem)
                input_batch.errorMask |= self.errorMask
              else:
                self.updateBatchItem(self.batchItem)
                batch_items.append(self.batchItem)
            # Extend self.store_items with new batch items if iterations >1 and not last iteration
            if maxBatchIterations > 1 and self.curBatchIterations <= maxBatchIterations:
              self.batchItemsExtendUnique(self.store_items, self.collectURLsItems)
            beforeItems += len(self.collectURLsItems)
        # Exit the batch cycling if no one new item added or max iterations reached
        if self.curBatchIterations > maxBatchIterations or beforeItems == 0:
          self.logger.debug("Exit from batching iteration:" + \
                            "self.curBatchIterations=%s, maxBatchIterations=%s, beforeItems=%s, self.store_items=%s",
                            str(self.curBatchIterations), str(maxBatchIterations), str(beforeItems),
                            str(len(self.store_items)))
          break
        else:
          self.batch.items = self.store_items
          self.logger.debug("Next batching iteration %s\n%s", str(self.curBatchIterations), varDump(self.store_items))

      if input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        process_task_batch = input_batch
        process_task_batch.items = self.store_items
      else:
        process_task_batch = Batch(input_batch.id, batch_items)
        process_task_batch.errorMask |= self.errorMask
      self.saveChainStorageData()
      self.logger.debug("batch_items len: " + str(len(process_task_batch.items)))
      self.logger.debug("output_batch:\n" + varDump(process_task_batch))
      self.logger.info("Output batch id: %s, items: %s", str(process_task_batch.id), str(len(process_task_batch.items)))
      output_pickled_object = pickle.dumps(process_task_batch)
      Utils.storePickleOnDisk(output_pickled_object, ENV_CRAWLER_STORE_PATH, "crawler.out." + str(self.batch.id))
      sys.stdout.write(output_pickled_object)
      sys.stdout.flush()
    except Exception, err:
      ExceptionLog.handler(self.logger, err, 'Crawler has failed.', (err))


  ##regenerateBatchitems methon added new items into the batch.items list
  #
  #@param batch - incoming batch
  def regenerateBatchitems(self, batch):
    additionBatchItems = []
    for batchItem in batch.items:
      self.batchItem = batchItem
      self.resetVars()
      self.loadSite()
      self.readSiteProperties()
      self.batchItem.siteProperties = copy.deepcopy(self.siteProperties)
      self.changeBatchItemByUrlSchema(self.batchItem, batch.id)
      additionBatchItems += self.schemaBatchItems
    tmpBatchItems = []
    for elem in additionBatchItems:
      if elem.urlId not in [e.urlId for e in tmpBatchItems]:
        tmpBatchItems.append(elem)
    batch.items += tmpBatchItems
    #TODO maxItems, maxItems <= len(batch.items)
    if batch.maxItems is not None and batch.maxItems < len(batch.items):
      batch.items = batch.items[0: self.batch.maxItems]
      batch.items[-1].urlObj.errorMask |= APP_CONSTS.ERROR_MAX_ITEMS


  ##changeBatchItemByUrlSchema generates new url bu urlSchema and replace batchItem.urlObject if url was change
  #
  #@param batchItem - incoming batchItem
  #@return batchItem with changed urlObj or untoched batchItem
  def changeBatchItemByUrlSchema(self, batchItem, batchId):
    #self.schemaBatchItems
    if "URLS_SCHEMA" in self.siteProperties:
      urlSchema = UrlSchema(self.siteProperties["URLS_SCHEMA"], batchItem.urlObj.siteId)
      newUrls = urlSchema.generateUrlSchema(batchItem.urlObj.url)
      if len(newUrls) > 0:
        self.logger.debug(">>> changeBatchItemByUrlSchema 0. Url was changed. From %s to %s", batchItem.urlObj.url,
                          newUrls[0])
        urlUpdateObj = dc_event.URLUpdate(batchItem.urlObj.siteId, batchItem.urlObj.url)
        urlUpdateObj.urlMd5 = batchItem.urlObj.urlMd5
        urlUpdateObj.batchId = batchId
        urlUpdateObj.crawled = 0
        urlUpdateObj.processed = 0
        urlUpdateObj.status = dc_event.URL.STATUS_CRAWLED
        result = self.dbWrapper.urlUpdate(urlUpdateObj)
        self.logger.debug(">>> changeBatchItemByUrlSchema 1. Url Update result: " + str(result))
        batchItem.urlObj.url = newUrls[0]
        batchItem.urlObj.urlMd5 = hashlib.md5(newUrls[0]).hexdigest()
        batchItem.urlId = batchItem.urlObj.urlMd5
        result = self.dbWrapper.urlNew([batchItem.urlObj])
        self.logger.debug(">>> changeBatchItemByUrlSchema 2. Url New result: " + str(result))
      if len(newUrls) > 1:
        for newUrl in newUrls[1:]:
          localBatchItem = copy.deepcopy(batchItem)
          localBatchItem.urlObj.url = newUrl
          localBatchItem.urlObj.urlMd5 = hashlib.md5(newUrl).hexdigest()
          localBatchItem.urlId = localBatchItem.urlObj.urlMd5
          self.schemaBatchItems.append(localBatchItem)
        if len(self.schemaBatchItems) > 0:
          result = self.dbWrapper.urlNew([elem.urlObj for elem in self.schemaBatchItems])
          self.logger.debug(">>> changeBatchItemByUrlSchema 3. Url New List result: " + str(result))
    return batchItem


  ##updateUrlObjInBatchItem update urlObj's fields in batchitem
  #
  #@param urlObj - reference to batchItem.urlObj
  #@param batchId - batch's id
  def updateUrlObjInBatchItem(self, urlObj):
    if urlObj is not None:
      urlObj.errorMask |= self.errorMask
      self.logger.debug(">>> Set error_mask = " + str(urlObj.errorMask))
      if self.crawledResource is not None:
        urlObj.httpCode = self.crawledResource.http_code
        self.logger.debug(">>> Set HTTP Code = " + str(self.crawledResource.http_code))
      if self.errorMask == 0:
        urlObj.batchId = self.batch.id


  # #updateBatchItem
  #
  def updateBatchItem(self, batchItem):
    #update batchItem properties
    # RSS feed items -> batch items
    if batchItem is not None:
      self.logger.debug("self.processorName=%s\nself.feedItems=%s", varDump(self.processorName),
                        varDump(self.feedItems))
      if self.processorName == PCONSTS.PROCESSOR_FEED_PARSER:
        if self.feedItems is not None and len(self.feedItems) > 0:
          batchItem = self.createBatchItemsFromFeedItems(batchItem)
        #self.batchItem.properties[dc_event.BatchItem.PROP_FEED] = self.feedItems
        #self.logger.debug("Batch property: %s" % varDump(self.batchItem.properties))
      elif self.url is not None:
        self.logger.debug("Before: batchItem urlObj errorMask: %s, url ErrorMask: %s" % (batchItem.urlObj.errorMask,
                                                                                          self.url.errorMask))
        batchItem.urlObj.errorMask |= self.url.errorMask
        batchItem.urlObj.errorMask |= self.errorMask
        self.logger.debug("After: batchItem urlObj errorMask: %s, url ErrorMask: %s" % (batchItem.urlObj.errorMask,
                                                                                      self.url.errorMask))
      if type(batchItem) == types.ListType:
        self.batchItemsExtendUnique(self.store_items, batchItem)
      else:
        self.batchItemsExtendUnique(self.store_items, [batchItem], False)
      if self.feedItems is not None:
        self.logger.debug(">>> feed_items = " + str(len(self.feedItems)))
      self.logger.debug(">>> store_items = " + str(len(self.store_items)))
    else:
      self.logger.debug(">>> wrong !!! updateBatchItem, batchItem is None")


  # #createBatchItems
  #
  def createBatchItemsFromFeedItems(self, parentBatchItem):
    items = []
    for elem in self.feedItems:
      urlMd5 = elem["urlMd5"]
      self.logger.debug("URLMD5: %s" % str(urlMd5))
      self.logger.debug("value: %s" % str(elem))
      siteId = self.batchItem.siteId
      urlObj = elem["urlObj"]

      #serialize content
      elem.pop("urlObj", None)

      # list of objects must be converted to string before json dumps
      dates = ["published_parsed", "updated_parsed"]
      for date in dates:
        if date in elem["entry"] and elem["entry"][date] is not None:
          elem["entry"][date] = strftime("%a, %d %b %Y %H:%M:%S +0000", elem["entry"][date])
      elem["urlMd5"] = urlMd5

      #store raw content on disk
      saveBatchItemUrlId = self.batchItem.urlId
      try:
        self.crawledResource = CrawledResource()  # pylint: disable=W0201
        self.crawledResource.binary_content = json.dumps(elem)
        self.batchItem.urlId = urlMd5
        self.getDir()
        self.writeData()
        self.batchItem.urlId = saveBatchItemUrlId
      except Exception, err:
        ExceptionLog.handler(self.logger, err, "Can't save object on disk. Reason:", (), \
                             {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        self.batchItem.urlId = saveBatchItemUrlId
        continue

      urlObj.contentMask = dc_event.URL.CONTENT_STORED_ON_DISK
      if parentBatchItem is not None:
        batchItem = copy.deepcopy(parentBatchItem)
        batchItem.siteId = siteId
        batchItem.urlId = urlMd5
        batchItem.urlObj = urlObj
        self.logger.debug("set siteId = '" + str(batchItem.siteId) + "' from 'createBatchItemsFromFeedItems'")
      else:
        batchItem = dc_event.BatchItem(siteId, urlMd5, urlObj)
      batchItem.urlObj.urlPut = None
      items.append(batchItem)
    return items


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load db backend
  # load from cli argument or default config file
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(className, self.DB_HOST_CFG_NAME)
      dbPort = int(self.config.get(className, self.DB_PORT_CFG_NAME))
      dbUser = self.config.get(className, self.DB_USER_CFG_NAME)
      dbPWD = self.config.get(className, self.DB_PWD_CFG_NAME)

      db_dc_sites = self.config.get(className, self.DB_SITES_CFG_NAME)
      db_dc_urls = self.config.get(className, self.DB_URLS_CFG_NAME)

      self.dc_sites_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_sites, dbPort, charset="utf8",
                                             use_unicode=True)
      self.dcUrlsDbConnect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_urls, dbPort, charset="utf8", use_unicode=True)
      # self.dc_sites_db_connect.set_character_set("utf8")
      # self.dcUrlsDbConnect.set_character_set("utf8")
      self.site_connector = self.dc_sites_db_connect
      self.urlConnector = self.dcUrlsDbConnect
      self.siteTable = None
      self.urlTable = None
    except Exception, err:
      ExceptionLog.handler(self.logger, err, "Error load config params:")
      raise


  # #loadKeyValueDB load key-value db
  #
  #
  def loadKeyValueDB(self):
    try:
      className = self.__class__.__name__
      self.kvDbDir = self.config.get(className, self.DB_DATA_DIR)
    except Exception, err:
      ExceptionLog.handler(self.logger, err, "Error load config params:")
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      self.logger = Utils.MPLogger().getLogger()
    except:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.raw_data_dir = self.config.get(self.__class__.__name__, "raw_data_dir")
      self.defaultHeaderFile = self.config.get(self.__class__.__name__, "headers_file")
      self.defaultCookieFile = self.config.get(self.__class__.__name__, "cookie_file")
      self.defaultIcrCrawlTime = self.config.getfloat(self.__class__.__name__, "default_icr_crawl_time")
      self.headerFileDir = self.config.get(self.__class__.__name__, "header_file_dir")
      self.robotsFileDir = self.config.get(self.__class__.__name__, "robots_file_dir")
      db_task_ini = self.config.get(self.__class__.__name__, "db-task_ini")

      if self.config.has_option(self.__class__.__name__, "useZeroSiteIdSiteNotExists"):
        try:
          self.useZeroSiteIdSiteNotExists = \
          bool(int(self.config.get(self.__class__.__name__, "useZeroSiteIdSiteNotExists")))
        except Exception:
          self.useZeroSiteIdSiteNotExists = False

      # Add support operations updateCollectedURLs and removeURLs
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(db_task_ini)
      self.dbWrapper = DBTasksWrapper(cfgParser)

      # does call collectAddtionalProp and collectProperties methods
      self.max_fetch_time = self.config.getint(self.__class__.__name__, "max_fetch_time")

      # keep old resources on disk
      if self.config.has_option(self.__class__.__name__, "keep_old_resources"):
        self.keep_old_resources = self.config.getboolean(self.__class__.__name__, "keep_old_resources")
      self.collect_additional_prop = self.config.getboolean(self.__class__.__name__, "collect_additional_prop")
      self.urlProcess = URLProcess.URLProcess()
      self.urlProcess.dbWrapper = self.dbWrapper
      self.resourceProcess = ResourceProcess.ResourceProcess()
      self.resourceProcess.dbWrapper = self.dbWrapper
    except Exception, err:
      ExceptionLog.handler(self.logger, err, "Error load config params:")
      raise


  # #checkResponse
  #
  def checkResponse(self):
    self.logger.debug("Requests response history: " + str(self.res.redirects))
    # calculate http redirect
    if HTTP_REDIRECT in self.res.redirects:
      self.httpRedirects += 1
      self.logger.debug("http redirect: " + str(self.httpRedirects))
      self.logger.debug("http max redirect: " + str(self.max_http_redirects))
    # check http redirect
    if self.max_http_redirects and self.max_http_redirects != MAX_HTTP_REDIRECTS_UNLIMITED and \
    self.httpRedirects >= self.max_http_redirects:
      self.allow_http_redirects = False
      self.logger.debug("http redirect limit was reached! Max http redirects: %s, encountered http redirects: %s." %
                        (str(self.max_http_redirects), str(self.httpRedirects)))
    else:
      self.allow_http_redirects = True  # pylint: disable=W0201


  # #calcLastModified
  #
  def calcLastModified(self, resource, res):
    if resource.http_code == 304:
      resource.last_modified = self.url.tcDate
    elif 'Last-Modified' in res.headers:
      resource.last_modified = res.headers['Last-Modified']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    elif 'Date' in res.headers:
      resource.last_modified = res.headers['Date']
      resource.last_modified = parse(resource.last_modified).strftime('%Y-%m-%d %H:%M:%S')
    else:
      resource.last_modified = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(time.time() - self.defaultIcrCrawlTime))
    self.logger.debug("LastModified date:" + str(resource.last_modified))
    return str(resource.last_modified)


  # #fillItemsFromIterations
  #
  # Optionally resets and fills the list of the BatchItems with the new instances initialized from the URLs objects list
  # @param urlObjects the list of the URL objects
  # @param siteObject the site object
  # @param reset the Boolean defines reset the items list before fill, True means reset
  def fillItemsFromIterations(self, urlObjects=None, siteObject=None, reset=True):
    if reset:
      self.collectURLsItems = []

    if urlObjects is not None:
      if siteObject is None:
        siteObjectLocal = self.batchItem.siteObj
        siteIdLocal = self.batchItem.siteId
      else:
        siteObjectLocal = siteObject
        siteIdLocal = siteObject.id

      for urlObject in urlObjects:
        self.logger.debug(">>> fillItemsFromIterations Md5 = " + urlObject.urlMd5)
        batchItem = dc_event.BatchItem(siteId=siteIdLocal, urlId=urlObject.urlMd5, urlObj=urlObject,
                                       siteObj=siteObjectLocal)
        batchItem.properties = self.batchItem.properties
        self.collectURLsItems.append(batchItem)


  # #batchItemsExtendUnique
  #
  # Fills the batch items list with unique items from another list
  # @param destinationBatchItems the destination BatchItem objects list
  # @param destinationBatchItems the source BatchItem objects list
  def batchItemsExtendUnique(self, destinationBatchItems, sourceBatchItems, lookIncomingBatch=True, urlType=1):
    for srcBatchItem in sourceBatchItems:
      inList = False
      if lookIncomingBatch:
        for dstBatchItem in self.batch.items:
          if srcBatchItem.siteId == dstBatchItem.siteId and srcBatchItem.urlId == dstBatchItem.urlId:
            self.logger.debug(">>> batchItemsExtendUnique baseItems duplicate " + srcBatchItem.urlId + " " +
                              dstBatchItem.urlId)
            inList = True
            break

      if not inList:
        for dstBatchItem in destinationBatchItems:
          if srcBatchItem.siteId == dstBatchItem.siteId and srcBatchItem.urlId == dstBatchItem.urlId:
            self.logger.debug(">>> batchItemsExtendUnique duplicate " + srcBatchItem.urlId + " " + dstBatchItem.urlId)
            inList = True
            break
      if not inList:
        self.logger.debug(">>> batchItemsExtendUnique added " + srcBatchItem.urlId)
        srcBatchItem.urlObj.type = urlType
        #TODO maxItems, maxItems <= len(destinationBatchItems)
        if self.batch.maxItems is not None and self.batch.maxItems == len(destinationBatchItems):
          destinationBatchItems = destinationBatchItems[0: self.batch.maxItems]
          destinationBatchItems[-1].urlObj.errorMask |= APP_CONSTS.ERROR_MAX_ITEMS
          self.logger.debug(">>> Set Error Mask=" + str(destinationBatchItems[-1].urlObj.errorMask))
        else:
          destinationBatchItems.append(srcBatchItem)


  # #updateCollectedURLs
  #
  def updateCollectedURLs(self):
    self.logger.debug("updateCollectedURLs")
    self.dbWrapper.fieldRecalculating(Utils.autoFillSiteId(self.batchItem.siteId, self.logger))


#---> Chain processing methods ----------------------------------------
  # #setChainId
  #
  # @param batchItem incoming batch item
  def setChainId(self, batchItem):
    self.logger.debug(">>> batchItem start")
    if batchItem.urlObj.chainId is None and "URL_CHAIN" in self.siteProperties and \
    self.siteProperties["URL_CHAIN"] is not None:
      batchItem.urlObj.chainId = self.chainIndex
      self.chainIndex += 1


  # #fillItemsFromIterationsWithChain
  #
  # @param urlObjects the list of the URL objects
  def fillItemsFromIterationsWithChain(self, urlObjects, batchItem):
    if urlObjects is not None and len(urlObjects) > 0:
      self.logger.debug(">>> fillItemsFromIterationsWithChain start")
      if batchItem.urlObj.chainId not in self.chainDict:
        self.chainDict[batchItem.urlObj.chainId] = {}
        self.chainDict[batchItem.urlObj.chainId]["batchItem"] = batchItem
        self.chainDict[batchItem.urlObj.chainId]["chainUrlMD5List"] = []
        self.logger.debug(">>> insert for urlMd5 = " + str(self.chainDict[batchItem.urlObj.chainId]["batchItem"].urlId))

      for urlObj in urlObjects:
        urlObj.chainId = batchItem.urlObj.chainId
      self.fillItemsFromIterations(urlObjects, None, False)


  # #fillChainUrlMD5List
  #
  # @param batchItem incoming batch item
  def fillChainUrlMD5List(self, batchItem):
    if batchItem.urlObj.chainId is not None and batchItem.urlObj.chainId in self.chainDict:
      self.logger.debug(">>> fillChainUrlMD5List start")
      localChainElem = self.chainDict[batchItem.urlObj.chainId]
      if batchItem.urlObj.urlMd5 != localChainElem["batchItem"].urlObj.urlMd5 and \
      batchItem.urlObj.urlMd5 not in localChainElem["chainUrlMD5List"]:
        localChainElem["chainUrlMD5List"].append(batchItem.urlObj.urlMd5)


  # #saveChainStorageData
  #
  def saveChainStorageData(self):
    urlPutList = []
    self.logger.debug(">>> saveChainStorageData start")
    for localChainKay in self.chainDict:
      localChainElem = self.chainDict[localChainKay]
      saveBuf = '\n'.join(localChainElem["chainUrlMD5List"])
      saveBuf = saveBuf.strip()
      putDict = {"data": base64.b64encode(saveBuf)}
      urlPutObj = dc_event.URLPut(localChainElem["batchItem"].siteId, localChainElem["batchItem"].urlObj.urlMd5, \
                                  dc_event.Content.CONTENT_CHAIN_PARTS, putDict)
      urlPutList.append(urlPutObj)
      if len(urlPutList) > 0:
        self.dbWrapper.putURLContent([urlPutObj])

      self.urlProcess.siteId = localChainElem["batchItem"].siteId
      self.logger.debug("set siteId = '" + str(self.urlProcess.siteId) + "' from 'saveChainStorageData'")
      self.urlProcess.updateURLStatus(localChainElem["batchItem"])
      self.logger.debug(">>> delay update for urlMd5 = " + str(localChainElem["batchItem"].urlId))
