"""
@package: dc
@file URLProcess.py
@author Scorp <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import datetime
import hashlib
import logging
import MySQLdb as mdb
import re
import requests
import time
import urllib
import urlparse
import json
import CrawlerTask
import app.Utils as Utils
import app.Consts as APP_CONSTS
import dc.EventObjects
import dc.Constants as DC_CONSTS
import dc_processor.Constants as PCONSTS
from app.Filters import Filters
from dc_crawler.CrawlerException import CrawlerException
from app.Utils import SQLExpression
from time import strftime
from dtm.EventObjects import GeneralResponse
from app.Utils import ExceptionLog

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

class URLProcess(object):

  DC_URLS_TABLE_PREFIX = "urls_"
  DETECT_MIME_TIMEOUT = 1
  PATTERN_WITH_PROTOCOL = re.compile('[a-zA-Z]+:(//)?')
  URL_TEMPLATE_CONST = "%URL%"
  PROTOCOL_PREFIX = "://"
  DEFAULT_PROTOCOLS = ["http", "https"]
  DETECT_MIME_MAIN_CONTENT = "1"


  def __init__(self, protocols=None):
    self.isUpdateCollection = False
    self.urlObj = None
    self.batchItemUrlObj = None
    self.url = None
    self.dbWrapper = None
    self.siteId = None
    self.urlTable = None
    self.protocolsList = self.DEFAULT_PROTOCOLS
    self.setProtocols(protocols)


  def setProtocols(self, protocols=None):
    if protocols is not None:
      try:
        self.protocolsList = json.loads(protocols)
      except Exception:
        self.protocolsList = []
    logger.debug(">>> THAT PROTOCOLS = " + str(self.protocolsList))


  def checkUrlByPath(self, url):
    ret = False
    position = url.find(self.PROTOCOL_PREFIX)
    if position > 0 and url.find('/') == (position + 1):
      ret = True
    return ret


  # #checkUrlByProtocol finds url's protocol in allowed protocols list
  #
  def checkUrlByProtocol(self, url):
    ret = False
    for elem in self.protocolsList:
      if url.lower().startswith(elem + self.PROTOCOL_PREFIX):
        ret = True
        break
    if not ret:
      logger.debug(">>> URL skiped by protocol = " + url)
    return ret


  # #checkClassFields method checks all class's mandatory fields
  #
  def checkClassFields(self, checkList):
    for field in self.__dict__:
      if field in checkList and (not hasattr(self, field) or getattr(self, field) is None):
        raise Exception(">>> [CollectURLs] Mandatory field must be initialized, field Name = " + field)


  def resolveTableName(self, localSiteId):
    self.urlTable = self.DC_URLS_TABLE_PREFIX + localSiteId
    return self.urlTable


  @staticmethod
  def createURLObj(siteId, url):
    ret = dc.EventObjects.URL(siteId, mdb.escape_string(url))
    return ret


  # #readCurrentCnt return urls cnt from db
  #
  def readCurrentCnt(self, maxURLs):
    self.checkClassFields(["dbWrapper", "urlTable"])
    currentCnt = 0
    if maxURLs:
      countsql = "SELECT COUNT(*) AS cnt FROM `%s` WHERE NOT (Status=4 AND Crawled=0 AND Processed=0)" % \
                  (self.urlTable,)
      result = self.dbWrapper.customRequest(countsql, CrawlerTask.DB_URLS)
      if result is not None and len(result) > 0 and len(result[0]) > 0:
        currentCnt = result[0][0]
      else:
        currentCnt = 0
    return currentCnt


  def simpleURLCanonize(self, realUrl):
    self.checkClassFields(["url"])
    if not self.checkUrlByPath(self.url):
      self.url = urlparse.urljoin(realUrl, self.url)
      # normalization
    #retUrl = dc.EventObjects.URL(0, self.url).getURL()
    retUrl = Utils.UrlNormalizator.normalize(self.url, self.protocolsList)
    return retUrl


  # #processURL process incoming url and added it to the one of incoming lists [internalLinks or externalLinks]
  #
  def processURL(self, realUrl, internalLinks, externalLinks, filtersApply=None, siteFilters=None):
    self.checkClassFields(["urlObj", "dbWrapper", "siteId", "url"])
    retUrl = None
    retContinue = False

    logger.debug("URL: %s", self.url)

    if self.urlObj.type == dc.EventObjects.URL.TYPE_SINGLE or not self.url:
      logger.debug("URL type is TYPE_SINGLE - not collect urls. Skip url.")
      retContinue = True

    #TODO: must be replaced with more general check on supported protocols list for this site
    '''
    if not retContinue and self.PATTERN_WITH_PROTOCOL.match(self.url) and not self.checkUrlByProtocol(self.url):
      retContinue = True
    '''
    if not retContinue:
      #if (filtersApply(None, self.url, 0, self.dbWrapper, self.siteId, Filters.STAGE_COLLECT_URLS_PROTOCOLS) \
          #if filtersApply is not None else not self.checkUrlByPath(self.url)):
      if not self.checkUrlByPath(self.url):
        self.url = urlparse.urljoin(realUrl, self.url)
        # normalization
      #retUrl = dc.EventObjects.URL(0, self.url).getURL()
      retUrl = Utils.UrlNormalizator.normalize(self.url, self.protocolsList)
      if Utils.UrlNormalizator.isNormalUrl(retUrl):

        if siteFilters is not None:
          localFilters = Filters(siteFilters, self.dbWrapper, self.siteId)
        else:
          localFilters = Filters(None, self.dbWrapper, self.siteId)

        #if (filtersApply(None, retUrl, 0, self.dbWrapper, self.siteId, Filters.STAGE_COLLECT_URLS_PROTOCOLS) \
            #if filtersApply is not None else not self.checkUrlByProtocol(retUrl)):
        #if self.checkUrlByProtocol(retUrl):
        if filtersApply is not None:
          protocolAllowed = filtersApply(localFilters, retUrl, 0, self.dbWrapper, self.siteId, \
                                         Filters.STAGE_COLLECT_URLS_PROTOCOLS)
        if (filtersApply is None) or (localFilters.searchFiltersWithStage(Filters.STAGE_COLLECT_URLS_PROTOCOLS) == 0):
          protocolAllowed = self.checkUrlByProtocol(retUrl)
        if protocolAllowed:
          if Utils.parseHost(retUrl) == Utils.parseHost(realUrl):
            internalLinks.append(retUrl)
            logger.debug("URL classified as internal")
          elif Utils.parseHost(retUrl):
            externalLinks.append(retUrl)
            logger.debug("URL classified as external")
          else:  # not valid url like http://
            retContinue = True
        else:
          retContinue = True
      else:
        logger.debug(">>> Bad url normalization, url = " + retUrl)
        retContinue = True
    return retUrl, retContinue


  # #isUrlExist slelect localUrl from dc_urls db if it exist
  #
  def isUrlExist(self, recrawlPeriod, siteProperties, urlMd5):
    self.checkClassFields(["url", "dbWrapper", "siteId", "urlTable"])
    ret = False
    if "RECRAWL_URL_AGE_EXPRESSION" in siteProperties and siteProperties["RECRAWL_URL_AGE_EXPRESSION"] != "":
      ageExpr = siteProperties["RECRAWL_URL_AGE_EXPRESSION"].replace("%RECRAWL_PERIOD%", str(recrawlPeriod))
    else:
      ageExpr = "(DATE_ADD(UDate, INTERVAL %s MINUTE)-NOW())" % (str(recrawlPeriod))
    query = "SELECT COUNT(*), %s, `Type` FROM `%s` WHERE `URLMd5` = '%s'" % (ageExpr, self.urlTable, urlMd5)
    result = self.dbWrapper.customRequest(query, CrawlerTask.DB_URLS)
    if result is not None and len(result) > 0 and len(result[0]) > 0 and result[0][0] > 0:
      if recrawlPeriod == 0 or result[0][1] > 0 or result[0][2] == dc.EventObjects.URL.TYPE_FETCHED or \
      ("RECRAWL_NO_ROOT_URLS" in siteProperties and siteProperties["RECRAWL_NO_ROOT_URLS"] == "0"):
        logger.debug("URL skipped, exists and re-crawling not active, time not reached or URL Type is " +
                     "RSS feed (not to fetch)\n %s %s", self.url, urlMd5)
        ret = True
      else:
        self.updateURLFields(urlMd5, self.dbWrapper, self.siteId, siteProperties)
        logger.debug("URL state updated to NEW because re-crawling\n %s %s", self.url, urlMd5)
        ret = True
    else:
      logger.debug("URL %s treated as new\n %s", self.url, urlMd5)

    return ret


  # #update some fields in dc_urls
  #
  #urlMd5 param - urls md5
  #wrapper param - db_db wrapper
  #siteId param - site's id
  def updateURLFields(self, urlMd5, wrapper, siteId, siteProperties):
    urlUpdateObj = dc.EventObjects.URLUpdate(siteId, urlMd5, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    #Status field
    if "RECRAWL_URL_UPDATE_STATUS" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_STATUS"] != "-1":
      urlUpdateObj.status = int(siteProperties["RECRAWL_URL_UPDATE_STATUS"])
    else:
      if "RECRAWL_URL_UPDATE_STATUS" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_STATUS"] == "-1":
        urlUpdateObj.status = None
      else:
        urlUpdateObj.status = dc.EventObjects.URL.STATUS_NEW
    if self.batchItemUrlObj is not None:
      self.batchItemUrlObj.status = urlUpdateObj.status
    #TcDate field
    if "RECRAWL_URL_UPDATE_TCDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_TCDATE"] != "":
      urlUpdateObj.tcDate = siteProperties["RECRAWL_URL_UPDATE_TCDATE"]
    else:
      if "RECRAWL_URL_UPDATE_TCDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_TCDATE"] == "":
        urlUpdateObj.tcDate = None
      else:
        urlUpdateObj.tcDate = SQLExpression("NOW()")
    #CDate field
    if "RECRAWL_URL_UPDATE_CDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_CDATE"] != "":
      urlUpdateObj.CDate = siteProperties["RECRAWL_URL_UPDATE_CDATE"]
    #UDate field
    if "RECRAWL_URL_UPDATE_UDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_UDATE"] != "":
      urlUpdateObj.UDate = siteProperties["RECRAWL_URL_UPDATE_UDATE"]
    else:
      if "RECRAWL_URL_UPDATE_UDATE" in siteProperties and siteProperties["RECRAWL_URL_UPDATE_UDATE"] == "":
        urlUpdateObj.UDate = None
      else:
        urlUpdateObj.UDate = SQLExpression("NOW()")

    wrapper.urlUpdate(urlUpdateObj, "`State`=0")



  # # detect mime type for an URL using HEAD method
  #
  # @return mime_type detected mime type, empty string if failed
  def detectUrlMime(self):
    self.checkClassFields(["url"])
    ret = ''
    try:
      res = requests.head(self.url, timeout=self.DETECT_MIME_TIMEOUT)
      ret = res.headers.get('content-type', '').lower()
    except Exception:
      logger.warn("detect mime type for %s failed", self.url, exc_info=True)
    return ret


  # #getDepthFromUrl method
  #
  #urlMd5 - url's md5
  def getDepthFromUrl(self, urlMd5):
    self.checkClassFields(["dbWrapper", "siteId"])
    ret = 0
    urlStatusObj = dc.EventObjects.URLStatus(self.siteId, urlMd5)
    result = self.dbWrapper.urlStatus(urlStatusObj, True)
    if len(result) > 0 and type(result[0]) == dc.EventObjects.URL:
      ret = result[0].depth
    return ret


  # #updateURLForFailed
  #
  # @param errorBit BitMask of error
  # @param batchItem incoming batchItem instance
  def updateURLForFailed(self, errorBit, batchItem, httpCode=None, status=dc.EventObjects.URL.STATUS_CRAWLED,
                         updateUdate=True):
    self.checkClassFields(["dbWrapper", "siteId"])
    logger.debug(">>> errorBit = " + str(errorBit))
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = status
    if self.batchItemUrlObj is not None:
      self.batchItemUrlObj.status = urlUpdateObj.status
    urlUpdateObj.errorMask = errorBit
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    if updateUdate:
      urlUpdateObj.UDate = SQLExpression("NOW()")
    if httpCode is not None:
      urlUpdateObj.httpCode = httpCode
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #getRealUrl
  #
  def getRealUrl(self):
    self.checkClassFields(["url"])
    if self.url.startswith("http%3A") or self.url.startswith("https%3A"):
      ret = urllib.unquote(self.url.url).decode('utf-8')
    else:
      ret = self.url.decode('utf8')
    return ret


  # #resolveHTTP method resolves http method, and fill headersDict["If-Modified-Since"] element
  #
  # @param postForms incoming post forms
  # @param headersDict incoming headersDict dict, where filled element with "If-Modified-Since" key
  # return None or filled postData
  def resolveHTTP(self, postForms, headersDict):
    self.checkClassFields(["urlObj"])
    postData = None
    try:
      method = self.urlObj.httpMethod.lower()
    except Exception:
      method = "get"
    if method == "post":
      postData = postForms
      logger.debug("use post, post_data:%s", postData)
    else:
      logger.debug("last modified: <<%s>>", str((self.urlObj.lastModified)))
      if str(self.urlObj.lastModified) != "None" and str(self.urlObj.lastModified) != "NULL":
        logger.debug("If-Modified-Since: <<%s>>", self.urlObj.lastModified)
        headersDict["If-Modified-Since"] = \
        Utils.convertToHttpDateFmt(datetime.datetime.strptime(str(self.urlObj.lastModified), "%Y-%m-%d %H:%M:%S"))
    return postData


  # #updateCrwledURL
  #
  # @param site - site object
  def updateCrawledURL(self, crawledResource, batchItem, contentSize, status=dc.EventObjects.URL.STATUS_CRAWLED):
    self.checkClassFields(["urlObj", "dbWrapper", "siteId"])
    logger.debug(">>> Start urls update")
    state = dc.EventObjects.URL.STATE_ENABLED
    updatedCount = self.urlObj.mRate * self.urlObj.mRateCounter
    if crawledResource.http_code != 304:
      updatedCount += 1
    mrate = updatedCount / (self.urlObj.mRateCounter + 1)
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.status = status
    if self.batchItemUrlObj is not None:
      self.batchItemUrlObj.status = urlUpdateObj.status
    urlUpdateObj.state = state
    urlUpdateObj.contentType = crawledResource.content_type
    urlUpdateObj.charset = crawledResource.charset
    urlUpdateObj.errorMask = crawledResource.error_mask
    urlUpdateObj.crawlingTime = crawledResource.crawling_time
    urlUpdateObj.totalTime = crawledResource.crawling_time
    urlUpdateObj.UDate = SQLExpression("NOW()")
    urlUpdateObj.httpCode = crawledResource.http_code
    urlUpdateObj.lastModified = crawledResource.last_modified
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.mRate = mrate
    urlUpdateObj.size = contentSize
    urlUpdateObj.mRateCounter = SQLExpression("`MRateCounter` + 1")
    updatedRowsCount = self.dbWrapper.urlUpdate(urlUpdateObj, "`Status` = 3")
    return updatedRowsCount > 0


  # #updateURL
  #
  # @param batchItem - incoming batchItem
  # @param batchId - base batchId
  # @param status - value for status field updating
  def updateURL(self, batchItem, batchId, status=dc.EventObjects.URL.STATUS_CRAWLING):
    self.checkClassFields(["urlObj", "dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    if not self.urlObj.httpMethod:
      urlUpdateObj.httpMethod = "get"
    else:
      urlUpdateObj.httpMethod = self.urlObj.httpMethod
    urlUpdateObj.status = status
    if self.batchItemUrlObj is not None:
      self.batchItemUrlObj.status = urlUpdateObj.status
    urlUpdateObj.crawled = SQLExpression("`Crawled`+1")
    urlUpdateObj.batchId = batchId
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    updatedRowsCount = self.dbWrapper.urlUpdate(urlUpdateObj)
    return updatedRowsCount > 0


  # #updateURLStatus
  #
  # @param batchItem - incoming batchItem
  # @param status - value for status field updating
  def updateURLStatus(self, batchItem, status=dc.EventObjects.URL.STATUS_CRAWLED):
    updatedRowsCount = 0
    if status is not None:
      self.checkClassFields(["siteId"])
      urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.status = status
      updatedRowsCount = self.dbWrapper.urlUpdate(urlUpdateObj)
    return updatedRowsCount > 0


  # #resetErrorMask reset url's errorMask, using db-task wrapper
  #
  # @param batchItem - incoming batchItem
  def resetErrorMask(self, batchItem):
    self.checkClassFields(["dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.errorMask = 0
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #addURLFromBatchToDB inserts new URL in to the db with auto-removing
  #
  # @param batchItem - incoming BatchItem object
  # @param crawlerType - url's crawlingType, we have used for internal checks
  # @param recrawlPeriod - url's recrawlPeriod, we have used for internal checks
  # @param autoRemoveProps - param send into URLProcess.autoRemoveURL method call
  def addURLFromBatchToDB(self, batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
    self.checkClassFields(["dbWrapper", "siteId", "urlTable"])
    ret = True
    try:
      siteStatusObj = dc.EventObjects.SiteStatus(Utils.autoFillSiteId(self.siteId, logger))
      result = self.dbWrapper.siteStatus(siteStatusObj)
      if result is not None:
        maxURLs = result.maxURLs
        if ((crawlerType != dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER) and \
        (result.state != dc.EventObjects.Site.STATE_ACTIVE)) or \
        ((crawlerType == dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER) and \
        (result.state == dc.EventObjects.Site.STATE_DISABLED)):
          logger.debug("Warning: Batch CrawlerType: %s, site state is %s but not STATE_ACTIVE!", crawlerType,
                       str(result.state))
          raise CrawlerException("Site state is not active, state=" + str(result.state))

        if (result.maxErrors > 0) and (result.errors > result.maxErrors):
          msg = "Site maxErrors limit " + str(result.maxErrors) + " reached " + str(result.errors)
          logger.debug(msg)
          raise CrawlerException(msg)

        #Check the limit of the maxURLs for count of active URLs (not migrated with batches)
        if DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE_ACTIVE in autoRemoveProps:
          where = autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE_ACTIVE]
        else:
          where = "NOT (`Status`=4 AND `Crawled`=0 AND `Processed`=0)"
        query = "SELECT COUNT(*) FROM `%s` " % self.urlTable
        query += "WHERE " + where
        result = self.dbWrapper.customRequest(query, CrawlerTask.DB_URLS)
        if len(result) > 0 and len(result[0]) > 0:
          activeURLs = result[0][0]
          logger.debug("Active URLs count: " + str(activeURLs) + ", maxURLs: " + str(maxURLs))
          if (maxURLs > 0) and (activeURLs >= maxURLs):
            autoRemoved = URLProcess.autoRemoveURL(autoRemoveProps, recrawlPeriod, self.urlTable, self.dbWrapper)
            if autoRemoved < 1:
              msg = "Active URLs:" + str(activeURLs) + " > MaxURLs:" + str(maxURLs) + " and no one auto-removed!"
              logger.debug(msg)
              raise CrawlerException(msg)
            else:
              logger.debug(str(autoRemoved) + " URLs auto-removed to insert new URL from batch")
        else:
          msg = "Error of query processing, no rows returned:\n" + query
          logger.debug(msg)
          raise CrawlerException(msg)

        batchItem.urlObj.CDate = str(datetime.datetime.now())
        batchItem.urlObj.UDate = batchItem.urlObj.CDate
        batchItem.urlObj.tcDate = batchItem.urlObj.CDate
        batchItem.urlObj.batchId = 0  #self.batch.id
        result = self.dbWrapper.urlNew([batchItem.urlObj])
        logger.debug("rows_count: %s", result)
        self.isUpdateCollection = True
        #self.updateCollectedURLs()
      else:
        raise CrawlerException("Execute last SQL query(SiteStatus), no rows returned:\n")
    except CrawlerException as err:
      ExceptionLog.handler(logger, err, 'Error add new url from batch (another host source):')
      ret = False
    except Exception as err:
      ExceptionLog.handler(logger, err, 'Error add new url from batch (another host source):', (err))
      ret = False
      raise err
    return ret


  # #Auto remove resources
  # Removes resources in condition based on dc_sites.sites_properties items
  #
  # @param autoRemoveProps - list of autoremove properties
  # @param recrawlPeriod - used only for replacing in autoRemoveProps field
  # @param urlTable - currently table in dc_urls db
  # @param wrapper - db_task wrapper
  # @return number of URLs
  @staticmethod
  def autoRemoveURL(autoRemoveProps, recrawlPeriod, urlTable, wrapper):
    ret = 0
    try:
      #logger.debug("Auto remove properties:\n%s", varDump(autoRemoveProps))
      #If defined auto remove properties and set in proper values
      if DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE] != "" and\
        DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER in autoRemoveProps and\
        autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER] != "":
        # Select candidates to remove
        #TODO: CHANGE -> (Custom request)
        query = "SELECT Site_Id, URLMd5 FROM %s WHERE %s ORDER BY %s LIMIT %s" % (urlTable,
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_WHERE].replace("%RecrawlPeriod%", str(recrawlPeriod)),
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_ORDER],
                autoRemoveProps[DC_CONSTS.SITE_PROP_AUTO_REMOVE_RESOURCES])
        logger.debug("SQL to select auto remove candidates: %s", query)
        result = wrapper.customRequest(query, CrawlerTask.DB_URLS)
        if len(result) > 0:
          urlsToDelete = []
          for row in result:
            # Create new URLDelete object
            urlDelete = dc.EventObjects.URLDelete(row[0], row[1], dc.EventObjects.URLStatus.URL_TYPE_MD5,
                                                  reason=dc.EventObjects.URLDelete.REASON_CRAWLER_AUTOREMOVE)
            urlsToDelete.append(urlDelete)
            logger.debug("URL added to auto remove URLMd5:[%s]", row[1])
          drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, urlsToDelete)
          responseDRCESyncTasksCover = wrapper.process(drceSyncTasksCoverObj)
          logger.debug("Response from db-task module on URLDelete operation:\n%s", \
                            Utils.varDump(responseDRCESyncTasksCover))
          deleted = 0
          if isinstance(responseDRCESyncTasksCover, DC_CONSTS.DRCESyncTasksCover):
            #a = DC_CONSTS.DRCESyncTasksCover.eventObject
            #a = DC_CONSTS.DRCESyncTasksCover["eventObject"]
            #generalResponse = responseDRCESyncTasksCover["eventObject"]
            #generalResponse = responseDRCESyncTasksCover[1]
            generalResponse = responseDRCESyncTasksCover.eventObject
            if isinstance(generalResponse, GeneralResponse):
              for code in generalResponse.statuses:
                if code:
                  deleted += 1
          #ret = len(result)
          ret = deleted
        else:
          logger.debug("No auto remove candidates or SQL query error!")
      else:
        logger.debug("No mandatory auto remove properties in auto_remove_props:\n" + Utils.varDump(autoRemoveProps))
    except Exception as err:
      ExceptionLog.handler(logger, err, 'Error of auto remove operation:')
    return ret


  # #updateCollectTimeAndMime
  # add time cost of collect url to URL.CrawlingTime
  # and update Detected MIME type
  #
  # @param detectedMime - mine type from current page
  # @param batchItem - incoming batch item
  # @param crawledTime - datatime used for filling some fields
  # @param autoDetectMime - used for check detectedMime param before contentType field updating
  def updateCollectTimeAndMime(self, detectedMime, batchItem, crawledTime, autoDetectMime, httpHeaders=None,
                               strContent=None):
    self.checkClassFields(["dbWrapper", "siteId"])
    if crawledTime is not None:
      collectTime = int((time.time() - crawledTime) * 1000)
    else:
      collectTime = 0
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    if strContent is not None:
      urlUpdateObj.rawContentMd5 = hashlib.md5(strContent).hexdigest()
    urlUpdateObj.crawlingTime = SQLExpression(("`CrawlingTime` + %s" % str(collectTime)))
    urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(collectTime)))
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.UDate = SQLExpression("NOW()")
    logger.debug(">>> detectMime = " + str(detectedMime))
    if httpHeaders is not None:
      for header in httpHeaders:
        if header.lower() == "etag":
          urlUpdateObj.eTag = httpHeaders[header]
    if detectedMime is not None and autoDetectMime == self.DETECT_MIME_MAIN_CONTENT:
      urlUpdateObj.contentType = str(detectedMime)
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #readUrlFromDB read url object from database
  #
  # @param batchItem - incoming batch item
  # @param crawlerType - url's crawlingType, we have used for internal checks
  # @param recrawlPeriod - url's recrawlPeriod, we have used for internal checks
  # @param autoRemoveProps - list of autoremove properties
  def readUrlFromDB(self, batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
    self.urlObj = None
    self.checkClassFields(["dbWrapper", "siteId"])
    self.isUpdateCollection = False
    urlStatus = dc.EventObjects.URLStatus(self.siteId, batchItem.urlId)
    urlStatus.urlType = dc.EventObjects.URLStatus.URL_TYPE_MD5
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_STATUS, [urlStatus])
    responseDRCESyncTasksCover = self.dbWrapper.process(drceSyncTasksCoverObj)
    row = responseDRCESyncTasksCover.eventObject
    logger.debug("responseDRCESyncTasksCover: %s", row)
    try:
      if len(row) > 0:
        self.urlObj = row[0]
        #self.logger.debug("lastModified: %s", str(row[0]['LastModified']))
      else:
        #When url come from another dc cluster's host it is not present in the db
        if self.addURLFromBatchToDB(batchItem, crawlerType, recrawlPeriod, autoRemoveProps):
          self.readUrlFromDB(batchItem, crawlerType, recrawlPeriod, autoRemoveProps)
        else:
          msg = "Can't add url from batch."
          logger.debug(msg)
          raise CrawlerException(msg)
    except CrawlerException as err:
      ExceptionLog.handler(logger, err, "Can't load url from db:", (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
      raise
    logger.debug("Loaded url object: " + str(self.urlObj))

    return self.urlObj



  # #updateAdditionProps update some fields for URL (makes URLUpdate operation)
  #
  # @param internalLinksCount - count of internal links from page
  # @param externalLinksCount - count of enternal links from page
  # @param batchItem - incoming batch item
  # @param size - value for .size field update
  # @param freq - value for .freq field update
  # @param contentMd5 - value for .contentMd5 field update
  def updateAdditionProps(self, internalLinksCount, externalLinksCount, batchItem, size, freq, contentMd5):
    self.checkClassFields(["dbWrapper", "siteId"])
    urlUpdateObj = dc.EventObjects.URLUpdate(self.siteId, batchItem.urlId, dc.EventObjects.URLStatus.URL_TYPE_MD5)
    urlUpdateObj.tcDate = SQLExpression("NOW()")
    urlUpdateObj.size = size
    urlUpdateObj.linksI = internalLinksCount
    urlUpdateObj.linksE = externalLinksCount
    urlUpdateObj.freq = freq
    urlUpdateObj.rawContentMd5 = contentMd5
    self.dbWrapper.urlUpdate(urlUpdateObj)


  # #createUrlObjForCollectURLs - creates and returns urlObj for collectedURL operation
  #
  # @param urlMd5 - url's md5
  # @param formMethods - formMethods post/get form data
  # @param urlId - parent URL md5
  # @param depth - url's depth in site structure
  # @param detectedMime -
  # @param maxURLsFromPage
  def createUrlObjForCollectURLs(self, urlMd5, formMethods, parentMd5, depth, detectedMime, maxURLsFromPage):
    self.checkClassFields(["url", "siteId", "urlObj"])
    ret = dc.EventObjects.URL(self.siteId, mdb.escape_string(self.url))
    ret.type = self.urlObj.type
    ret.urlMd5 = urlMd5
    ret.requestDelay = self.urlObj.requestDelay
    ret.httpTimeout = self.urlObj.httpTimeout
    ret.httpMethod = formMethods.get(self.url, "get")
    ret.parentMd5 = parentMd5
    ret.maxURLsFromPage = maxURLsFromPage
    ret.tcDate = SQLExpression("NOW()")
    ret.UDate = SQLExpression("NOW()")
    ret.depth = (depth + 1)
    ret.contentType = detectedMime
    ret.priority = self.urlObj.priority
    return ret


  # #createUrlObjForCollectURLs - creates and returns urlObj for collected chain URL operation, checks collected url
  #by pattern before
  #
  # @param pattern for check url canditae to chain urls
  # @param urlMd5 - url's md5
  # @param formMethods - formMethods post/get form data
  # @param urlId - parent URL md5
  # @param depth - url's depth in site structure
  # @param detectedMime - incoming value for detectedMime field
  # @param maxURLsFromPage - incoming value for maxURLsFromPage field
  def createUrlObjForChain(self, pattern, urlMd5, formMethods, parentMd5, depth, detectedMime, maxURLsFromPage):
    ret = None
    self.checkClassFields(["url"])
    #logger.debug(">>> chain patter is = " + str(pattern) + " url = " + self.url)
    if re.search(pattern, self.url) is not None:
      ret = self.createUrlObjForCollectURLs(urlMd5, formMethods, parentMd5, depth, detectedMime, maxURLsFromPage)
      ret.type = dc.EventObjects.URL.TYPE_CHAIN
    return ret


  # #updateTypeForURLObjects updates url type in DB
  #
  # @param urlObjects - list of url objects
  # @param typeArg - updated url type
  def updateTypeForURLObjects(self, urlObjects, typeArg=dc.EventObjects.URL.TYPE_CHAIN):
    updateUrlObjects = []
    for urlObject in urlObjects:
      localUrlObject = dc.EventObjects.URLUpdate(urlObject.siteId, urlObject.url)
      localUrlObject.urlMd5 = urlObject.urlMd5
      localUrlObject.type = typeArg
      updateUrlObjects.append(localUrlObject)
    if len(updateUrlObjects) > 0:
      self.dbWrapper.urlUpdate(updateUrlObjects)



  # #fillRssFieldInUrlObj - fills some fields, related with rss, in internal urlObj
  #
  # @param oldUrl - incoming previous ulr
  # @param objectUrlUlr - incoming url object
  # @param batchItem - incoming bachItem
  # @param processorName - processor name
  # @param feed - incoming source of rss data
  # @return dict with filled rss values
  def fillRssFieldInUrlObj(self, oldUrl, objectUrlUlr, batchItem, processorName, feed, rootFeed=False):
    logger.debug("oldUrl=%s\nobjectUrlUlr=%s\nbatchItem=%s\nprocessorName=%s\nfeed=%s\n",
                 Utils.varDump(oldUrl),
                 Utils.varDump(objectUrlUlr),
                 Utils.varDump(batchItem),
                 Utils.varDump(processorName),
                 #Utils.varDump(feed)
                 str(feed)
                 )
    self.checkClassFields(["url", "siteId", "urlObj"])
    ret = None
    status = dc.EventObjects.URL.STATUS_CRAWLED
    crawled = 1
    localType = dc.EventObjects.URL.TYPE_FETCHED
    if processorName == PCONSTS.PROCESSOR_RSS:
      status = dc.EventObjects.URL.STATUS_NEW
      crawled = 0
      localType = dc.EventObjects.URL.TYPE_SINGLE
    if rootFeed:
      ret = self.fillRssFieldOneElem(feed.feed, objectUrlUlr, batchItem, status, crawled, localType)
    else:
      for entry in feed.entries:
        logger.debug("entry.link=%s, oldUrl=%s", Utils.varDump(entry.link), Utils.varDump(oldUrl))
        if entry.link == oldUrl:
          ret = self.fillRssFieldOneElem(entry, objectUrlUlr, batchItem, status, crawled, localType)
    logger.debug("ret=\n%s", Utils.varDump(ret))

    return ret


  # #fillRssFieldOneElem - fills one rss field element
  #
  # @param entry - one rss entry
  # @param objectUrlUlr - incoming url object
  # @param batchItem - incoming bachItem
  # @param status - calculated ststus for urlObj
  # @param crawled - calculated crawled value for urlObj
  # @param localType - calculated localType value for urlObj
  # @return one filled rss dict element
  def fillRssFieldOneElem(self, entry, objectUrlUlr, batchItem, status, crawled, localType):
    ret = {}
    ret["entry"] = entry
    ret["urlObj"] = dc.EventObjects.URL(self.siteId, self.url)
    ret["parent_rss_feed"] = objectUrlUlr
    ret["parent_rss_feed_urlMd5"] = batchItem.urlId
    #pubdate
    ret["pubdate"] = None
    dates = ["published_parsed", "updated_parsed"]
    for date in dates:
      if date in entry:
        try:
          ret["pubdate"] = strftime("%Y-%m-%d %H:%M:%S", entry[date])
        except TypeError:
          logger.debug("Unsupported date format: <%s>", Utils.varDump(entry[date]))
    pubdate = ret["pubdate"]
    contentType = entry.links[0].type
    ret["urlObj"].status = status
    ret["urlObj"].crawled = crawled
    ret["urlObj"].contentType = contentType
    ret["urlObj"].pDate = pubdate
    ret["urlObj"].type = localType
    size = len(str(ret))
    ret["urlObj"].size = size
    return ret


  # #urlTemplateApply - method applies url's template for current url
  #
  # @param url - incoming url
  # @param crawlerType - crawler's type
  # @param urlTempalteRegular - url template, that applying in regularCrawl case
  # @param urlTempalteRealtime - url template, that applying in realTimeCrawl case
  # @param urlTempalteRegularEncode - make url enconing for regularCrawl or not
  # @param urlTempalteRealtimeEncode - make url enconing for realTimeCrawl or not
  # @return processed url, after url template applying
  def urlTemplateApply(self, url, crawlerType, urlTempalteRegular, urlTempalteRealtime, urlTempalteRegularEncode,
                       urlTempalteRealtimeEncode):
    ret = url
    if crawlerType == dc.EventObjects.Batch.TYPE_REAL_TIME_CRAWLER:
      if urlTempalteRealtime is not None:
        try:
          if urlTempalteRealtimeEncode is not None and bool(int(urlTempalteRealtimeEncode)):
            encodedUrl = urllib.quote(url)
          else:
            encodedUrl = url
        except ValueError:
          encodedUrl = url
        ret = urlTempalteRealtime.replace(self.URL_TEMPLATE_CONST, encodedUrl)
    else:
      if urlTempalteRegular is not None:
        try:
          if urlTempalteRegularEncode is not None and bool(int(urlTempalteRegularEncode)):
            encodedUrl = urllib.quote(url)
          else:
            encodedUrl = url
        except ValueError:
          encodedUrl = url
        ret = urlTempalteRegular.replace(self.URL_TEMPLATE_CONST, encodedUrl)
    if ret != url:
      logger.debug(">>> url was replaced ")
      logger.debug(">>> new url = " + ret)
    return ret
