'''
@package: dc
@author scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import logging
import base64
import json
import copy
import types
import dc.EventObjects
import app.Consts as APP_CONSTS

logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


# class ProcessedContentInternalStruct contents static methods for processing internal structure of processedContent
class ProcessedContentInternalStruct(object):

  DATA_FIELD = "data"
  CDATE_FIELD = "CDate"


  ## parseProcessedBuf method decodes incoming processedContent buff, and fills, depend of contentMask value,
  # outgoing Contents list
  #
  #@param buf - incoming processedContent buff
  #@param cDateValue - incoming CDate value
  #@param contentMask - incoming contentMask
  #@return list of Content or tuple(Content, Content) objects
  @staticmethod
  def parseProcessedBuf(buf, cDateValue, contentMask):
    ret = []
    try:
      processedContent = json.loads(base64.b64decode(buf))
    except Exception as excp:
      processedContent = None
      logger.debug(">>> Wrong something bad with processedContent decode, =" + str(excp))
    if processedContent is not None:
      if type(processedContent) is types.DictType:
        if contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED:
          if len(processedContent["custom"]) > 0:
            resultElem = base64.b64encode(processedContent["custom"][0])
          else:
            localProcessedContent = copy.deepcopy(processedContent)
            del localProcessedContent["internal"]
            del localProcessedContent["custom"]
            resultElem = json.dumps(localProcessedContent)
            resultElem = base64.b64encode(resultElem)
          content = dc.EventObjects.Content(resultElem, cDateValue, dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
          ret.append(content)
        if contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_INTERNAL and \
        contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_CUSTOM:
          i = 0
          for elem in processedContent["internal"]:
            resultElem = json.dumps(elem)
            resultElem = base64.b64encode(resultElem)
            contentInternal = dc.EventObjects.Content(resultElem, cDateValue,
                              dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
            if i < len(processedContent["custom"]):
              elemCustom = processedContent["custom"][i]
              resultElem = base64.b64encode(elemCustom)
              contentCustom = dc.EventObjects.Content(resultElem, cDateValue,
                                dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
            else:
              contentCustom = None
            insertTuple = (contentInternal, contentCustom)
            ret.append(insertTuple)
            i += 1
        elif contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_INTERNAL:
          for elem in processedContent["internal"]:
            resultElem = json.dumps(elem)
            resultElem = base64.b64encode(resultElem)
            content = dc.EventObjects.Content(resultElem, cDateValue,
                      dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
            ret.append(content)
            if (contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_ALL) == 0:
              break
        elif contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_CUSTOM:
          for elem in processedContent["custom"]:
            resultElem = base64.b64encode(elem)
            content = dc.EventObjects.Content(resultElem, cDateValue,
                      dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
            ret.append(content)
            if (contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED_ALL) == 0:
              break
      elif type(processedContent) is types.ListType:
        if contentMask & dc.EventObjects.URLContentRequest.CONTENT_TYPE_PROCESSED:
          resultElem = json.dumps(processedContent)
          resultElem = base64.b64encode(resultElem)
          content = dc.EventObjects.Content(resultElem, cDateValue, dc.EventObjects.Content.CONTENT_PROCESSOR_CONTENT)
          ret.append(content)
    return ret
