'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import hashlib
import logging
import Constants
import dc.EventObjects
import MySQLdb
import copy
import types
import app.SQLCriterions
from app.Utils import SQLExpression
from BaseTask import BaseTask
from Constants import DC_URLS_TABLE_NAME_TEMPLATE
from dtm.EventObjects import GeneralResponse
from StatisticLogManager import StatisticLogManager
from SiteDeleteTask import SiteDeleteTask

logger = logging.getLogger(Constants.LOGGER_NAME)


# @todo move to apropriate place

class SiteTask(BaseTask):


  # #constructor
  #
  # @param dcSiteTemplate path to sql template for dc_urls_* tables
  def __init__(self, dcSiteTemplate, keyValueDefaultFile, keyValueStorageDir, dBDataTask, dcStatTemplates,
               dcLogTemplate, tasksManager=None):
    super(SiteTask, self).__init__()
    self.dcSiteTemplate = dcSiteTemplate
    self.keyValueDefaultFile = keyValueDefaultFile
    self.keyValueStorageDir = keyValueStorageDir
    self.dBDataTask = dBDataTask
    self.dcStatTemplates = dcStatTemplates
    self.dcLogTemplate = dcLogTemplate
    if self.dBDataTask is not None:
      self.siteDeleteTask = SiteDeleteTask(keyValueStorageDir, self.dBDataTask.rawDataDir, dBDataTask)
    else:
      self.siteDeleteTask = None
    self.tasksManager = tasksManager


  # #make all necessary actions to add new site into mysql db
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  # @return generalResponse instance of GeneralResponse object
  def process(self, site, queryCallback):
    response = GeneralResponse()
    #if not self.isSiteExist(site.urls[0], queryCallback):
    if not self.isSiteExist(site.id, queryCallback, site.userId):
      self.addSite(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.addSitesFilter(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.addSiteProperties(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.createTableFromTemplate(site, self.dcSiteTemplate, Constants.SECONDARY_DB_ID, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.createTableFromTemplate(site, self.dcStatTemplates, Constants.STAT_DB_ID, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.createTableFromTemplate(site, self.dcLogTemplate, Constants.LOG_DB_ID, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.addSiteURLSites(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        if site.moveURLs:
          self.addSiteURLURLs(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        self.addSiteInKVDB(site, queryCallback)
      if self.tasksManager is None or self.tasksManager.SQLErrorCode == Constants.EXIT_CODE_OK:
        response.statuses.append(site.id)
      else:
        logger.debug(">>> Was some sql error while SITE_NEW operation start SiteDelete/rollback")
        self.siteDelete(site, queryCallback)
        response = GeneralResponse(Constants.TASK_SQL_ERR, Constants.TASK_SQL_ERR_MSG)
        response.statuses.append(None)
    else:
      response = GeneralResponse(Constants.TASK_DUPLICATE_ERR, Constants.TASK_DUPLICATE_ERR_MSG)
      response.statuses.append(None)

    return response


  # # add data in sites table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def addSite(self, site, queryCallback):
    fields, values = Constants.getFieldsValuesTuple(site, Constants.siteDict)
    fieldValueString = Constants.createFieldsValuesString(fields, values)
    if len(fieldValueString) > 0:
      query = Constants.INSERT_COMMON_TEMPLATE % ("sites", fieldValueString)
      queryCallback(query, Constants.PRIMARY_DB_ID)
    else:
      pass


  # # add data in sites_filters table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def addSitesFilter(self, site, queryCallback):
    if site.filters is not None:
      for localFilter in site.filters:
        if localFilter is not None:
          localUDate = "NULL"
          if localFilter.uDate is not None:
            localUDate = str(localFilter.uDate)
          query = (Constants.SITE_FILTER_SQL_TEMPLATE %
                  (localFilter.siteId, str(localFilter.pattern), str(localFilter.subject), str(localFilter.opCode),
                   str(localFilter.stage), str(localFilter.action), localUDate, str(localFilter.type),
                   str(localFilter.mode), str(localFilter.state), str(localFilter.groupId)))
          queryCallback(query, Constants.PRIMARY_DB_ID)



  # # update data in sites_filters table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def updateSitesFilter(self, site, queryCallback):
    if site.filters is not None:
      for localFilter in site.filters:
        if localFilter is not None:
          localUDate = "`UDate`"
          if localFilter.uDate is not None:
            localUDate = str(localFilter.uDate)
          query = (Constants.SITE_FILTER_SQL_UPDATE %
                  (str(localFilter.pattern), str(localFilter.subject), str(localFilter.opCode), str(localFilter.stage),
                   str(localFilter.action), localUDate, str(localFilter.groupId), localFilter.siteId,
                   str(localFilter.type), str(localFilter.mode), str(localFilter.state)))
          queryCallback(query, Constants.PRIMARY_DB_ID)


  # # creates site properties dict
  #
  # @field - incoming site properties field
  # @site - incomig site
  # @return created site properties dict
  def createPropDict(self, field, site):
    localDict = {}
    if isinstance(site.properties, dict):
      if isinstance(site.properties[field], list):
        if site.properties[field][0] is not None:
          localDict["siteId"] = str(site.properties[field][0])
        else:
          localDict["siteId"] = site.id
        localDict["name"] = str(field)
        if site.properties[field][1] is not None:
          localDict["value"] = str(site.properties[field][1])
        if site.properties[field][2] is not None:
          localDict["urlMd5"] = str(site.properties[field][2])
      else:
        localDict["siteId"] = site.id
        localDict["name"] = str(field)
        localDict["value"] = str(site.properties[field])
    elif isinstance(site.properties, list) and isinstance(field, dict):
      localDict = field
    if "uDate" in localDict and type(localDict["uDate"]) in types.StringTypes:
      localDict["uDate"] = SQLExpression(localDict["uDate"])
    if "cDate" in localDict and type(localDict["cDate"]) in types.StringTypes:
      localDict["cDate"] = SQLExpression(localDict["cDate"])
    return localDict


  # # add data in sites_properties table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def addSiteProperties(self, site, queryCallback):
    if site.properties is not None:
      localDict = {}
      query = None
      for field in site.properties:
        if field is not None:
          query = None
          localDict = self.createPropDict(field, site)
          fields, values = Constants.getFieldsValuesTuple(localDict, Constants.propDict)
          if len(fields) > 0 and len(fields) == len(values):
            fieldValueString = Constants.createFieldsValuesString(fields, values)
            query = Constants.INSERT_COMMON_TEMPLATE % ("sites_properties", fieldValueString)
            queryCallback(query, Constants.PRIMARY_DB_ID)


  # # update data in sites_properties table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def updateSiteProperties(self, site, queryCallback):
    if site.properties is not None:
      localDict = {}
      query = None
      for field in site.properties:
        if field is not None:
          query = None
          localDict = self.createPropDict(field, site)
          if "siteId" in localDict and localDict["siteId"] is not None and "name" in localDict and \
          localDict["name"] is not None:
            fields, values = Constants.getFieldsValuesTuple(localDict, Constants.propDict)
            if len(fields) > 0 and len(fields) == len(values):
              fieldValueString = Constants.createFieldsValuesString(fields, values)
              query = Constants.SITE_PROP_SQL_UPDATE % (fieldValueString, localDict["siteId"], localDict["name"])
              queryCallback(query, Constants.PRIMARY_DB_ID)
          else:
            logger.debug("Site.Properties \"SiteId\" or \"name\" fields are empty")


  # # add data in sites_urls table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def addSiteURLSites(self, site, queryCallback):
    for urlObject in site.urls:
      if urlObject is not None:
        fields, values = Constants.getFieldsValuesTuple(urlObject, Constants.SiteURLTableDitct)
        fieldValueString = Constants.createFieldsValuesString(fields, values)
        query = Constants.SITE_URL_SQL_TEMPLATE % fieldValueString
        queryCallback(query, Constants.PRIMARY_DB_ID, Constants.EXEC_INDEX, True)


  # # update data in sites_urls table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def updateSiteURLSites(self, site, queryCallback):
    for urlObject in site.urls:
      if urlObject is not None:
        fields, values = Constants.getFieldsValuesTuple(urlObject, Constants.SiteURLTableDitct)
        fieldValueString = Constants.createFieldsValuesString(fields, values)
        query = Constants.SITE_URL_SQL_UPDATE % (fieldValueString, site.id, urlObject.url)
        queryCallback(query, Constants.PRIMARY_DB_ID)

      self.statisticLogUpdate(site, urlObject.urlMd5, site.id, urlObject.status, queryCallback)


  # # add data in dc_urls.url_site.id table
  #
  # @param site instance of Site object
  # @param queryCallback function for queries execution
  def addSiteURLURLs(self, site, queryCallback):
    for urlObject in site.urls:
      urlMD5 = hashlib.md5(urlObject.url).hexdigest()
      StatisticLogManager.addNewRecord(queryCallback, site.id, urlMD5)
      if type(site) == type(dc.EventObjects.Site("")):
        localURLObject = copy.copy(urlObject)
        localURLObject.siteId = site.id
        if site.urlType is not None:
          localURLObject.type = site.urlType
        localURLObject.url = MySQLdb.escape_string(urlObject.url)
        if site.requestDelay is not None:
          localURLObject.requestDelay = site.requestDelay
        if site.httpTimeout is not None:
          localURLObject.httpTimeout = site.httpTimeout
        localURLObject.urlMd5 = urlMD5
        DC_SITE_URL_SQL_TEMPLATE = "INSERT IGNORE INTO `%s` SET %s"
        fields, values = Constants.getFieldsValuesTuple(localURLObject, Constants.URLTableDict)
        fieldValueString = Constants.createFieldsValuesString(fields, values)
        query = DC_SITE_URL_SQL_TEMPLATE % ((DC_URLS_TABLE_NAME_TEMPLATE % site.id), fieldValueString)
      else:
        DC_SITE_URL_SQL_TEMPLATE = ("INSERT IGNORE INTO `%s` SET `Site_Id`='%s', `URL`='%s', `URLMD5`='%s'")
        query = DC_SITE_URL_SQL_TEMPLATE % ((DC_URLS_TABLE_NAME_TEMPLATE % site.id), site.id,
                                            MySQLdb.escape_string(urlObject.url), urlMD5)
      queryCallback(query, Constants.SECONDARY_DB_ID)


  # # create new table in dc_urls db
  #
  # @param site instance of Site object
  # @param template path to the current create table template file
  # @param dbId current database id
  # @param queryCallback function for queries execution
  def createTableFromTemplate(self, site, template, dbId, queryCallback):
    try:
      fTemplate = open(template).read()
      query = fTemplate.replace("%SITE_ID%", str(site.id))
      queryCallback(query, dbId)
    except Exception:
      logger.debug(">>> Can't open file - " + template)


  # # create empty file for Key-Value db
  #
  # @param site instance of Site object
  # @param queryCallback callback for sql requests
  def addSiteInKVDB(self, site, queryCallback):
    ret = None
    if self.dBDataTask is not None:
      dataCreateRequest = dc.EventObjects.DataCreateRequest(site.id, None, None)
      ret = self.dBDataTask.process(dataCreateRequest, queryCallback)
    return ret


  # # makes roolback operation for current site if SITE_NEW operating got fault
  #
  # @param site instance of Site object
  # @param queryCallback callback for sql requests
  def siteDelete(self, site, queryCallback):
    if self.siteDeleteTask is not None:
      siteDelete = dc.EventObjects.SiteDelete(site.id)
      self.siteDeleteTask.process(siteDelete, queryCallback)


  ##execSiteCriterion method extract sites ids by siteCriterions
  #
  #@param criterions - criterions dict
  #@param queryCallback function for queries execution
  #@return list of appropriate site's ids
  @staticmethod
  def execSiteCriterions(criterions, queryCallback):
    ret = []
    SITE_CRITERION_TEMPLATE = "SELECT `Id` FROM `sites`"
    additionWhere = app.SQLCriterions.generateCriterionSQL(criterions, None)
    if additionWhere is not None and additionWhere != "":
      query = SITE_CRITERION_TEMPLATE + additionWhere
      result = queryCallback(query, Constants.PRIMARY_DB_ID)
      if hasattr(result, '__iter__'):
        ret = [elem[0] for elem in result]
    return ret
