# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
DateTimeType Class content main functional extract of datetime.

@package: dc_processor
@file DateTimeType.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2015 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import sys
reload(sys)
sys.setdefaultencoding('utf8')

import datetime
import re
from email.Utils import formatdate  # pylint: disable=E0611, F0401
from dateutil.parser import *  # pylint: disable=W0401,W0614
from dateutil import parser
import time
from app.Utils import ExceptionLog

## Class DateTimeType for extract data
#
class DateTimeType(object):
  ##Constans used in class
  ISO_SEP = ' '
  BAD_SIMBOLS = '=(),|@.'
  TAG_NAMES = ['pubdate', 'dc_date']


  LANG_ENG = "ENG"
  LANG_RUS = "RUS"
  LANG_UKR = "UKR"
  LANG_GER = "GERMAN"
  LANG_JAP = "JAPAN"

  wordsListEng = [u'Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec', \
                  u'Year', u'Today', u'Yesterday', u'Day before yesterday']
  wordsListRus = [u'Янв', u'Февр', u'Март', u'Апр', u'Май', u'Июнь', u'Июль', u'Авг', u'Сент', u'Окт', u'Нояб', \
                  u'Дек', u'Сегодня', u'Вчера', u'Поза вчера']
  wordsListUkr = [u'Сiч', u'Лют', u'Бер', u'Квiт', u'Трав', u'Черв', u'Лип', u'Серп', u'Вер', u'Жовт', u'Лист', \
                  u'Груд', u'Рік', u'Сьогодні', u'Вчора', u'Позавчора']
  wordsListGer = [u'März', u'Mai', u'Juni', u'Juli', u'Sept', u'Okt', u'Dez', u'Jahr', u'Heute', u'Gestern', \
                  u'Vorgestern', u'Uhr']
  wordsListJap = [u'一月', u'二月', u'三月', u'四月', u'五月', u'六月', u'七月', u'八月', u'九月', u'十月', u'十一月', \
                  u'十二月', u'年', u'今日', u'イエスタデイ', u'おととい', u'月', u'日', u'時', u'分']

  monthListEng = [u'Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec']
  monthListRus = [u'Янв', u'Февр', u'Март', u'Апр', u'Май', u'Июнь', u'Июль', u'Авг', u'Сент', u'Окт', u'Нояб', u'Дек']
  monthListUkr = [u'Сiч', u'Лют', u'Бер', u'Квiт', u'Трав', u'Черв', u'Лип', u'Серп', u'Вер', u'Жовт', u'Лист', u'Груд']
  monthListGer = [u'Jan', u'Feb', u'März', u'Apr', u'Mai', u'Juni', u'Juli', u'Aug', u'Sept', u'Okt', u'Nov', u'Dez']
  monthListJap = [u'一月', u'二月', u'三月', u'四月', u'五月', u'六月', u'七月', u'八月', u'九月', u'十月', u'十一月', u'十二月']

  dayStateEng = [u'Today', u'Yesterday', u'Day before yesterday']
  dayStateRus = [u'Сегодня', u'Вчера', u'Поза вчера']
  dayStateUkr = [u'Сьогодні', u'Вчора', u'Позавчора']
  dayStateGer = [u'Heute', u'Gestern', u'Vorgestern']
  dayStateJap = [u'今日', u'イエスタデイ', u'おととい']

  LANG_DICT = {LANG_ENG: wordsListEng, LANG_RUS: wordsListRus, LANG_UKR: wordsListUkr, LANG_GER: wordsListGer, \
               LANG_JAP: wordsListJap}

  MONTH_DICT = {LANG_ENG: monthListEng, LANG_RUS: monthListRus, LANG_UKR: monthListUkr, LANG_GER: monthListGer, \
               LANG_JAP: monthListJap}

  DAY_STATE_DICT = {LANG_ENG: dayStateEng, LANG_RUS: dayStateRus, LANG_UKR: dayStateUkr, LANG_GER: dayStateGer, \
               LANG_JAP: dayStateJap}


  patternListDate = [r'(?P<mon>[ 1][0-9]) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[ 01][0-9])/(?P<day>[ 0123][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[ 01][0-9])/(?P<day>[0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[A-Z][a-z][a-z]) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[A-Z][a-z][a-z]) (?P<day>[0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[0-9])(\w{2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[ 0123][0-9])(\w{2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9]) (?P<mon>[0-1][0-9]) (?P<day>[ 0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])-(?P<mon>[0-1][0-9])-(?P<day>[ 0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])/(?P<mon>[0-1][0-9])/(?P<day>[ 0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])(?P<mon>[0-1][0-9])(?P<day>[ 0123][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\W+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9])(\w{2}) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9]) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9]) (?P<mon>\W+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9])/(?P<mon>[0-9][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{2})/(?P<mon>[0-9][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{2})/(?P<mon>\d+})/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\w+)',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\W+)',
                     r'(?P<day>[0-9]) (?P<mon>\w+)',
                     r'(?P<day>[0-9]) (?P<mon>\W+)']

  patternListTime = [r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2}) (?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2}) (?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2})(?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2})(?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) (?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) (?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2})(?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2})(?P<tf>[PpAaMm]{2})',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>[0-9][0-9])',
                      r'(?P<hour>[ 0-9][0-9])(?P<min>\d{2}) GMT',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) GMT',
                      r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2})']

  ##Constant of error messages
  ERROR_INPUT_PARAMS = 'Error initialization by input parameters.'
  ERROR_FORMAT_STRING_TYPE = 'Format string is not string.'
  ERROR_DATA_STRING_TYPE = 'Data string is not string.'
  ERROR_BAD_INPUT_DATA = 'Bad inputted data.'

  ##Constructor
  #
  #@param dataString - data string for extract datatime, can be as timestamp or string with data for extract
  #@param formatString - format string for formatting data
  def __init__(self, dataString=None, formatString=None):
    self.datetime = None
    self.isError = False
    self.errorMsg = ''

    try:
      self.datetime = self.__initDataTime(dataString, formatString)

    except Exception, err:
      raise Exception(self.ERROR_INPUT_PARAMS + ' ' + str(err))


  ## initialization of datatime
  #
  #@param dataString - data string for extract datatime, can be as timestamp or string with data for extract
  #@param formatString - format string for formatting data
  #@return datatime - extracted datatime or None
  def __initDataTime(self, dataString=None, formatString=None):
    #variable for result
    ret = None
    if dataString is not None and isinstance(dataString, int):
      ret = datetime.datetime.fromtimestamp(dataString)
    elif dataString is not None and formatString is not None:
      #validate of input type of format string
      if not isinstance(formatString, str):
        raise Exception(self.ERROR_FORMAT_STRING_TYPE)
      else:
        pass
      #validate of input type of data string
      if not isinstance(dataString, str):
        raise Exception(self.ERROR_DATA_STRING_TYPE)
      else:
        pass
      #input types checked and can be used
      ret = datetime.datetime.strptime(dataString, formatString)
    elif dataString is None and formatString is None:
      pass
    else:
      raise Exception(self.ERROR_BAD_INPUT_DATA)

    return ret


  ## Return datatime as timestamp
  #
  #@param - None
  #@return datatime as timestamp
  def getInt(self):
    #variable for result
    ret = None
    try:
      ret = int((self.datetime - datetime.datetime.fromtimestamp(0)).total_seconds())
    except Exception, err:
      self.isError = True
      self.errorMsg = str(err)
      ret = None

    return ret


  ## Return datatime as string
  #
  #@param - formatString - format string for formatting data
  #@return datatime as timestamp
  def getString(self, formatString=None):
    #variable for result
    ret = None

    try:
      if formatString is None:
        ret = self.datetime.isoformat(self.ISO_SEP)
      else:
        if not isinstance(formatString, str):
          raise Exception(self.ERROR_FORMAT_STRING_TYPE)
        else:
          ret = self.datetime.strftime(formatString)
    except Exception, err:
      self.isError = True
      self.errorMsg = str(err)
      ret = None

    return ret


  ## Intendification of lang
  #
  #@param inputStr - string for detection of lang
  #@return constant of lang name
  @staticmethod
  def getLang(inputStr):
    ret = None
    found = False
    langDict = DateTimeType.LANG_DICT

    for key in langDict.keys():
      for month in langDict[key]:
        if inputStr.lower().find(month) > -1 or inputStr.lower().find(month.lower()) > -1:
          ret = key
          found = True
          break
      if found:
        break

    return ret


  ## Intendification of lang
  #
  #@param inputStr - string for detection of lang
  #@return constant of month number if found
  @staticmethod
  def getMonthNumber(inputStr, logger):

    #print('getMonthNumber inputStr: ' + str(inputStr))
    #if logger:
    #  logger.debug("getMonthNumber inputStr: '" + str(inputStr) + "' type: " + str(type(inputStr)))

    ret = None
    inputStr = inputStr.strip()

    #if logger:
    #  logger.debug("getMonthNumber inputStr.isdigit() = " + str(bool(inputStr.isdigit())))

    if len(inputStr) < 3 and inputStr.isdigit():
      ret = int(inputStr)
    else:
      found = False
      monthDict = DateTimeType.MONTH_DICT

      for key in monthDict.keys():
        #print(key)
        #if logger:
        #  logger.debug("key: '" + str(key) + "'")

        monthNumber = 0
        for months in monthDict[key]:
          monthNumber = monthNumber + 1
          month = months.lower()

          #if logger:
          #  logger.debug(inputStr.lower() + ' <=> ' + month)
          #print(inputStr.lower() + ' <=> ' + month)

          if inputStr.lower().find(month) > -1:
            ret = monthNumber
            found = True
            break
        if found:
          break

    return ret


  ## Exctract english date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateEng(inputStr, useCurrentYear, logger=None):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, DateTimeType.LANG_ENG)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger)

    return pubdate


  ## Exctract russian date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateRus(inputStr, useCurrentYear, logger=None):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, DateTimeType.LANG_RUS)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger)

    return pubdate

  ## Exctract ukrainian date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateUkr(inputStr, useCurrentYear, logger=None):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, DateTimeType.LANG_UKR)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger)

    return pubdate


  ## Exctract german date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateGerman(inputStr, useCurrentYear, logger=None):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, DateTimeType.LANG_GER)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger)

    return pubdate


  ## Exctract japan date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateJapan(inputStr, useCurrentYear, logger=None):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, DateTimeType.LANG_JAP)

    if pubdate is None:
      match = re.search(r'[0-9]', inputStr)
      if match:
        localStr = inputStr[match.start():]
        #print('localStr: ' + str(localStr))
        pubdate = DateTimeType.convertPubDateToRFC2822(localStr, logger)

    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger)

    return pubdate


  ## static method for parse
  #
  #@param dataString - string for parse
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def parse(dataString, useCurrentYear=True, logger=None):
    #variable for result
    ret = None
    #Intendification of lang
    langType = DateTimeType.getLang(dataString)

    if logger is not None:
      logger.debug('lang type detected as: ' + str(langType))

    if langType == DateTimeType.LANG_ENG:
      #extract english date
      ret = DateTimeType.extractDateEng(dataString, useCurrentYear, logger)
    elif langType == DateTimeType.LANG_RUS:
      #extract russian date
      ret = DateTimeType.extractDateRus(dataString, useCurrentYear, logger)
    elif langType == DateTimeType.LANG_UKR:
      #extract ukrainian date
      ret = DateTimeType.extractDateUkr(dataString, useCurrentYear, logger)
    elif langType == DateTimeType.LANG_GER:
      #extract germany date
      ret = DateTimeType.extractDateGerman(dataString, useCurrentYear, logger)
    elif langType == DateTimeType.LANG_JAP:
      #extract japan date
      ret = DateTimeType.extractDateJapan(dataString, useCurrentYear, logger)
    else:
      ret = DateTimeType.extractDateCommon(dataString, useCurrentYear, logger)

    return ret


  ## Prepare input string (remove all bad simbols)
  #
  #@param inputStr - input string for preparation
  #@return already prepared string
  @staticmethod
  def prepareString(inputStr):
    ret = inputStr

    for tagName in DateTimeType.TAG_NAMES:
      if inputStr.lower().find('%' + tagName + '%') > -1:
        ret = ret.replace('%' + tagName + '%', '')
      else:
        pass

    for bad in DateTimeType.BAD_SIMBOLS:
      ret = ret.replace(bad, ' ')

    ret = ret.replace('  ', ' ')

    return ret


  ## Extract year, month and day from string
  #
  #@param inputStr - input string for extract date
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return year, month, day - trancated input string
  @staticmethod
  def extractDate(inputStr, useCurrentYear=True, logger=None):
    #variables for results
    month = 0
    day = 0
    year = 0

    #print('extractDate: ' + inputStr + '\n')

    try:
      for pattern in DateTimeType.patternListDate:
        match = re.search(pattern, inputStr)

        #print('match: ' + str(match) + ' pattern: ' + str(pattern))

        if match:
          #print('match.groupdict(): ' + str(match.groupdict()))
          if 'year' in match.groupdict():
            year = match.groupdict()['year']

          if 'mon' in match.groupdict():
            month = match.groupdict()['mon']

          if 'day' in match.groupdict():
            day = match.groupdict()['day']

          #print('month = ' + str(month))

          if logger:
            logger.debug('month = ' + str(month))

          monthNumber = DateTimeType.getMonthNumber(month, logger)

          if logger:
            logger.debug('monthNumber = ' + str(monthNumber))

          if monthNumber is not None:
            month = monthNumber
          else:
            month = day = year = 0

          if logger:
            logger.debug('year: ' + str(year) + ' month: ' + str(month) + ' day: ' + str(day))

          #print ('Data ISO: ' + datetime.date(int(year), int(month), int(day)).isoformat())
          #print('pattern: ' + str(pattern))

          #print('year / 10 > 100: ' + str(bool(int(year) / 10 > 100)))
          #print('year <= datetime.date.today().year: ' + str(bool(int(year) <= datetime.date.today().year)))
          #print('datetime.date.today().year = ' + str(datetime.date.today().year))
          #print('month: ' + str(month))
          #print('day: ' + str(day))

          if int(year) / 10 > 100 and int(year) <= datetime.date.today().year and int(month) <= 12 and int(day) <= 31:
            break

      if useCurrentYear:
        d = datetime.date.today()

        if year == 0 and month and day:
          year = d.year

        if year and month and day == 0:
          day = d.day

    except Exception, err:
      if logger:
        ExceptionLog.handler(logger, err, 'extractDate:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return int(year), int(month), int(day)


  ## Extract hour, minute and second from string
  #
  #@param inputStr - input string for extract time
  #@param logger - instance of logger for log if necessary
  #@return hour, minute, second and tf (time format)
  @staticmethod
  def extractTime(inputStr, logger=None):

    hour = 0
    minute = 0
    second = 0
    tf = ''

    try:
      for pattern in DateTimeType.patternListTime:
        match = re.search(pattern, inputStr)
        #print('match1: ' + str(match))
        if match:
          #print('match.groupdict(): ' + str(match.groupdict()))

          if 'hour' in match.groupdict():
            hour = match.groupdict()['hour']

          if 'min' in match.groupdict():
            minute = match.groupdict()['min']

          if 'sec' in match.groupdict():
            second = match.groupdict()['sec']

          if 'tf' in match.groupdict():
            tf = match.groupdict()['tf']

          #print ('Time ISO: ' + str(datetime.time(int(hour), int(minute), int(second)).isoformat()))
          #print('pattern: ' + str(pattern))
          break

    except Exception, err:
      if logger:
        ExceptionLog.handler(logger, err, 'extractTime:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return int(hour), int(minute), int(second), tf


  ## Exctract common date from string
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@return extracted datatime or None
  @staticmethod
  def extractDateCommon(inputStr, useCurrentYear=True, logger=None):
    #variable for results
    ret = None
    try:

      localStr = DateTimeType.prepareString(inputStr)

      #print('localStr: ' + str(localStr))

      if localStr:
        year, month, day = DateTimeType.extractDate(localStr, useCurrentYear, logger)

        #print('year: ' + str(year) + '\nmonth: ' + str(month) + '\nday: ' + str(day))

        hour, minute, second, tf = DateTimeType.extractTime(localStr, logger)

        #print('hour: ' + str(hour) + '\nminute: ' + str(minute) + '\nsecond: ' + str(second) + '\ntf: ' + str(tf))

        day, hour = DateTimeType.checkTimeFormat(day, hour, tf)

        if year and month and day:

          if useCurrentYear:
            now = datetime.datetime.now()
            if month == now.month and day == now.day:
              if hour == 0:
                hour = int(now.hour)

              if minute == 0:
                minute = int(now.minute)

              if second == 0:
                second = now.second

          ret = datetime.datetime(year, month, day, hour, minute, second)
        elif year == 0 and month == 0 and day == 0 and hour and minute and useCurrentYear:
          d = datetime.date.today()
          year = d.year
          month = d.month
          day = d.day
          ret = datetime.datetime(year, month, day, hour, minute, second)
        else:
          match = re.search(r'\d{10}', localStr)
          if match:
            ret = datetime.datetime.fromtimestamp(int(match.group()))
          else:
            pass

    except Exception, err:
      if logger:
        ExceptionLog.handler(logger, err, 'extractDateCommon:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        logger.debug('inputStr: ' + str(inputStr) + '\n')

    return ret


  ## Checking is exist offset from PM time format
  #
  #@param day - day value
  #@param hour - hour value
  #@param tf - time format (can be 'AM', 'PM' or other)
  #@return day, hour - result variables after change if necessary
  @staticmethod
  def checkTimeFormat(day, hour, tf):
    if tf and tf.lower().find('p') > -1:
      if (int(hour) + 12) < 24:
        hour = int(hour) + 12
      else:
        hour = int(hour) - 12
        day = int(day) + 1
    else:
      pass

    return day, hour


  ## Extract date from string used state of day ('Today', 'Yesterday', 'Day before yesterday')
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current datetime if wasn't selected
  #@param langName - const name of used language
  #@return extracted datatime or None
  @staticmethod
  def extractUseDayState(inputStr, useCurrentYear, langName):
    #variable for results
    ret = None
    index = 0
    for dayState in DateTimeType.DAY_STATE_DICT[langName]:
      if not inputStr.lower().find(dayState) < 0:
        d = datetime.date.today()
        hour, minute, second, tf = DateTimeType.extractTime(inputStr, useCurrentYear)  # pylint: disable=W0612
        t = datetime.time(hour, minute, second)
        dt = datetime.datetime.combine(d, t)
        ret = dt - datetime.timedelta(days=index)
        break
      else:
        index = index + 1

    return ret


  ## Extract date from string used intelligent algorithms
  #
  #@param inputStr - input string for preparation
  #@param useCurrentYear - flag of default usage current year if wasn't selected
  #@param logger - instance of logger for log if necessary
  #@param langName - const name of used language
  #@return extracted datatime or None
  @staticmethod
  def intelligentExtractor(inputStr, useCurrentYear=True, logger=None, langName=None):
    #variable for results
    ret = None

    try:
      if langName is None:
        langName = DateTimeType.getLang(inputStr)
      else:
        pass

      if langName is not None and useCurrentYear:
        ret = DateTimeType.extractUseDayState(inputStr, useCurrentYear, langName)

      ##TODO here extended functional in future
    except Exception, err:
      if logger:
        ExceptionLog.handler(logger, err, 'intelligentExtractor:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        logger.debug('inputStr: ' + str(inputStr) + '\n')

    return ret


  # Convertation japanise pubdate to RFC2822
  #
  #@param rawPubdate - candidate for extract data
  #@param logger - instance of logger for log if necessary
  #@return extracted datetime or None
  @staticmethod
  def convertPubDateToRFC2822(rawPubdate, logger=None):
    ret = None
    #replace all unicode digits to decimal (e.g. u'\uff13' to '3')
    import unicodedata

    if isinstance(rawPubdate, unicode):
      for i in range(0, len(rawPubdate)):
        if rawPubdate[i].isdigit():
          rawPubdate = re.sub(rawPubdate[i], str(unicodedata.digit(rawPubdate[i])), rawPubdate)

    rawPubdate = rawPubdate.encode("utf_8")

    import calendar
    try:
      pubdate_parts = rawPubdate.split(",")
      if len(pubdate_parts) > 1:
        rawPubdate = pubdate_parts[0]
      rawPubdate = rawPubdate.replace("posted at", "")
      rawPubdate = rawPubdate.replace("Updated:", "")
      rawPubdate = re.sub(r"\(1/\d{1}ページ\)", "", rawPubdate)
      rawPubdate = re.sub(r"平成26", "2014", rawPubdate)
      rawPubdate = re.sub(r"\(木\)", "", rawPubdate)
      rawPubdate = re.sub(r" 年 ", "年", rawPubdate)
      rawPubdate = re.sub(r"@", "", rawPubdate)
      parsed_time_candidate_str = float(calendar.timegm(parse(rawPubdate).timetuple()))

      if logger is not None:
        logger.debug("pubdate in seconds: %s", str(parsed_time_candidate_str))
    except Exception:
      rawPubdate = DateTimeType.adjustJapaneseDate(rawPubdate, logger)

      t = u"%Y\xe5\xb9\xb4%m\xe6\x9c\x88%d\xe6\x97\xa5 %H:%M"
      parsed_time_candidate_str = float(calendar.timegm(time.strptime(rawPubdate, t)))
      if logger is not None:
        logger.debug("parsed_time_candidate_str: %s", str(parsed_time_candidate_str))

    #set result value
    ret = datetime.datetime.fromtimestamp(parsed_time_candidate_str)

    return ret


  # Adjust japanise date
  #
  #@param rawPubdate - candidate for extract data
  #@param logger - instance of logger for log if necessary
  #@return rawPubdate - rawPubdate as string
  @staticmethod
  def adjustJapaneseDate(rawPubdate, logger=None):
    # convert rawPubdate in format like
    # 8月20日 20:01
    # 9月3日 11時41分
    # （8月20日 紙面から）
    # 2014年8月7日
    # 2014年8月20日 夕刊
    # 2014年8月21日 朝刊>
    # 2014/8/14付
    # （2014/08/16-14:46）
    # 2014/6/3 18:53 (2014/6/3 20:13更新)
    # 2014.08.20 Wed posted at 17:52 JST
    # 2014.08.01 Fri posted at 12:36 JST
    # 2014年 08月 20日 14:34 JST
    # 2014年08月20日 19時46分
    # 2014年8月29日16時30分
    # 8/20 21:00 更新
    # 11月２３日
    # to
    # 2014年8月7日 20:01
    # check if 日 exist in date
    if logger is not None:
      logger.debug("pubdate has to be converted: <<%s>>", rawPubdate)
      logger.debug("pubdate type: <<%s>>", str(type(rawPubdate)))
    #self.logger.debug("pubdate charcode: <<%s>>", str(icu.CharsetDetector(rawPubdate).detect().getName()))
    #rawPubdate = rawPubdate.decode("utf_8")
    #self.logger.debug("pubdate has to be converted: <<%s>>", rawPubdate)
    #self.logger.debug("pubdate type: <<%s>>", str(type(rawPubdate)))
    rawPubdate = rawPubdate.strip(" \r\t\n)（）\xe6\x9b\xb4\xe6\x96\xb0\xef" +
                                    "\xbc\x89\xe5\x88\x86\xe4\xbb\x98JST\xe7\xb4\x99\xe9\x9d\xa2\xe3\x81\x8b\xe3\x82" +
                                    "　\xe5\xa4\x95\xe5\x88\x8a\xe6\x9c\x9d\xe5\x88\x8a\xe3\x80\x80\xe5\x88\x86")
    # 2014.08.20 Wed posted at 17:52 JST
    # if "Wed posted at" in rawPubdate:
    #  date_items = rawPubdate.replace(" Wed posted at ", " ").replace(".", " ").split()
    #  rawPubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] +\
    # "\xe6\x97\xa5 " + date_items[3]
    if "\xe6\x97\xa5" in rawPubdate:
      rawPubdate = rawPubdate.replace("\xe6\x97\xa5", "\xe6\x97\xa5 ")
      # check if date contain year
      # 2014年08月20日 19時46分
      if "\xe5\xb9\xb4" in rawPubdate and "\xe6\x99\x82" in rawPubdate:
        rawPubdate = rawPubdate.replace("\xe6\x99\x82", ":")
      elif "\xe5\xb9\xb4" in rawPubdate and not ":" in rawPubdate:
        rawPubdate = rawPubdate + " 00:00"
      # 2014年08月19日 11:53
      elif "\xe5\xb9\xb4" in rawPubdate and ":" in rawPubdate and rawPubdate.count(" ") == 1:
        pass
      # 2014年 08月 20日 14:34 JST
      elif "\xe5\xb9\xb4" in rawPubdate and ":" in rawPubdate and rawPubdate.count(" ") > 1:
        pos = rawPubdate.find('\xe3\x80\x80')
        if pos > 0:
          rawPubdate = rawPubdate[:pos]

        date_items = rawPubdate.split()
        if isinstance(date_items, list) and len(date_items) == 2:
          rawPubdate = date_items[0] + " " + date_items[1]
        elif isinstance(date_items, list) and len(date_items) == 3:
          rawPubdate = date_items[0] + date_items[1]
        elif isinstance(date_items, list) and len(date_items) == 4:
          rawPubdate = date_items[0] + date_items[1] + date_items[2] + " " + date_items[3]
        else:
          pass
      elif "\xe5\xb9\xb4" in rawPubdate:
        rawPubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + rawPubdate
    # 2014/8/14付
    if rawPubdate.count("/") == 2:
      # date_items = rawPubdate.split("/")
      # （2014/08/16-14:46）
      date_items = re.split("/|-", rawPubdate)
      if isinstance(date_items, list) and len(date_items) == 4:
        rawPubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 " + date_items[3]
      else:
        rawPubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 00:00"
    # 8月20日 紙面から
    if not "\xe5\xb9\xb4" in rawPubdate and not "/" in rawPubdate:
      date_items = rawPubdate.replace("\xe6\x9c\x88", " ").replace("\xe6\x97\xa5", " ").\
      replace("\xe6\x99\x82", " ").split()
      if isinstance(date_items, list) and len(date_items) == 4:
        rawPubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 " + date_items[2] + ":" + date_items[3]
      else:
        rawPubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 00:00"
    # 8/20 21:00 更新
    if not "\xe5\xb9\xb4" in rawPubdate and "/" in rawPubdate:
      date_items = rawPubdate.replace("/", " ").split()
      # 2014/6/3 18:53 (2014/6/3 20:13更新)
      if len(date_items) > 4:
        rawPubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + \
        "\xe6\x97\xa5 " + date_items[3]
      else:
        rawPubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + \
        "\xe6\x97\xa5 " + date_items[2]

    if logger is not None:
      logger.debug("pubdate converted is: <<%s>>", rawPubdate)

    return rawPubdate
