# coding: utf-8

"""@package docstring
 @file Scraper.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""
import re
import sys
import time
import json
import types
import ast
import pickle
import logging.config
import ConfigParser
import MySQLdb as mdb
import MySQLdb.cursors
import xml.sax.saxutils
import urlparse
import urllib
import copy
import datetime
from lepl.apps.rfc3696 import HttpUrl
from email.Utils import formatdate  # pylint: disable=E0611, F0401
from dateutil.parser import *  # pylint: disable=W0401,W0614
from dateutil import parser
from contextlib import closing
from app.Utils import varDump
from app.SelectorWrapper import SelectorWrapper
from cement.core import foundation
import app.Utils as Utils  # pylint: disable=F0401
from app.Utils import ExceptionLog

# scraper's modules
import app.Consts as APP_CONSTS
import dc_processor.Constants as CONSTS
import dc.EventObjects
from scraper_resource import Resource as Res
from scraper_result import Result as Result
from dc_processor.ScraperResponse import ScraperResponse
from dc.EventObjects import SiteFilter
from app.Utils import SQLExpression
from TemplateExtractorXPathPreparing import TemplateExtractorXPathPreparing
import app.Profiler
from dc_processor.DateTimeType import DateTimeType

# scraper's modules used via eval()
from newspaper_extractor import NewspaperExtractor  # pylint: disable=W0611
from goose_extractor import GooseExtractor  # pylint: disable=W0611
from scrapy_extractor import ScrapyExtractor
from ml_extractor import MLExtractor  # pylint: disable=W0611


APP_NAME = "scraper"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting."
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Error loading logging config file. Exiting."
MSG_ERROR_LOAD_EXTRACTORS = "Error load extractors "
MSG_ERROR_TEMPLATE_EXTRACTION = "Error template extraction "
MSG_ERROR_DYNAMIC_EXTRACTION = "Error dynamic extraction "
MSG_ERROR_LOAD_DB_BACKEND = "Error load db backend"
MSG_ERROR_LOAD_OPTIONS = "Error load options"
MSG_INFO_PREPARE_CONTENT = "Prepare content: "
MSG_ERROR_ADJUST_PR = "Error adjust partial references. "
MSG_ERROR_ADJUST_PUBDATE = "PUBDATE_ERROR "
MSG_ERROR_ADJUST_TITLE = "Can't adjust title. "

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

SQLITE_TIMEOUT = 30

ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
CONTENT_REPLACEMENT = "[\"\\n\", \"\\r\\n\", \"\\t\"]"

EXTENDED_NEWS_TAGS = {"description": ["//meta[@name='description']//@content"]}
LINKS_NEWS_TAGS = [CONSTS.TAG_MEDIA, CONSTS.TAG_LINK, CONSTS.TAG_MEDIA_CONTENT, "links", "href"]
#DATA_NEWS_TAGS = [CONSTS.TAG_DC_DATE]
DATA_NEWS_TAGS = []
WWW_PREFIX = "www."

TAGS_DATETIME_NEWS_NAMES = [CONSTS.TAG_PUB_DATE, CONSTS.TAG_DC_DATE]
TAGS_DATETIME_TEMPLATE_TYPES = [CONSTS.TAG_TYPE_DATETIME]

OPTION_SECTION_DATETIME_NEWS_NAMES = 'tags_datetime_news_names'
OPTION_SECTION_DATETIME_TEMPLATE_TYPES = 'tags_datetime_template_types'

# #Scraper
#
#
class Scraper(foundation.CementApp):


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      pass


  # #constructor
  # initialize default fields
  def __init__(self, usageModel=APP_CONSTS.APP_USAGE_MODEL_PROCESS, configFile=None, logger=None, inputData=None):
    if usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
      # call base class __init__ method
      foundation.CementApp.__init__(self)

    self.exitCode = EXIT_SUCCESS
    self.itr = None
    self.extractor = None
    self.extractors = []
    self.input_data = inputData
    self.logger = logger
    self.config_db_dir = None
    self.articles_tbl = None
    self.sqliteTimeout = SQLITE_TIMEOUT
    self.DBConnector = None
    self.db_dc_scraper = None
    self.db_dc_scraper_db_connect = None
    self.properties = {}
    self.algorithm_name = None
    self.pubdate = None
    self.timezone = None
    self.message_queue = []
    self.dbHost = None
    self.dbPort = None
    self.dbUser = None
    self.dbPWD = None
    self.dc_contents_db = None
    self.entry = None
    self.article = None
    self.outputFormat = None
    self.errorMask = APP_CONSTS.ERROR_OK
    self.metrics = None
    self.altTagsMask = None
    self.tagsCount = 0
    self.tagsMask = 0
    self.processedContent = None
    self.db_task_ini = None
    self.db_dc_scraper_table = None
    self.usageModel = usageModel
    self.configFile = configFile
    self.output_data = None
    self.urlHost = None
    self.xpathSplitString = ' '
    self.useCurrentYear = 0
    self.datetimeNewsNames = []
    self.datetimeTemplateTypes = []


  # #setup
  # setup application
  def setup(self):
    if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
      # call base class setup method
      foundation.CementApp.setup(self)


  # #run
  # run application
  def run(self):
    if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
      # call base class run method
      foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load sqlite db backend
    # self.loadSqliteDBBackend()

    # mysql
    self.loadDBBackend()

    # mysql
    self.loadPropertiesFromDB()

    # options
    self.loadOptions()

    # init extractor's list
    #self.loadExtractors()

    #Do applied algorithm's job
    self.processBatch()

    if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
      # Finish logging
      self.logger.info(APP_CONSTS.LOGGER_DELIMITER_LINE)



  # #adjust partial references
  # adjust partial references
  #
  def checkDOMElement(self, elem):
    ret = False
    #valid_http_url = HttpUrl()
    if re.search('<', elem):
      self.logger.debug("Media tag contain DOM element: %s", elem)
      ret = True
    return ret


  # #adjust partial references
  # adjust partial references
  #
  def adjustPartialReferences(self, response):
    if "links" in response.tags and type(response.tags["link"]) == types.DictType and \
      "media" in response.tags and type(response.tags["media"]) == types.DictType:
      try:
        url = None
        if self.input_data.template and "link" in self.input_data.template:
          self.logger.debug("url type: %s", str(type(response.tags["link"]["data"])))
          if type(response.tags["link"]["data"]) == str or type(response.tags["link"]["data"]) == unicode:
            url = response.tags["link"]["data"]
          else:
            url = response.tags["link"]["data"][0]
        else:
          url = self.input_data.url
        if self.input_data.template and "media" in self.input_data.template:
          self.logger.debug("resource has template with media tag. Try to adjust media.")
          #if type(response.tags["media"]) == str and response.tags["media"] == "": return
          self.logger.debug("response.tags['media']: " + str(response.tags["media"]))
          self.logger.debug("media tag in response: <<%s>>" % str(response.tags["media"]["data"]))
          self.logger.debug("link tag in response: <<%s>>" % str(url))
          res = []

          filter_patterns, filter_types = [], []
          if self.input_data.filters:
            #filter_types = [filter_item["Type"] for filter_item in self.input_data.filters]
            #filter_patterns = [re.compile(filter_item["Pattern"]) for filter_item in self.input_data.filters]
            filter_types = [filter_item.type for filter_item in self.input_data.filters]
            filter_patterns = [re.compile(filter_item.pattern) for filter_item in self.input_data.filters]
          self.logger.debug("filter: %s" % (str(self.input_data.filters)))
          for media in response.tags["media"]["data"]:
            self.logger.debug("Media link: <<%s>>", media)
            # instead pure url
            if self.checkDOMElement(media):
              res.append(media)
              break
            media = urlparse.urljoin(url, media)
            valid_http_url = HttpUrl()
            for filter_type, filter_pattern in zip(filter_types, filter_patterns):
              match = filter_pattern.match(media)
              if filter_type == SiteFilter.TYPE_EXCLUDE and match:
                break
              if filter_type == SiteFilter.TYPE_INCLUDE and match:
                res.append(media)
                break
            else:
              self.logger.debug("media: %s", media)
              self.logger.debug("url: %s", url)
              #if valid_http_url(media) and Scraper.parseHost(media) == Scraper.parseHost(url):
              if valid_http_url(media):
                res.append(media)

          # If media tag after adjusting is empty remove it from response
          if not len(res):
            self.logger.debug("media tag is empty. Remove media tag from response.")
            del response.tags["media"]
          else:
            self.logger.debug("media tag is adjusted. Copy media tag to response.")
            response.tags["media"]["data"] = res
          # End code block removing empty media tag
        else:
          self.logger.debug("resource hasn't template with media tag. adjustPartialReferences doesn't execute")
      except Exception as err:
        ExceptionLog.handler(self.logger, err, MSG_ERROR_ADJUST_PR, (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    else:
      self.logger.debug(">>> Response has not have link or media tag, Don't need adjust media")


  # adjustTitle
  #
  def adjustTitle(self, response):
    try:
      if self.input_data.template and "title" in self.input_data.template and "title" in response.tags:
        self.logger.debug("resource has template with title tag. Try to adjust title.")
        self.logger.debug("response.tags['title']: " + str(response.tags["title"]))
        localExtractor = self.extractor
        if localExtractor is None:
          if len(self.extractors) > 2:
            localExtractor = self.extractors[2]
          else:
            raise Exception(">>> Wrong! self.extractors list doesn't have 3'th element (index 2)")
        if type(response.tags["title"]) == str:
          self.logger.debug("response has not have title tag")
          sel = SelectorWrapper(text=self.input_data.raw_content)
          title = sel.xpath("//title/text()").extract()
          localExtractor.addTag(result=response, tag_name="title", tag_value=title)
        self.logger.debug("TYPE response.tags['title']['data']" + str(type(response.tags["title"]["data"])))
      else:
        self.logger.debug("resource hasn't template with title tag. Don't need adjust title.")
    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_ADJUST_TITLE, (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})


  # adjustLinkURL
  #
  def adjustLinkURL(self, response):
    flag = False
    try:
      if response.tags and "link" in response.tags:
        self.logger.debug("resource has template with link tag. Try to adjust link.")
        self.logger.debug("response.tags['link']: " + str(response.tags["link"]))
        self.logger.debug("self.extractor: %s", str(self.extractor))
        flag = True
        if self.extractor:
          self.logger.debug("Extractor exists")
          if type(response.tags["link"]) == str:
            self.logger.debug("response has not have link tag")
            self.extractor.addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
          # bypass
          else:
            response.tags["link"]["data"] = self.input_data.url
        else:
          if len(self.extractors) > 2:
            self.extractors[2].addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
          else:
            self.logger.debug(">>> Wrong! self.extractors list doesn't have 3'th element (index 2)")
        self.logger.debug("TYPE response.tags['link']['data']" + str(type(response.tags["link"]["data"])))
      else:
        self.logger.debug("resource hasn't template with link tag. Don't need adjust link.")
    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_ADJUST_PR, (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return flag


  ## Normalize datetime tags procedure
  #
  #@param response - scraper response instance
  #@return - 'pubdate tag value'
  def normalizeDatetime(self, response):
    ret = None
    try:
      if response is not None and response.tags is not None:
        self.logger.debug("normalizeDatetime scraper response: " + varDump(response))

        tagNames = []
        if self.input_data.template and self.algorithm_name == CONSTS.PROCESS_ALGORITHM_REGULAR:
          #temlate

          for responseType in self.datetimeTemplateTypes:
            for responseTagName in response.tags:
              self.logger.debug("normalizeDatetime responseTagName: '" + str(responseTagName) + "'")

              if (responseTagName in response.tags and response.tags[responseTagName] is not None and \
              response.tags[responseTagName].has_key('type') and \
              response.tags[responseTagName]['type'] == responseType) or \
              (responseTagName in response.tags and response.tags[responseTagName] is not None and \
              responseTagName == CONSTS.TAG_PUB_DATE):
                tagNames.append(responseTagName)
              else:
                pass
        else:
          #dynamic
          tagNames = self.datetimeNewsNames

        self.logger.debug('normalizeDatetime  tagNames: ' + varDump(tagNames))

        for tagName in tagNames:
          pubdate = self.extractPubDate(response, tagName)
          if self.extractor and tagName in response.tags:
            self.extractor.addTag(result=response, tag_name=tagName + '_normalized', tag_value=pubdate, \
                                  xpath=response.tags[tagName]['xpath'])

          if tagName == CONSTS.TAG_PUB_DATE:
            ret = pubdate
          else:
            pass

    except Exception, err:
      ExceptionLog.handler(self.logger, err, 'normalizeDatetime error:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return ret


  ## Extract pubdate
  #
  #@param response - response instance
  #@param dataTagName - tag name for extracting
  #@return pubdate if success or None
  def extractPubDate(self, response, dataTagName):
    pubdate = None
    try:
      if response is not None and dataTagName in response.tags and response.tags[dataTagName] != "":

        #self.logger.debug("extractPubDate response: " + varDump(response))

        inputData = response.tags[dataTagName]["data"]
        self.logger.debug("extractPubDate response has '" + str(dataTagName) + "' is: " + str(inputData))
        self.logger.debug("extractPubDate type of '" + str(dataTagName) + "' is: " + str(type(inputData)))

        inputList = []
        if type(inputData) in types.StringTypes:
          inputList = [inputData]
        elif isinstance(inputData, list):
          inputList = inputData
        else:
          pass

        pubdate = []
        for inputElem in inputList:
          d = DateTimeType.parse(inputElem, bool(self.useCurrentYear), self.logger)
          if d is not None:
            pubdate.append(d.isoformat(DateTimeType.ISO_SEP))
          else:
            pubdate.append('')

        self.logger.debug("extractPubDate result pubdate: " + str(pubdate))
        if len(pubdate) > 0:
          response.tags[dataTagName]["data"] = pubdate
        else:
          pubdate = None

    except Exception, err:
      ExceptionLog.handler(self.logger, err, 'extractPubDate error:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return pubdate


  ## refineBadDateTags, deleles, from result, datetime tags with bad datetime value.
  #
  def refineBadDateTags(self, response):
    removeKeys = []
    for key in response.tags:
      if key in DATA_NEWS_TAGS:
        tagsValue = None

        if type(response.tags[key]) in types.StringTypes:
          tagsValue = response.tags[key]
        elif type(response.tags[key]) is types.DictType and "data" in response.tags[key]:
          if type(response.tags[key]["data"]) in types.StringTypes:
            tagsValue = response.tags[key]["data"]
          elif type(response.tags[key]["data"]) is types.ListType and len(response.tags[key]["data"]) > 0 and \
          type(response.tags[key]["data"][0]) in types.StringTypes:
            tagsValue = response.tags[key]["data"][0]

        if tagsValue is not None:
          try:
            dt = parser.parse(tagsValue)
            int(time.mktime(dt.timetuple()))
          except Exception:
            removeKeys.append(key)

    for key in removeKeys:
      if key in response.tags:
        logging.debug(">>> Remove " + key + " element besause it empty")
        del response.tags[key]


  # #Internal method of url's domain crc calculating
  #
  # @param url - incoming url
  def calcUrlDomainCrc(self, url):
    auth = urlparse.urlsplit(url)[1]
    if auth is not None:
      self.urlHost = (re.search('([^@]*@)?([^:]*):?(.*)', auth).groups())[1]
      if self.urlHost is not None and self.urlHost.find(WWW_PREFIX) == 0:
        self.urlHost = self.urlHost[len(WWW_PREFIX): len(self.urlHost)]


  # #main content processing
  # main content processing
  #
  def process(self):
    # info input data
    self.logger.info("input_data url: %s, urlId: %s, siteId: %s", str(self.input_data.url), str(self.input_data.urlId),
                     str(self.input_data.siteId))
    if self.input_data.template and self.algorithm_name == CONSTS.PROCESS_ALGORITHM_REGULAR:
      # Reconfigure processor's properties to involve only template scraper
      responses = self.templateExtraction()
    else:
      # get iterator to ranked list of extractors
      self.itr = iter(sorted(self.extractors, key=lambda extractor: 0, reverse=True))
      self.logger.debug("Extractors: %s" % varDump(self.itr))
      responses = self.dynamicExtraction()

    for response in responses:
      response.metricsPrecalculate()
      response.stripResult()

      # put extracted article to the db

      if self.algorithm_name != CONSTS.PROCESS_ALGORITHM_REGULAR:
        self.adjustTitle(response)
        self.adjustLinkURL(response)
        self.adjustPartialReferences(response)

        self.logger.debug("CONSTS.TAG_PUB_DATE response: " + varDump(response))

      self.preparseResponse(response)

      # Setting pubdate in depend of different sources masks
      pdateSourceMask = CONSTS.PDATE_SOURCES_MASK_BIT_ALL
      if CONSTS.PDATE_SOURCES_MASK_PROP_NAME in self.properties:
        pdateSourceMask = self.properties[CONSTS.PDATE_SOURCES_MASK_PROP_NAME]

      if pdateSourceMask & CONSTS.PDATE_SOURCES_MASK_BIT1:
        self.pubdate = self.normalizeDatetime(response)

      if (self.input_data.batch_item.urlObj.pDate is not None) and \
      (type(self.input_data.batch_item.urlObj.pDate) != type(SQLExpression())) and \
      pdateSourceMask & CONSTS.PDATE_SOURCES_MASK_BIT2:
        data = {"extractor":"Taken from rss feed parser", "data":"", "name":"pubdate"}
        data["data"] = self.input_data.batch_item.urlObj.pDate
        response.tags[CONSTS.TAG_PUB_DATE] = data
      elif CONSTS.TAG_DC_DATE in response.tags and pdateSourceMask & CONSTS.PDATE_SOURCES_MASK_BIT3:
        if CONSTS.TAG_PUB_DATE not in response.tags or \
        (type(response.tags[CONSTS.TAG_PUB_DATE]["data"]) in types.StringTypes and \
        response.tags[CONSTS.TAG_PUB_DATE]["data"].strip() == ""):
          response.tags[CONSTS.TAG_PUB_DATE] = copy.deepcopy(response.tags[CONSTS.TAG_DC_DATE])
          response.tags[CONSTS.TAG_PUB_DATE]["name"] = CONSTS.TAG_PUB_DATE
        else:
          pass
      if self.pubdate is None and self.input_data.batch_item.urlObj.lastModified is not None and \
      pdateSourceMask & CONSTS.PDATE_SOURCES_MASK_BIT4:
        self.pubdate = str(self.input_data.batch_item.urlObj.lastModified)
      else:
        pass

      if self.pubdate is None and pdateSourceMask & CONSTS.PDATE_SOURCES_MASK_BIT5:
        self.pubdate = str(SQLExpression("NOW()"))

      if self.outputFormat is not None:
        self.formatOutputData(response, self.outputFormat)
      else:
        self.logger.debug(">>> Warning, can't extracr output format")
      response.recalcTagMaskCount(None, self.altTagsMask)
      self.tagsCount = response.tagsCount
      self.tagsMask = response.tagsMask
      #self.putArticleToDB({"default":response})
      self.logger.debug("self.tagsCount: %s", self.tagsCount)
      self.logger.debug("self.tagsMasks: %s", self.tagsMask)

      self.logger.debug(">>> Resp: %s\n", varDump(response))

      #TODO: Seems need to be done more system way
      response.finish = time.time()
      response.data["time"] = "%s" % (response.finish - response.start)

    self.getProcessedContent(responses)
    #return response


  def preparseResponse(self, response):
    for key in response.tags:
      if "data" in response.tags[key]:
        if type(response.tags[key]["data"]) in types.StringTypes:
          localStr = response.tags[key]["data"]
          response.tags[key]["data"] = []
          response.tags[key]["data"].append(localStr)


  def formatOutpuElement(self, elem, localOutputFormat):
    ret = elem
    if localOutputFormat == "json":
      #self.logger.debug(">>> JSON HTML = " + elem)
      localStr = json.dumps(elem, ensure_ascii=False)
      if localStr[0] == '\"' or localStr[0] == '\'':
        localStr = localStr[1:]
      if localStr[-1] == '\"' or localStr[-1] == '\'':
        localStr = localStr[0:-1]
      ret = localStr
      #self.logger.debug(">>> JSON HTML = " + ret)
    elif localOutputFormat == "html" or localOutputFormat == "xml":
      ret = xml.sax.saxutils.escape(elem, {"'": "&apos;", "\"" : "&quot;"})
    elif localOutputFormat == "sql":
      ret = mdb.escape_string(elem)  # pylint: disable=E1101
    return ret


  def formatOutputData(self, response, localOutputFormat):
    #result.tags[key]["data"]
    for key in response.tags:
      if "data" in response.tags[key]:
        if type(response.tags[key]["data"]) == types.ListType:
          for i, elem in enumerate(response.tags[key]["data"]):
            response.tags[key]["data"][i] = self.formatOutpuElement(elem, localOutputFormat)
        elif type(response.tags[key]["data"]) in types.StringTypes:
          response.tags[key]["data"] = self.formatOutpuElement(response.tags[key]["data"], localOutputFormat)


  def templateExtraction(self):
    self.extractor = ScrapyExtractor(self.config, self.input_data.template, self.urlHost)
    resultsList = []
    # create empty result
    result = Result(None, self.input_data.urlId, self.metrics)
    sel = SelectorWrapper(text=self.input_data.raw_content)
    if type(self.input_data.template) == type({}):
      template = self.input_data.template
    else:
      template = ast.literal_eval(self.input_data.template)
    #Andrey Add
    #self.logger.debug(">>> Dict == " + str(template))
    # Add End
    for key in template:
      self.logger.debug(">>> Template key: " + key)
      if "state" in template[key] and not bool(int(template[key]["state"])):
        self.logger.debug(">>> Template disable: template name = " + str(key))
        continue
      xPathPreparing = TemplateExtractorXPathPreparing(self.properties[CONSTS.TAG_MARKUP_PROP_NAME] \
                                                      if CONSTS.TAG_MARKUP_PROP_NAME in self.properties else None)
      for path in template[key]:
        if type(path) != type({}):
          self.logger.debug(">>> WARNING path not DICT type ")
          continue
        pathDict = Utils.getPairsDicts(path)
        isExtract = True
        localResult = Result(None, self.input_data.urlId, self.metrics)
        # Added new template format conversion
        xpath = None
        xpathValue = None
          # Added new template type specification
        self.xpathSplitString = xPathPreparing.resolveDelimiter(path, self.properties, self.xpathSplitString)
        try:
          xpath, xpathValue = xPathPreparing.process(path, sel, self.xpathSplitString)
        except Exception as excp:
          ExceptionLog.handler(self.logger, excp, ">>> some rule/xpath exception =", (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
          continue

        self.logger.debug(">>> xpathValue " + str(type(xpathValue)) + " " + str(xpathValue))
        self.logger.debug(">>> xpath: %s" % str(xpath))
        if (type(xpathValue) == types.ListType and len(xpathValue) == 0) or\
        (type(xpathValue) in types.StringTypes and xpathValue == ''):
          self.logger.debug(">>> set default xpathValue")
          xpathValue = []
          xpathValue.append(path["default"])
          isExtract = False
        self.logger.debug("result before:\n%s", varDump(localResult))
        self.extractor.addTag(localResult, key, xpathValue, xpath, not isExtract, False, path["type"])
        self.logger.debug("reslt after:\n%s", varDump(localResult))

        self.logger.debug(">>> tag type = " + str(type(localResult.tags)))
        self.logger.debug(">>> tags data type = " + str(type(localResult.tags[key]["data"])))
        if key in localResult.tags and type(localResult.tags[key]["data"]) in types.StringTypes:
          self.logger.debug(">>> Convert result = " + str(key))
          localString = localResult.tags[key]["data"]
          localResult.tags[key]["data"] = []
          localResult.tags[key]["data"].append(localString)

        self.formatTag(localResult, path, key, pathDict, isExtract)

        if isExtract and "postProcessing" in path and path["postProcessing"] is not None and \
        path["postProcessing"] != "":
          self.applyPostProcessing(localResult, key, path["postProcessing"])

        resultsList.append({"obj": localResult, "join": path["join"], "isExtract": isExtract, "mandatory":
                            (bool(path["mandatory"]) if "mandatory" in path else False),
                            "delimiter": (path["delimiter"] if "delimiter" in path else self.xpathSplitString),
                            "type": path["type"]})

      prepareResultsList = self.prepareResults(resultsList)
      self.compileResults(result, prepareResultsList, key, xPathPreparing)
      resultsList = []
      #self.formatOutputData(path[])
      #result = Result(self.config, self.input_data.urlId, self.metrics)
      #for
      #result.tags.update(localResult.tags)
      result.finish = time.time()
    return [result]


  def compileResults(self, result, resultsList, key, xPathPreparing=None):
    for elem in resultsList:
      if key in result.tags:
        if result.tags[key]["xpath"] == None:
          result.tags[key]["xpath"] = elem["obj"].tags[key]["xpath"]
        else:
          result.tags[key]["xpath"] += ' '
          result.tags[key]["xpath"] += elem["obj"].tags[key]["xpath"]
        if result.tags[key]["data"] == None or len(result.tags[key]["data"]) == 0:
          result.tags[key]["data"] = elem["obj"].tags[key]["data"]
        else:
          if xPathPreparing is not None:
            self.xpathSplitString = xPathPreparing.resolveDelimiter(elem, self.properties, self.xpathSplitString)
          result.tags[key]["data"][0] += self.xpathSplitString
          result.tags[key]["data"][0] += elem["obj"].tags[key]["data"][0]
      else:
        result.tags.update(elem["obj"].tags)


  def prepareResults(self, resultsList):
    ret = []
    if len(resultsList) > 0:
      localElemWeight = 0
      firstElemWeight = 0
      firstElem = None
      tempList = []
      for elem in resultsList:
        localElemWeight = 0
        if elem["join"] == "concat":
          tempList.append(elem)
        else:
          if elem["mandatory"]:
            #>>> Mandatory breaking block -------------
            if not elem["isExtract"]:
              return []
            #-------------
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_MANDATORY_FIELD
          if elem["join"] == "best":
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_RULE_PRIORITY
          if elem["isExtract"]:
            localElemWeight = localElemWeight | CONSTS.TAGS_RULES_MASK_DEFAULT_VALUE

          self.logger.debug(">>> Rule weight = " + str(localElemWeight))
          self.logger.debug(">>> Rule join = " + elem["join"])
          if localElemWeight > firstElemWeight:
            firstElemWeight = localElemWeight
            firstElem = elem

      if firstElem is not None:
        tempList = [firstElem] + tempList
      isExtractResults = any([elem["isExtract"] for elem in tempList])
      if isExtractResults:
        ret = [elem for elem in tempList if elem["isExtract"]]
      else:
        ret.append(tempList[0])
    return ret


  # #dataUrlsCanonizator canonizates incoming string buf as url string, parses it before by " " and "," symbols
  #
  # @param data - incoming string/strings list buffer
  # @param baseUrl - base url
  # @return - canonizated url string
  def elemUrlsCanoizator(self, data, baseUrl=None, firstDelim=" ", secondDelim=",", useAdditionEncoding=False):
    ret = data
    if data.strip() != "":
      ret = ""
      for elem in data.split(firstDelim):
        result = ""
        for elem1 in elem.split(secondDelim):
          if elem1.strip() != "":
            localUrl = elem1
            if baseUrl is not None:
              localUrl = urlparse.urljoin(baseUrl, localUrl)
            processedUrl = dc.EventObjects.URL(0, localUrl).getURL()
            if useAdditionEncoding:
              processedUrl = xml.sax.saxutils.escape(processedUrl, {})
            result += processedUrl + secondDelim
          else:
            result += elem1 + secondDelim
        if result != "" and result[-1] == secondDelim:
          result = result[0: len(result) - 1]
        ret += result + firstDelim
      if ret != "" and ret[-1] == firstDelim:
        ret = ret[0: len(ret) - 1]
    return ret


  # #dataUrlsCanonizator canonizates incoming string/list as url string
  #
  # @param data - incoming string/strings list buffer
  # @param baseUrl - base url
  # @return - canonizated url string
  def dataUrlsCanonizator(self, data, baseUrl=None, useAdditionEncoding=False):
    ret = data
    #self.logger.debug(">>> url canonizator = " + str(data))
    if type(data) in types.StringTypes:
      ret = self.elemUrlsCanoizator(data, baseUrl, useAdditionEncoding=useAdditionEncoding)
    elif type(data) is types.ListType:
      ret = []
      for elem in data:
        elem = self.elemUrlsCanoizator(elem, baseUrl, useAdditionEncoding=useAdditionEncoding)
        ret.append(elem)
    return ret


  ## formatTag
  #
  def formatTag(self, result, path, key, pathDict, isExtract):
    #Andrey Add
    self.logger.debug("Tag type: %s, tag format: %s" % (str(path["type"]), str(path["format"])))
    #Add End
    if path["type"] == "text":
      localText = ''
      for elem in result.tags[key]["data"]:
        localText += (elem.strip() + self.xpathSplitString)
      localText = localText.strip(self.xpathSplitString)
      localMaxCh = None
      if "format" in pathDict and "maxCh" in pathDict["format"]:
        self.logger.debug(">>> get LocalMaxCh from DICT[\"format\"][\"maxCh\"]")
        localMaxCh = pathDict["format"]["maxCh"]
      else:
        self.logger.debug(">>> get LocalMaxCh from [\"format\"]")
        localMaxCh = path["format"]
      try:
        if localMaxCh is not None and int(localMaxCh) > 0 and len(localText) > int(localMaxCh):
          localText = localText[0: int(localMaxCh)]
      except ValueError:
        self.logger.debug(">>> Wrong maxCh value")
      result.tags[key]["data"] = []
      result.tags[key]["data"].append(localText)
    elif path["type"] == "html":
      #>>> html
      for i, elem in enumerate(result.tags[key]["data"]):
        result.tags[key]["data"][i] = re.sub(r"\<![ \r\n\t]*(--([^\-]|[\r\n]|-[^\-])*--[ \r\n\t]*)\>", "", elem)
      self.logger.debug(">>> After RE = " + str(result.tags[key]["data"]))
      #>>> html END
    elif path["type"] == "datetime":
      #>>> datetime
      bestData = ''
      try:
        self.logger.debug("Try to convert data")
        if not isExtract:
        #New use default value as a format string for current date
          if len(result.tags[key]["data"][0]) > 0 and result.tags[key]["data"][0][0] == '@':
            localFormatStr = result.tags[key]["data"][0][1: len(result.tags[key]["data"][0])]
            localTm = datetime.datetime.fromtimestamp(time.time())
            result.tags[key]["data"][0] = datetime.datetime.strftime(localTm, localFormatStr)
        else:
          bestData = self.getBestDatatimeData(result.tags[key]["data"])
          self.logger.debug(">>> Time log Before = " + bestData)
          if path["format"] != "" and path["format"] != "FULL":
            result.tags[key]["data"][0] = datetime.datetime.strftime(parser.parse(bestData), path["format"])
          else:
            result.tags[key]["data"][0] = str(parser.parse(bestData))
        self.logger.debug(">>> Time log after = " + result.tags[key]["data"][0])
      except Exception as err:
        self.logger.debug("Can't convert data <<< " + str(result.tags) + " " + str(key) + " err = " + str(err))
        result.tags[key]["data"][0] = bestData
      if len(result.tags[key]["data"]) > 0:
        result.tags[key]["data"] = [result.tags[key]["data"][0]]
      #>>> datetime END
    elif path["type"] == "image":
      if path["format"] == "URL" and "canonicalizeURLs" in path and int(path["canonicalizeURLs"]) == 1:
        result.tags[key]["data"] = self.dataUrlsCanonizator(result.tags[key]["data"], self.input_data.url)
    elif path["type"] == "link":
      formatName = path["format"]
      if len(formatName.split(',')) > 1:
        formatName = formatName.split(',')[1]
      if formatName == "email-address" or formatName == "email-to":
        localText = ''
        if type(result.tags[key]["data"]) in types.StringTypes:
          self.logger.debug(">>> mail to str type")
          localText = result.tags[key]["data"].strip(self.xpathSplitString)
          index = localText.find("mailto:")
          if index >= 0:
            localText = localText[index + len("mailto:"), len(localText)]
          else:
            localText = ""
        elif type(result.tags[key]["data"]) is types.ListType:
          self.logger.debug(">>> mail to list type")
          for elem in result.tags[key]["data"]:
            elemText = elem.strip(self.xpathSplitString)
            index = elemText.find("mailto:")
            if index >= 0:
              elemText = elemText[index + len("mailto:"): len(elemText)]
              if formatName == "email-address":
                elemText = Utils.emailParse(elemText)
              else:
                elemText = Utils.emailParse(elemText, True)
            else:
              elemText = ""
            if elemText != "":
              localText += (elemText + self.xpathSplitString)

        result.tags[key]["data"] = []
        result.tags[key]["data"].append(localText)
      if "canonicalizeURLs" in path and int(path["canonicalizeURLs"]) == 1:
        result.tags[key]["data"] = self.dataUrlsCanonizator(result.tags[key]["data"], self.input_data.url)
    elif path["type"] == "attribute":
      if isExtract:
        localText = ''
        if type(result.tags[key]["data"]) in types.StringTypes:
          localText = result.tags[key]["data"]
        elif type(result.tags[key]["data"]) is types.ListType:
          localText = self.xpathSplitString.join([elem for elem in result.tags[key]["data"] if elem != ''])
        splittedFormatString = path["format"].split(',')
        if len(splittedFormatString) >= 2:
          try:
            if int(splittedFormatString[0]) < len(localText):
              localText = localText[0: int(splittedFormatString[0])]
          except Exception as err:
            self.logger.debug("Error: %s; Wrong path format for attribute rule, format=%s", str(err), path["format"])
        result.tags[key]["data"] = []
        result.tags[key]["data"].append(localText)

    localElem = ''
    for elem in result.tags[key]["data"]:
      localElem += elem
      localElem += self.xpathSplitString
    result.tags[key]["data"][0] = localElem
    result.tags[key]["data"][0] = result.tags[key]["data"][0].strip(self.xpathSplitString)


  def applyPostProcessing(self, result, key, postProcessingRE):
    if key in result.tags and "data" in result.tags[key] and result.tags[key]["data"] is not None and \
    len(result.tags[key]["data"]) > 0:
      try:
        matchingVal = re.compile(postProcessingRE)
      except re.error as err:
        self.logger.debug(">>> RE error = " + str(err))
        self.errorMask = self.errorMask | APP_CONSTS.ERROR_RE_ERROR
      else:
        tmpStr = ""
        matchingResult = matchingVal.findall(result.tags[key]["data"][0])
        if matchingResult is not None:
          for elem in matchingResult:
            if type(elem) in types.StringTypes:
              tmpStr += str(elem)
              tmpStr += self.xpathSplitString
            else:
              for innerElem in elem:
                if innerElem is not None and innerElem != '':
                  tmpStr += str(innerElem)
                  tmpStr += self.xpathSplitString
        tmpStr = tmpStr.strip(self.xpathSplitString)
        if tmpStr != "":
          self.logger.debug(">>> Replace value, prev. value is = " + result.tags[key]["data"][0])
          result.tags[key]["data"][0] = tmpStr


  def processingHTMLData(self, htmlBuf, bufFormat):
    ret = htmlBuf
    if bufFormat.find("NO_SCRIPT") >= 0:
      ret = Utils.stripHTMLComments(htmlBuf, soup=None)
    if bufFormat.find("NO_META") >= 0:
      pass
    if bufFormat.find("NO_COMMENTS") >= 0:
      pass
    if bufFormat.find("ENTITIES_ENCODED") >= 0:
      pass
    return ret


  def getBestDatatimeData(self, data):
    ret = ""
    if type(data) == types.ListType:
      for elem in data:
        for ch in elem:
          if ch >= '0' and ch <= '9':
            ret = elem
            break
        if ret is not None:
          break
      if ret is None:
        ret = data[0]
    else:
      ret = data
    if type(ret) in types.StringTypes:
      ret = ret.replace('\n', '')
      ret = ret.replace('\t', '')
    else:
      ret = ""
    return ret


  def dynamicExtraction(self):
    ret = []
    # get resource as dictionary
    # resource_set = self.getResourceFromDB(resId)
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    # self.logger.info("raw_content: " + str(self.input_data.raw_content))
    resource = Res(resource_set)

    # create empty result
    #result = Result(self.config, self.input_data.urlId, self.metrics)
    #result.metrics.append(WordsMetric())

    # get list of empty tags from result
    #empty_tags = result.getEmptyTags()
    #self.logger.debug("get list of empty tags from result: " + str(empty_tags))

    # get best matching extractor
    self.extractor = self.getNextBestExtractor()
    self.logger.debug("get best matching extractor: " + str(self.extractor))

    # search engine parsing ???
    '''
    if not resource.site_id:
      return self.extractor.extractTags(resource, result)
    '''
    collectResult = Result(self.config, self.input_data.urlId, self.metrics)
    # main loooop
    while self.extractor:
      result = Result(self.config, self.input_data.urlId, self.metrics)
      self.logger.debug(">>> TAG BEGIN extractor = " + str(self.extractor))
      result = self.extractor.extractTags(resource, result)

      self.logger.debug(">>> TAG END")
      #result = Utils.timeLimit(self.extractor.extractTags(resource, result),10)
      # metric support
      #self.extractor.calculateMetrics(result)
      # print result.tags
      empty_tags = result.getEmptyTags()
      self.logger.debug("get list of empty tags from result: " + str(empty_tags))
      filled_tags = result.getFilledTags()
      self.logger.debug("get list of filled_tags from result: " + str(filled_tags))
      self.extractor = self.getNextBestExtractor()
      self.logger.debug("get best matching extractor: " + str(self.extractor))

      self.commonResultOperations(result)
      for key in result.tags:
        if key not in collectResult.tags or not collectResult.isTagFilled(key):
          collectResult.tags[key] = copy.deepcopy(result.tags[key])
      ret.append(result)
    self.logger.debug(">>> EXIT LOOP")
    ret = [collectResult] + ret
    return ret


  def commonResultOperations(self, result):
    empty_tags = result.getEmptyTags()
    for localKey in EXTENDED_NEWS_TAGS:
      if localKey in empty_tags or (localKey in result.tags and result.isTagFilled(localKey) is False):
        self.extractAdditionTagsByScrapy(result, localKey, EXTENDED_NEWS_TAGS[localKey])
    for tagName in LINKS_NEWS_TAGS:
      if tagName in result.tags:
        if type(result.tags[tagName]) is types.DictType and (result.tags[tagName]["xpath"] == "" or \
        result.tags[tagName]["xpath"].find("/@src") != -1 or result.tags[tagName]["xpath"].find("/@href") != -1):
          result.tags[tagName]["data"] = \
          self.dataUrlsCanonizator(result.tags[tagName]["data"], self.input_data.url)

    self.refineCommonText(CONSTS.TAG_CONTENT_UTF8_ENCODED, result)
    self.refineBadDateTags(result)
    result.finish = time.time()


  def replaceLoopValue(self, buf, replaceFrom, replaceTo):
    localValue = buf
    replaceValue = localValue.replace(replaceFrom, replaceTo)
    while len(replaceValue) != len(buf):
      localValue = replaceValue
      replaceValue = localValue.replace(replaceFrom, replaceTo)
    return localValue


  def refineCommonText(self, tagName, result):
    if tagName in result.tags:
      if type(result.tags[tagName]) is types.DictType:
        localValue = None
        if type(result.tags[tagName]["data"]) == types.ListType and len(result.tags[tagName]["data"]) > 0:
          localValue = result.tags[tagName]["data"][0]
        elif type(result.tags[tagName]["data"]) in types.StringTypes:
          localValue = result.tags[tagName]["data"]
        if localValue is not None:
          replaceList = None
          if CONSTS.TAG_REDUCE_PROP_NAME in self.properties:
            try:
              replaceList = json.loads(self.properties[CONSTS.TAG_REDUCE_PROP_NAME])
            except Exception:
              self.logger.debug(">>> Bad processor_property json format, [" + CONSTS.TAG_REDUCE_PROP_NAME + "]")
          if replaceList is None:
            replaceList = json.loads(CONTENT_REPLACEMENT)
          if " " not in replaceList:
            replaceList.append(" ")
          self.logger.debug(">>> Repl list = " + str(replaceList))
          for elem in replaceList:
            #self.logger.debug(">>> Value before = " + localValue)
            localValue = Utils.replaceLoopValue(localValue, (elem * 2), elem)
            #self.logger.debug(">>> Value after = " + localValue)
          localValue = localValue.replace("\r", " ")

          if type(result.tags[tagName]["data"]) == types.ListType and len(result.tags[tagName]["data"]) > 0:
            result.tags[tagName]["data"][0] = localValue
          elif type(result.tags[tagName]["data"]) in types.StringTypes:
            result.tags[tagName]["data"] = localValue


  def extractAdditionTagsByScrapy(self, localResult, key, tagsXpaths):
    self.logger.debug(">>> Start addition news extracting")
    extractor = self.getExtractorByName("ScrapyExtractor")
    if extractor is not None:
      sel = SelectorWrapper(text=self.input_data.raw_content)
      for tagsXpath in tagsXpaths:
        if tagsXpath is not None and tagsXpath != "":
          localXpath = sel.xpath(tagsXpath)
          localValue = Utils.innerText(localXpath, ' ', ' ', self.properties[CONSTS.TAG_MARKUP_PROP_NAME] \
                                      if CONSTS.TAG_MARKUP_PROP_NAME in self.properties else None)
          if localValue != "":
            extractor.addTag(localResult, key, localValue, tagsXpath)
            break
          else:
            self.logger.debug(">>> Cant extract tag=%s for xpath=%s" % (key, tagsXpath))


  def getNextBestExtractor(self):
    # return extractor with highest rank
    try:
      extractor = next(self.itr)
    except StopIteration:
      extractor = None
    return extractor


  def getProcessedContent(self, result):
    for elem in result:
      elem.get()
    self.processedContent = {}
    self.processedContent["default"] = result[0]
    self.processedContent["internal"] = result
    self.processedContent["custom"] = []
    self.tagsCount = result[0].tagsCount
    self.tagsMask = result[0].tagsMask


  # #load extractors
  #
  def loadExtractors(self):
    try:
      # modules
      if CONSTS.MODULES_KEY in self.properties and self.algorithm_name in self.properties[CONSTS.MODULES_KEY]:
        modules = self.properties[CONSTS.MODULES_KEY][self.algorithm_name]
      else:
        self.logger.debug(">>> No moduler_key or algorithm_name in self.properties")
        modules = []

      self.logger.debug("Algorithm name: <%s>" % (self.algorithm_name))
      self.logger.debug("Modules: %s" % modules)

      self.extractors = []
      for module in modules:
        exrtactor = self.createModule(module)
        # Check if module was created successfully and then insert it to extractors
        if exrtactor is not None:
          self.extractors.append(exrtactor)

      # Info show extractors loaded
      self.logger.debug("*******************")
      self.logger.debug("Loaded extractors:")
      for exrtactor in self.extractors:
        self.logger.debug(exrtactor.name)
      self.logger.debug("*******************")

    except Exception as err:
      ExceptionLog.handler(self.logger, err, MSG_ERROR_LOAD_EXTRACTORS)
      raise


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    # logger
    for entry in self.message_queue:
      self.logger.debug(entry)

    if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
      # read pickled batch object from stdin
      input_pickled_object = sys.stdin.read()

    try:
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        scraper_in_data = pickle.loads(input_pickled_object)
    except Exception as err:
      ExceptionLog.handler(self.logger, err, 'pickle.loads() error:')
      self.logger.debug("input_pickled_object:\n" + str(input_pickled_object))
      self.exitCode = EXIT_FAILURE
      raise Exception(err)

    try:
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        self.input_data = scraper_in_data
      if self.input_data.batch_item.urlObj is not None:
        urlString = self.input_data.batch_item.urlObj.url
      else:
        urlString = ""
      logMsg = "BatchItem.siteId=" + str(self.input_data.batch_item.siteId) + \
               ", BatchItem.urlId=" + str(self.input_data.batch_item.urlId) + \
               ", BatchItem.urlObj.url=" + urlString
      app.Profiler.messagesList.append(logMsg)
      self.logger.info("Incoming data: %s", logMsg)
      #self.logger.debug("self.input_data:\n%s", varDump(self.input_data))
      self.calcUrlDomainCrc(self.input_data.url)

      if "templates" in self.input_data.batch_item.properties["template"] and \
          len(self.input_data.batch_item.properties["template"]["templates"]) > 0 and \
          "output_format" in self.input_data.batch_item.properties["template"]["templates"][0] and \
          "name" in self.input_data.batch_item.properties["template"]["templates"][0]["output_format"]:
        self.outputFormat = self.input_data.batch_item.properties["template"]["templates"][0]["output_format"]["name"]

      if "TAGS_MAPPING" in self.input_data.batch_item.properties and \
      self.input_data.batch_item.properties["TAGS_MAPPING"] is not None:
        try:
          self.altTagsMask = json.loads(self.input_data.batch_item.properties["TAGS_MAPPING"])
          self.logger.debug(">>> AltTags = " + str(self.altTagsMask))
        except Exception as exp:
          self.logger.debug(">>> Bad TAGS_MAPPING properties value, err=" + str(exp))
      # check properties in input data
      try:
        if (self.input_data is not None) and (self.input_data.processor_properties is not None):
          processor_properties = self.input_data.processor_properties
          self.logger.debug("Processor's properties was taken from input data: %s" % processor_properties)
          self.logger.debug("Processor's properties type: %s" % str(type(processor_properties)))
          if type(processor_properties) != type({}):
            processor_properties = json.loads(self.input_data.processor_properties)
          self.logger.debug("Processor's properties was taken from input data: %s" % processor_properties)
          self.properties.update(processor_properties)
      except Exception as err:
        ExceptionLog.handler(self.logger, err, 'Error load properties from input data:')

      self.algorithm_name = self.properties[CONSTS.ALGORITHM_KEY][CONSTS.ALGORITHM_NAME_KEY]
      self.logger.debug("Algorithm : %s" % self.algorithm_name)
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        Utils.storePickleOnDisk(input_pickled_object, ENV_SCRAPER_STORE_PATH, "scraper.in." + \
                                 str(self.input_data.urlId))
      if "metrics" in self.properties:
        try:
          self.metrics = json.loads(self.properties["metrics"])
          self.logger.debug(">>> Metrics loads" % str(self.metrics))
        except Exception as excp:
          self.logger.debug(">>> Metrcis dumps exception = " + str(excp))
      # TODO main processing over every url from list of urls in the batch object
      tmp = sys.stdout
      sys.stdout = open("/dev/null", "wb")

      # initialization of scraper
      # load scraper's modules
      self.loadExtractors()

      self.logger.info("Process with extractor algorithm: " + str(self.algorithm_name))
      # SUPPORT METRICS ALGORITHTM
      #if self.algorithm_name == CONSTS.PROCESS_ALGORITHM_METRIC:
        #self.processMetrics()
      # SUPPORT FEED_PARSER ALGORITHTM
      if self.algorithm_name == CONSTS.PROCESS_ALGORITHM_FEED_PARSER:
        self.feedParserProcess()
      else:
        self.process()

      # send response to the stdout
      sys.stdout = tmp

      scraperResponse = ScraperResponse(self.tagsCount, self.tagsMask, self.pubdate, self.processedContent,
                                        self.errorMask)
      self.logger.debug("scraperResponse:\n%s", varDump(scraperResponse))
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        output_pickled_object = pickle.dumps(scraperResponse)
        Utils.storePickleOnDisk(output_pickled_object, ENV_SCRAPER_STORE_PATH,
                                "scraper.out." + str(self.input_data.urlId))
        print output_pickled_object
        sys.stdout.flush()
      else:
        self.output_data = scraperResponse

    except Exception as err:
      ExceptionLog.handler(self.logger, err, 'Scraper process batch error:')
      self.exitCode = EXIT_FAILURE
      raise Exception('Scraper process batch error:' + str(err))



  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        if self.pargs.config:
          self.config.read(self.pargs.config)
        else:
          self.config.read(APP_NAME)
      else:
        self.config.read(self.configFile)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      if self.usageModel == APP_CONSTS.APP_USAGE_MODEL_PROCESS:
        log_conf_file = self.config.get("Application", "log")
        logging.config.fileConfig(log_conf_file)
        # Logger initialization
        self.logger = Utils.MPLogger().getLogger()
    except Exception, err:
      raise Exception(CONSTS.MSG_ERROR_LOAD_CONFIG + " : " + str(err))


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      class_name = self.__class__.__name__
      self.db_task_ini = self.config.get(class_name, "db-task_ini")

      self.config_db_dir = self.config.get(class_name, "config_db_dir")
      self.articles_tbl = self.config.get("sqlite", "articles_tbl")
      self.sqliteTimeout = self.config.getint("sqlite", "timeout")

      self.useCurrentYear = self.config.getint("DateTimeType", "useCurrentYear")

      if self.config.has_section(OPTION_SECTION_DATETIME_NEWS_NAMES):
        self.datetimeNewsNames = []
        for key, value in self.config.items(OPTION_SECTION_DATETIME_NEWS_NAMES):
          self.datetimeNewsNames.append(key)
      else:
        self.logger.debug("Config file hasn't section: " + str(OPTION_SECTION_DATETIME_NEWS_NAMES))
        self.datetimeNewsNames = TAGS_DATETIME_NEWS_NAMES;

      if self.config.has_section(OPTION_SECTION_DATETIME_TEMPLATE_TYPES):
        self.datetimeTemplateTypes = []
        for key, value in self.config.items(OPTION_SECTION_DATETIME_TEMPLATE_TYPES):
          self.datetimeTemplateTypes.append(key)
      else:
        self.logger.debug("Config file hasn't section: " + str(OPTION_SECTION_DATETIME_TEMPLATE_TYPES))
        self.datetimeTemplateTypes = TAGS_DATETIME_TEMPLATE_TYPES

    except:
      print MSG_ERROR_LOAD_OPTIONS
      raise


  ## loadMetrics
  #
  '''
  def loadMetrics(self):
    if "metrics" in self.properties and len(self.properties["metrics"]):
      self.metrics_list = self.properties["metrics"]
      self.logger.debug("self.metrics_list: %s", str(self.metrics_list))
  '''


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, query):
    self.logger.debug("Connector: " + str(self.DBConnector))
    try:
      with closing(self.DBConnector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        self.DBConnector.commit()  # pylint: disable=E1101
        return cursor.fetchall()
    except mdb.Error as err:  # pylint: disable=E1101
      self.DBConnector.rollback()  # pylint: disable=E1101
      ExceptionLog.handler(self.logger, err, 'executeQuery:', (err))
      raise err


  ## loadDBBackend
  #
  #
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      self.dbHost = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_HOST)
      self.dbPort = int(self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PORT))
      self.dbUser = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_USER)
      self.dbPWD = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PWD)
      self.db_dc_scraper = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_SCRAPERS)
      self.db_dc_scraper_table = self.config.get(className, CONSTS.DB_SCRAPERS)
      self.dc_contents_db = self.config.get(className, CONSTS.DC_CONTENTS_DB_NAME)
      self.db_dc_scraper_db_connect = mdb.connect(self.dbHost, self.dbUser, self.dbPWD, self.db_dc_scraper, self.dbPort)
    except Exception, err:
      ExceptionLog.handler(self.logger, err, CONSTS.MSG_ERROR_LOAD_DB_BACKEND)


  ## loadPropertiesFromDB
  #
  def loadPropertiesFromDB(self):
    self.logger.debug("Start load Scraper's properties from DB")
    self.DBConnector = self.db_dc_scraper_db_connect
    self.logger.debug("self.DBConnector: %s" % str(self.DBConnector))
    try:
      query = "SELECT `Value` FROM `%s` WHERE `Name`='%s'" % (self.db_dc_scraper_table, self.__class__.__name__)
      self.logger.debug("query: %s" % query)
      res = self.executeQuery(query)
      if len(res) > 0:
        self.properties = json.loads(res[0]["Value"])[CONSTS.PROPERTIES_KEY]
      else:
        self.logger.debug("properties wasn't found in %s table" % self.db_dc_scraper)
    except Exception, err:
      ExceptionLog.handler(self.logger, err, "Error load Scraper's properties from DB")
    self.logger.debug("Properties:\n%s", varDump(self.properties))
    self.logger.debug("End load Scraper's properties from DB")


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def createModule(self, module_name):
    appInst = None
    try:
      appInst = (module_name, eval(module_name)(self.config, None, self.urlHost, self.properties))[1]  # pylint: disable=W0123
      self.logger.debug("%s has been created!" % module_name)
    except Exception as err:
      ExceptionLog.handler(self.logger, err, "Can't create module %s. Error is:" % (module_name))

    return appInst


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def getExtractorByName(self, extractorName):
    for extractor in self.extractors:
      if extractor.__class__.__name__ == extractorName:
        return extractor


  # #
  #
  #
  def getExitCode(self):
    return self.exitCode

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # FeedParser section
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # #main content processing
  # main content processing
  #
  def feedParserProcess(self):
    self.logger.debug("URL: %s" % str(self.input_data.url))
    self.logger.debug("URLMd5: %s" % str(self.input_data.urlId))
    self.logger.debug("SiteId: %s" % str(self.input_data.siteId))
    if self.parseFeed():
      self.tagsCount = self.article.tagsCount
      self.tagsMask = self.article.tagsMask
      self.processedContent = self.article.get()
      # correct pubdate
      if CONSTS.PUBLISHED in self.article.tags:
        self.pubdate = parse(self.article.tags[CONSTS.PUBLISHED]["data"]).strftime(CONSTS.COMMON_DATE_FORMAT)
      else:
        self.logger.debug("Resource %s hasn't publish date" % str(self.article.tags[CONSTS.TAG_LINK]["data"]))
    else:
      self.logger.debug("Resource hasn't raw content. Exit.")


  # #main content processing
  # main content processing
  #
  def createArticle(self):
    resid = self.entry["urlMd5"]
    self.article = Result(self.config, resid, self.metrics)

    for tag in self.entry["entry"]:
      data = {"extractor":"feedParser extractor", "data":"", "name":""}
      data["data"] = self.entry["entry"][tag]
      data["name"] = tag
      self.article.tags[tag] = data

    date_tags = ["published", "updated", "updated_parsed"]
    if len(set(self.entry["entry"].keys()).intersection(date_tags)) == 0:
      self.logger.debug("PUBDATE_ERROR: list of tags from rss feed: %s" % str(self.entry["entry"].keys()))

    if "pubdate" in self.entry and self.article.tags["pubdate"] == "":
      data = {"extractor":"feedParser extractor", "data":"", "name":""}
      data["data"] = self.entry["pubdate"]
      data["name"] = "pubdate"
      self.article.tags["pubdate"] = data

    #parent rss feed
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed"]
    data["name"] = "parent_rss_feed"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed"] = data

    #parent rss feed urlMd5
    data = {"extractor":"feedParser extractor", "data":"", "name":""}
    data["data"] = self.entry["parent_rss_feed_urlMd5"]
    data["name"] = "parent_rss_feed_urlMd5"
    data["xpath"] = ""
    data["extractor"] = self.__class__.__name__
    self.article.tags["parent_rss_feed_urlMd5"] = data

    #tags count
    self.article.tagsCount = len(self.article.tags.keys())


  # #main content processing
  # main content processing
  #
  def parseFeed(self):
    ret = True
    try:
      self.entry = json.loads(self.input_data.raw_content)
      self.createArticle()
      self.putArticleToDB({"default":self.article})  # pylint: disable=E1101
    except ValueError, err:
      ExceptionLog.handler(self.logger, err, 'Bad raw content:', (self.input_data.raw_content), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
      ret = False

    return ret

