"""@package docstring
 @file newspaper_extractor.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


from base_extractor import BaseExtractor
from base_extractor import signal_handler
from NewspaperWrapper import NewspaperWrapper
import signal
import Constants as CONSTS
import app.Consts as APP_CONSTS
import logging
from app.Utils import varDump
from app.Utils import ExceptionLog


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class NewspaperExtractor(BaseExtractor):


  # local class constants general purpose
  EXTRACTOR_NAME = "Newspaper extractor"
  SECTION_NAME = "extractor"


  def __init__(self, config, templ=None, domain=None, processorProperties=None):
    BaseExtractor.__init__(self, config, templ, domain, processorProperties)
    logger.debug("Properties: %s", varDump(self.properties))
    if processorProperties is not None and "SCRAPER_TAG_ITEMS_DELIMITER" in processorProperties:
      self.imgDelimiter = processorProperties["SCRAPER_TAG_ITEMS_DELIMITER"]
    else:
      self.imgDelimiter = ','

    # set module rank from module's properties
    self.rankReading(self.__class__.__name__)

    self.name = "Newspaper extractor"
    self.data["extractor"] = "Newspaper extractor"
    self.userAgent = processorProperties["USER_AGEN"] if "USER_AGEN" in processorProperties else None


  def extractTags(self, resource, reslt):
    # support time execution limit
    signal.signal(signal.SIGALRM, signal_handler)
    signal.alarm(CONSTS.TIME_EXECUTION_LIMIT)
    isLoadUrlsParam = False
    imageRation = None
    if self.processorProperties is not None and "SCRAPER_DOWNLOAD_IMAGES" in self.processorProperties:
      isLoadUrlsParam = bool(int(self.processorProperties["SCRAPER_DOWNLOAD_IMAGES"]))
    if self.processorProperties is not None and "IMAGE_RATION" in self.processorProperties:
      imageRation = float(self.processorProperties["IMAGE_RATION"])

    kArgs = {}
    if imageRation is not None:
      kArgs = {"title": u'', "source_url": u'', "config": None, "image_dimension_ration": imageRation}
    if self.userAgent is not None:
      kArgs["browser_user_agent"] = self.userAgent
      logger.debug(">>> NewspaperExtractor sets userAgent, is = " + str(self.userAgent))
    kArgs["isLoadUrls"] = isLoadUrlsParam
    article = NewspaperWrapper(" ", **kArgs)

    article.html = resource.raw_html
    article.is_downloaded = True
    try:
      article.parse()
      self.addTag(result=reslt, tag_name=CONSTS.TAG_TITLE, tag_value=article.title)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_LINK, tag_value=article.canonical_link)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_DESCRIPTION, tag_value=article.summary)
      if hasattr(article, "published_date"):
        self.addTag(result=reslt, tag_name=CONSTS.TAG_PUB_DATE, tag_value=str(article.published_date))
      elif hasattr(article, "publish_date"):
        self.addTag(result=reslt, tag_name=CONSTS.TAG_PUB_DATE, tag_value=str(article.publish_date))
      self.addTag(result=reslt, tag_name=CONSTS.TAG_AUTHOR, tag_value=article.authors)
      # self.addTag(result=reslt, tag_name=CONSTS.TAG_GUID, tag_value=article.link_hash)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_DC_DATE, tag_value=article.additional_data)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, tag_value=article.text)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_KEYWORDS, tag_value=article.meta_keywords)
      if article.top_img is not None:
        imgList = []
        imgList.append(article.top_img)
        imgList.extend([x for x in article.imgs if x != article.top_img])
      else:
        imgList = article.imgs
      if imgList is not None:
        localValue = self.imgDelimiter.join(imgList)
        imgList = []
        imgList.append(localValue)
        self.addTag(result=reslt, tag_name=CONSTS.TAG_MEDIA, tag_value=imgList)
    except Exception as err:
      ExceptionLog.handler(logger, err, 'Newspaper parse error:', (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
    return reslt
