"""@package docstring
 @file scraper_result.py
 @author Alexey, bgv <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import json
import time
import copy
import logging
import types
import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS
from dc_processor.base_extractor import BaseExtractor
from app.Metrics import Metrics
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class Result(object):


  def __init__(self, config, resId, metrics=None):
    self.name = "Scraper result object"
    if metrics is None:
      self.metrics = {}
    else:
      self.metrics = metrics
    self.data = {}
    self.tags = {}
    self.defaultTags = []
    self.resId = resId
    if config is None:
      self.article_tags = []
    else:
      self.article_tags = [tag for tag in config.options("article_tags")]
    for tag in self.article_tags:
      self.tags[tag] = ""
    self.name = "Scraper result object"
    self.start = time.time()
    self.finish = self.start
    self.errorCode = CONSTS.ERROR_OK
    self.errorMessage = CONSTS.MSG_ERROR_OK
    self.tagsCount = 0
    self.tagsMask = 0


  def getEmptyTags(self):
    empty_tags = [key for key, value in self.tags.items() if key in self.article_tags and not value]
    return empty_tags


  def getFilledTags(self):
    filled_tags = [key for key, value in self.tags.items() if key in self.article_tags and value]
    return filled_tags


  def recalcTagMaskCount(self, container=None, altTagsMask=None):
    self.tagsCount = 0
    self.tagsMask = 0

    for key, value in self.tags.items():
      if value is not None and value != "":
        tag = {}
        for key in value:
          tag[key] = value[key]

        #set tag's mask only if tag is registered, also increment tags count.
        #Execute it functionaly if tag's value not default
        if tag["name"] not in self.defaultTags and "data" in value:
          realValueString = ""
          if type(value["data"]) in types.StringTypes:
            realValueString = value["data"]
          elif type(value["data"]) is types.ListType and len(value["data"]) > 0:
            realValueString = value["data"][0]
          if realValueString.strip() != "":
            if altTagsMask is not None:
              if tag["name"] in altTagsMask:
                self.tagsMask = self.tagsMask | altTagsMask[tag["name"]]
            elif tag["name"] in BaseExtractor.tagsMask:
              self.tagsMask = self.tagsMask | BaseExtractor.tagsMask[tag["name"]]
            self.tagsCount += 1

        if container is not None:
          container.append(copy.copy(tag))


  def metricsPrecalculate(self):
    if len(self.metrics) > 0:
      Metrics.fillMetricModulesList()
      Metrics.metricsPrecalculate(self.metrics, self)


  def get(self):
    data = {}
    data["resId"] = self.resId
    data["tagList"] = []

    # Convert old format to new collection format
    data["tagList"].append([])

    self.recalcTagMaskCount(data["tagList"][0])

    self.data["data"] = data
    self.data["error_code"] = self.errorCode
    self.data["error_message"] = self.errorMessage
    self.data["time"] = "%s" % (self.finish - self.start)

    self.metrics = json.dumps(self.metrics)
    self.data["metrics"] = self.metrics
    return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ":"))



  def mergeResults(self, results_dict):
    logger.debug("results_dict: %s", varDump(results_dict))
    tmp = {}
    for item_name, item_value in self.tags.items():
      logger.debug("item_name: %s, item_value: %s", item_name, item_value)
      tmp[item_name] = []
      for key, value in results_dict.items():
        logger.debug("key: %s, value: %s", key, value)
        tmp[item_name].append(value.tags[item_name])

      logger.debug("BEFORE: %s", varDump(tmp[item_name]))
      tmp[item_name] = self.getBestValue(tmp[item_name])
      logger.debug("AFTER: %s", varDump(tmp[item_name]))
    logger.debug("SUMMARY: %s", varDump(tmp))
    self.tags = tmp


  def getBestValue(self, items_list):
    tmp = [item for item in items_list if item != ""]
    response = ""
    # if more than one suggestions try to select best one
    if len(tmp) > 1:
      # for each tag own rule
      # for content_encoded select biggest text
      if tmp[0]["name"] == "content_encoded":
        response = max(tmp, key=lambda x: x["data"])
      # for any else apply the same rule
      else:
        response = max(tmp, key=lambda x: x["data"])
    # if only one suggestion return it
    elif len(tmp) > 0:
      response = tmp[0]
    # if no one suggestions return empty string
    else:
      response = ""
    return response


  def stripResult(self):
    removeKeys = []
    for key in self.tags:
      if type(self.tags[key]) in types.StringTypes:
        self.tags[key] = self.tags[key].strip()
        if self.tags[key] == "":
          removeKeys.append(key)
      elif type(self.tags[key]) is types.DictType and "data" in self.tags[key]:
        if type(self.tags[key]["data"]) in types.StringTypes:
          self.tags[key]["data"] = self.tags[key]["data"].strip()
          if self.tags[key]["data"] == "":
            removeKeys.append(key)
        elif type(self.tags[key]["data"]) is types.ListType and len(self.tags[key]["data"]) > 0 and \
        type(self.tags[key]["data"][0]) in types.StringTypes:
          self.tags[key]["data"][0] = self.tags[key]["data"][0].strip()
          if self.tags[key]["data"][0] == "":
            removeKeys.append(key)
        else:
          removeKeys.append(key)

    for key in removeKeys:
      if key in self.tags:
        logger.debug(">>> Remove " + key + " element besause it empty")
        del self.tags[key]


  def isTagFilled(self, tagsName):
    ret = False
    if tagsName in self.tags:
      if type(self.tags[tagsName]) in types.StringTypes:
        if self.tags[tagsName].strip() != "":
          ret = True
      elif type(self.tags[tagsName]) is types.DictType and "data" in self.tags[tagsName]:
        if type(self.tags[tagsName]["data"]) in types.StringTypes:
          if self.tags[tagsName]["data"].strip() != "":
            ret = True
        elif type(self.tags[tagsName]["data"]) is types.ListType:
          for elem in self.tags[tagsName]["data"]:
            if type(elem) in types.StringTypes and elem != "":
              ret = True
              break
    return ret
