'''
Created on Nov 23, 2015

@author: scorp
'''
import ppath
import unittest
from dc_crawler.UrlSchema import UrlSchema


class Test(unittest.TestCase):

  def __init__(self, methodName='runTest'):
    unittest.TestCase.__init__(self, methodName)
    self.testUrls = {}
    self.testSchemas = {}
    self.testUrls["url1"] = "http://domen1.com/file?%param1%&from%param2%and%param3%_end"
    self.testUrls["schema1"] = ("{\"type\": 1,  \"mode\": 0, \"parameters\" : {\"param1\" : [\"z1\", \"z2\"], " +
                               "\"param2\" : [\"ffA1\", \"ffA2\", \"ffA\"]}, \"max_items\": 33}")
    self.testUrls["result1"] = ["http://domen1.com/file?z1&fromffA1and%param3%_end"]
    self.testUrls["url2"] = "http://domen1.com/file?%param1%&from%param2%and%param3%_end"
    self.testUrls["schema2"] = ("{\"type\": 2,  \"mode\": 0, \"parameters\" : {\"param1\" : {\"min\": 10, " +
                                "\"max\": 22, \"step\": 3}, \"param2\" : {\"min\": -22, " +
                                "\"max\": 0, \"step\": 1}}, \"max_items\": 33}")
    self.testUrls["result2"] = ["http://domen1.com/file?10&from-22and%param3%_end"]
    self.testUrls["url3"] = "http://domen1.com/file?%param1%&from%param2%and%param3%_end"
    self.testUrls["schema3"] = ("{\"type\": 3,  \"mode\": 0, \"parameters\" : {\"param1\" : {\"min\": 10, " +
                                "\"max\": 22, \"step\": 3}, \"param2\" : {\"min\": -22, " +
                                "\"max\": 0, \"step\": 1}}, \"max_items\": 33}")
    self.testUrls["url4"] = "http://domen1.com/file?%param1%&from%param2%and%param3%_end"
    self.testUrls["schema4"] = ("{\"type\": 3,  \"mode\": 0, \"parameters\" : {\"param1\" : {\"min\": 10, " +
                                "\"max\": 22, \"chars\": 0, \"case\": 0}, \"param2\" : {\"min\": -22, " +
                                "\"max\": 0, \"chars\": 1, \"case\": 1}}, \"max_items\": 33}")

    self.testUrls["url5"] = "http://domen1.com/file?%param1%&from%param2%and%param3%_end"
    self.testUrls["schema5"] = ("{\"type\": 1,  \"mode\": 1, \"parameters\" : {\"param1\" : [\"z1\", \"z2\"], " +
                               "\"param2\" : [\"ffA1\", \"ffA2\", \"ffA\"]}, \"max_items\": 3}")
    self.testUrls["result5"] = ["http://domen1.com/file?z1&fromffA1and%param3%_end"]


  def test_1_Predefined_One(self):
    schema = UrlSchema(self.testUrls["schema1"])
    self.assertTrue(schema.generateUrlSchema(self.testUrls["url1"]) == self.testUrls["result1"])


  def test_2_IncrementInt_One(self):
    schema = UrlSchema(self.testUrls["schema2"])
    self.assertTrue(schema.generateUrlSchema(self.testUrls["url2"]) == self.testUrls["result2"])


  def test_3_RandomInt_One(self):
    schema = UrlSchema(self.testUrls["schema3"])
    retList = schema.generateUrlSchema(self.testUrls["url3"])
    self.assertTrue(len(retList) == 1)
    self.assertTrue(retList[0].find("param1") == -1)
    self.assertTrue(retList[0].find("param2") == -1)


  def test_4_RandomChar_One(self):
    schema = UrlSchema(self.testUrls["schema4"])
    retList = schema.generateUrlSchema(self.testUrls["url4"])
    self.assertTrue(len(retList) == 1)
    self.assertTrue(retList[0].find("param1") == -1)
    self.assertTrue(retList[0].find("param2") == -1)


  def test_5_Predefined_List(self):
    schema = UrlSchema(self.testUrls["schema5"])
    retList = schema.generateUrlSchema(self.testUrls["url5"])
    self.assertTrue(len(retList) == 3)
    self.assertTrue(retList[0] == self.testUrls["result5"][0])


if __name__ == "__main__":
  #import sys;sys.argv = ['', 'Test.testName']
  unittest.main()
