/**
 * @file HandlerProperties.hpp
 *
 * Implementation class HandlerProperties of HCE cluster node.
 *
 * @author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef HANDLER_PROPERTIES_HPP
#define HANDLER_PROPERTIES_HPP

#include <Poco/Dynamic/Var.h>
#include <Poco/Types.h>

#include "JsonSerializable.hpp"
#include "Dictionary.hpp"

namespace HCE
{
namespace handlers
{
//-----------------------------------------------------------------------------
class HandlerProperties : public HCE::handlers::Dictionary<std::string, Poco::Dynamic::Var>, public IJsonSerializable
{
public:
  enum ErrorCodes
  {
    NO_ERROR=0,
    ERROR_SERIALIZE=1,
    ERROR_UNSERIALIZE=2,
    ERROR_LOAD_JSON_FILE=3
  };
public:
  HandlerProperties(void);
  ~HandlerProperties(void) {}

  bool serialize(std::string& json);
  bool unserialize(const std::string& json);

  bool load(const std::string& fileName);
  bool save(const std::string& fileName);

template <typename _Property> void set(const std::string& name, _Property value);
template <typename _Property> _Property get(const std::string& name, _Property defaultValue=_Property());

private:
  typedef HCE::handlers::Dictionary<std::string, Poco::Dynamic::Var> inherited;

  friend std::istream& operator>>(std::istream& is, HandlerProperties& handlerProperties);
  friend std::ostream& operator<<(std::ostream& os, const HandlerProperties& handlerProperties);
};
//-----------------------------------------------------------------------------
template <typename _Property> void HandlerProperties::set(const std::string& name, _Property value)
{
  setItem(name, value);
}
//-----------------------------------------------------------------------------
template <typename _Property> _Property HandlerProperties::get(const std::string& name, _Property defaultValue)
{
  Poco::Dynamic::Var var = getItem(name);
  _Property value = defaultValue;
  try
  {
    value = var.convert<_Property>();
  }
  catch(...)
  {
    value = defaultValue;
  }
  return value;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace handlers
} // end namespace HCE 

#endif // HANDLER_PROPERTIES_HPP
